@@ -9,7 +9,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
+   version 2 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -92,6 +92,14 @@ extern "C" {
  */
 + (NSCharacterSet*) lowercaseLetterCharacterSet;
 
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+/**
+ * Returns a character set containing the newline characters, values 
+ * 0x000A and 0x000D and nextline 0x0085 character.
+ */
++ (NSCharacterSet*) newlineCharacterSet;
+#endif
+
 /**
  *  Returns a character set containing characters for diacritical marks, which
  *  are usually only rendered in conjunction with another character.
