@@ -41,7 +41,7 @@
  The mode defaults to 'gnustep' but may be set using the GSPathHandling()
  function.<br />
  You should probably stick to using the default 'gnustep' mode in which the
- path handling methods cope with both unix and windows style paths in
+ path handling methods cope with both 'unix' and 'windows' style paths in
  portable and tolerant manner:<br />
  Paths are read in literally so they can be in the native format provided
  by the operating system or in a non-native format.<br />
@@ -49,6 +49,10 @@
  is running on (eg on windows slashes are converted to backslashes).<br />
  The path handling methods accept either a forward or backward slash as a
  path separator when parsing any path.<br />
+ Unless operating in 'unix' mode, a leading letter followed by a colon is
+ considered the start of a windows style path (the drive specifier), and a
+ path beginning with something of the form '//host/share/' is considered
+ the start of a UNC style path.<br />
  The path handling methods add forward slashes when building new paths
  internally or when standardising paths, so those path strings provide
  a portable representation (as long as they are relative paths, not including
@@ -252,7 +256,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 + (id) string;
 + (id) stringWithCharacters: (const unichar*)chars
 		     length: (NSUInteger)length;
-#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)
+#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
 + (id) stringWithCString: (const char*)byteString
 		encoding: (NSStringEncoding)encoding;
 #endif
@@ -264,7 +268,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 
 // Initializing Newly Allocated Strings
 - (id) init;
-#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)
+#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
 - (id) initWithBytes: (const void*)bytes
 	      length: (NSUInteger)length
 	    encoding: (NSStringEncoding)encoding;
@@ -273,13 +277,49 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 		  encoding: (NSStringEncoding)encoding 
 	      freeWhenDone: (BOOL)flag;
 #endif
-#if OS_API_VERSION(100500,GS_API_LATEST)
+#if OS_API_VERSION(100400,GS_API_LATEST)
 + (id) stringWithContentsOfFile: (NSString*)path
                    usedEncoding: (NSStringEncoding*)enc
                           error: (NSError**)error;
 - (id) initWithContentsOfFile: (NSString*)path
                  usedEncoding: (NSStringEncoding*)enc
                         error: (NSError**)error;
++ (id) stringWithContentsOfFile: (NSString*)path
+                       encoding: (NSStringEncoding)enc
+                          error: (NSError**)error;
+- (id) initWithContentsOfFile: (NSString*)path
+                     encoding: (NSStringEncoding)enc
+                        error: (NSError**)error;
++ (id) stringWithContentsOfURL: (NSURL*)url
+                  usedEncoding: (NSStringEncoding*)enc
+                         error: (NSError**)error;
+- (id) initWithContentsOfURL: (NSURL*)url
+                usedEncoding: (NSStringEncoding*)enc
+                       error: (NSError**)error;
++ (id) stringWithContentsOfURL: (NSURL*)url
+                      encoding: (NSStringEncoding)enc
+                         error: (NSError**)error;
+- (id) initWithContentsOfURL: (NSURL*)url
+                    encoding: (NSStringEncoding)enc
+                       error: (NSError**)error;
+- (BOOL) writeToFile: (NSString*)path
+	  atomically: (BOOL)atomically
+	    encoding: (NSStringEncoding)enc
+	       error: (NSError**)error;
+- (BOOL) writeToURL: (NSURL*)url
+	 atomically: (BOOL)atomically
+	   encoding: (NSStringEncoding)enc
+	      error: (NSError**)error;
+#endif
+#if OS_API_VERSION(100500,GS_API_LATEST)
+- (NSString*)stringByReplacingOccurrencesOfString: (NSString*)replace
+                                       withString: (NSString*)by
+                                          options: (NSStringCompareOptions)opts
+                                            range: (NSRange)searchRange;
+- (NSString*)stringByReplacingOccurrencesOfString: (NSString*)replace
+                                       withString: (NSString*)by;
+- (NSString*) stringByReplacingCharactersInRange: (NSRange)aRange 
+                                      withString: (NSString*)by;
 #endif
 - (id) initWithCharactersNoCopy: (unichar*)chars
 			 length: (NSUInteger)length
@@ -367,7 +407,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (const char*) cString;
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION(010200,GS_API_LATEST)
+#if OS_API_VERSION(100400,GS_API_LATEST) && GS_API_VERSION( 10200,GS_API_LATEST)
 - (const char*) cStringUsingEncoding: (NSStringEncoding)encoding;
 - (BOOL) getCString: (char*)buffer
 	  maxLength: (NSUInteger)maxLength
@@ -586,8 +626,10 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (NSString*) stringByExpandingTildeInPath;
 
 /**
+ * First calls -stringByExpandingTildeInPath if necessary.<br />
  * Replaces path string by one in which path components representing symbolic
  * links have been replaced by their referents.<br />
+ * Removes a leading '/private' if the result is valid.<br />
  * If links cannot be resolved, returns an unmodified copy of the receiver.
  */
 - (NSString*) stringByResolvingSymlinksInPath;
@@ -679,7 +721,7 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 - (NSComparisonResult) localizedCaseInsensitiveCompare: (NSString *)string;
 - (BOOL) writeToFile: (NSString*)filename
 	  atomically: (BOOL)useAuxiliaryFile;
-- (BOOL) writeToURL: (NSURL*)anURL atomically: (BOOL)atomically;
+- (BOOL) writeToURL: (NSURL*)url atomically: (BOOL)atomically;
 - (double) doubleValue;
 + (NSStringEncoding*) availableStringEncodings;
 + (NSString*) localizedNameOfStringEncoding: (NSStringEncoding)encoding;
@@ -805,114 +847,13 @@ typedef NSUInteger NSStringEncodingConversionOptions;
 extern struct objc_class _NSConstantStringClassReference;
 #endif
 
-#if GS_API_VERSION(GS_API_NONE, 011700)
-
-@interface NSMutableString (GNUstep)
-- (NSString*) immutableProxy;
-@end
-
-/**
- * Provides some additional (non-standard) utility methods.
- */
-@interface NSString (GSCategories)
-/**
- * Alternate way to invoke <code>stringWithFormat</code> if you have or wish
- * to build an explicit <code>va_list</code> structure.
- */
-+ (id) stringWithFormat: (NSString*)format
-	      arguments: (va_list)argList;
-
-/**
- * Returns a string formed by removing the prefix string from the
- * receiver.  Raises an exception if the prefix is not present.
- */
-- (NSString*) stringByDeletingPrefix: (NSString*)prefix;
-
-/**
- * Returns a string formed by removing the suffix string from the
- * receiver.  Raises an exception if the suffix is not present.
- */
-- (NSString*) stringByDeletingSuffix: (NSString*)suffix;
-
-/**
- * Returns a string formed by removing leading white space from the
- * receiver.
- */
-- (NSString*) stringByTrimmingLeadSpaces;
-
-/**
- * Returns a string formed by removing trailing white space from the
- * receiver.
- */
-- (NSString*) stringByTrimmingTailSpaces;
-
-/**
- * Returns a string formed by removing both leading and trailing
- * white space from the receiver.
- */
-- (NSString*) stringByTrimmingSpaces;
-
-/**
- * Returns a string in which any (and all) occurrences of
- * replace in the receiver have been replaced with by.
- * Returns the receiver if replace
- * does not occur within the receiver.  NB. an empty string is
- * not considered to exist within the receiver.
- */
-- (NSString*) stringByReplacingString: (NSString*)replace
-			   withString: (NSString*)by;
-@end
-
-
-/**
- * GNUstep specific (non-standard) additions to the NSMutableString class.
- */
-@interface NSMutableString (GSCategories)
-
-/**
- * Removes the specified suffix from the string.  Raises an exception
- * if the suffix is not present.
- */
-- (void) deleteSuffix: (NSString*)suffix;
-
-/**
- * Removes the specified prefix from the string.  Raises an exception
- * if the prefix is not present.
- */
-- (void) deletePrefix: (NSString*)prefix;
-
-/**
- * Replaces all occurrences of the string replace with the string by
- * in the receiver.<br />
- * Has no effect if replace does not occur within the
- * receiver.  NB. an empty string is not considered to exist within
- * the receiver.<br />
- * Calls - replaceOccurrencesOfString:withString:options:range: passing
- * zero for the options and a range from 0 with the length of the receiver.
- */
-- (void) replaceString: (NSString*)replace
-	    withString: (NSString*)by;
-
-/**
- * Removes all leading white space from the receiver.
- */
-- (void) trimLeadSpaces;
-
-/**
- * Removes all trailing white space from the receiver.
- */
-- (void) trimTailSpaces;
-
-/**
- * Removes all leading or trailing white space from the receiver.
- */
-- (void) trimSpaces;
-@end
-
-#endif	/* GS_API_NONE */
-
 #if	defined(__cplusplus)
 }
 #endif
 
+#if     !NO_GNUSTEP && !defined(GNUSTEP_BASE_INTERNAL)
+#import <GNUstepBase/NSString+GNUstepBase.h>
+#import <GNUstepBase/NSMutableString+GNUstepBase.h>
+#endif
+
 #endif /* __NSString_h_GNUSTEP_BASE_INCLUDE */
