@@ -29,8 +29,6 @@
 #ifndef NeXT_Foundation_LIBRARY
 #import "Foundation/NSException.h"
 #import "Foundation/NSRange.h"
-#else
-#import <Foundation/Foundation.h>
 #endif
 
 #import "GNUstepBase/GSObjCRuntime.h"
