@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-02-20 16:21:43 +0000 (Sun, 20 Feb 2011) $ $Revision: 32247 $
 */
 
 #import "common.h"
@@ -54,7 +54,7 @@
 - (void) _becomeThreaded: (NSNotification*)n
 {
   [[NSNotificationCenter defaultCenter] removeObserver: self];
-  isa = [NSLock class];
+  object_setClass(self, [NSLock class]);
   if (locked == YES)
     {
       if ([self tryLock] == NO)
@@ -87,17 +87,14 @@
       DESTROY(self);
       self = [NSLock new];
     }
-  else
+  else if (self != nil)
     {
-      if (self != nil)
-	{
-	  [[NSNotificationCenter defaultCenter]
-	    addObserver: self
-	    selector: @selector(_becomeThreaded:)
-	    name: NSWillBecomeMultiThreadedNotification
-	    object: nil];
-	}
       locked = NO;
+      [[NSNotificationCenter defaultCenter]
+	addObserver: self
+	selector: @selector(_becomeThreaded:)
+	name: NSWillBecomeMultiThreadedNotification
+	object: nil];
     }
   return self;
 }
@@ -199,7 +196,7 @@
 - (void) _becomeThreaded: (NSNotification*)n
 {
   [[NSNotificationCenter defaultCenter] removeObserver: self];
-  isa = [NSRecursiveLock class];
+  object_setClass(self, [NSRecursiveLock class]);
   while (counter-- > 0)
     {
       if ([self tryLock] == NO)
