@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>GSObjCRuntime function and macro reference</title>
-   $Date: 2010-05-30 09:24:54 +0100 (Sun, 30 May 2010) $ $Revision: 30487 $
+   $Date: 2011-04-12 06:41:53 +0100 (Tue, 12 Apr 2011) $ $Revision: 32837 $
    */
 
 #import "common.h"
@@ -38,13 +38,12 @@
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSEnumerator.h"
 #import "Foundation/NSException.h"
+#import "Foundation/NSInvocation.h"
 #import "Foundation/NSLock.h"
 #import "Foundation/NSMethodSignature.h"
 #import "Foundation/NSNull.h"
 #import "Foundation/NSSet.h"
 #import "Foundation/NSValue.h"
-#else
-#import <Foundation/Foundation.h>
 #endif
 #import "GNUstepBase/GSObjCRuntime.h"
 
@@ -59,6 +58,9 @@
 #ifndef NeXT_RUNTIME
 #include <pthread.h>
 #endif
+#ifdef __GNUSTEP_RUNTIME__
+extern struct objc_slot	*objc_get_slot(Class, SEL);
+#endif
 
 #ifdef NeXT_Foundation_LIBRARY
 @interface NSObject (MissingFromMacOSX)
@@ -135,11 +137,21 @@ GSSelectorFromName(const char *name)
 {
   return sel_getUid(name);
 }
+
 SEL
 GSSelectorFromNameAndTypes(const char *name, const char *types)
 {
 #if NeXT_RUNTIME
   return sel_getUid(name);
+/* Don't use sel_registerTypedName() ... it's broken when first introduced
+ * into gcc (fails to correctly check for multple registrations with same
+ * types but different layout info).
+ *
+#elif defined (__GNU_LIBOBJC__)
+  return sel_registerTypedName(name, types);
+ */
+#elif defined (__GNUSTEP_RUNTIME__)
+  return sel_registerTypedName_np(name, types);
 #else
   if (name == 0)
     {
@@ -172,17 +184,28 @@ GSSelectorFromNameAndTypes(const char *name, const char *types)
     }
 #endif
 }
+
 const char *
 GSTypesFromSelector(SEL sel)
 {
 #if NeXT_RUNTIME
   return 0;
+#elif defined (__GNU_LIBOBJC__)
+  return sel_getTypeEncoding(sel);
+#elif defined (__GNUSTEP_RUNTIME__)
+  return sel_getType_np(sel);
 #else
   if (sel == 0)
-    return 0;
-  return sel_get_type(sel);
+    {
+      return 0;
+    }
+  else
+    {
+      return sel_get_type(sel);
+    }
 #endif
 }
+
 void
 GSFlushMethodCacheForClass (Class cls)
 {
@@ -409,6 +432,7 @@ GSObjCMakeClass(NSString *name, NSString *superName, NSDictionary *iVars)
   Class		classSuperClass;
   const char	*classNameCString;
   char		*tmp;
+  int		len;
 
   NSCAssert(name, @"no name");
   NSCAssert(superName, @"no superName");
@@ -419,8 +443,10 @@ GSObjCMakeClass(NSString *name, NSString *superName, NSDictionary *iVars)
   NSCAssert1(!NSClassFromString(name), @"A class %@ already exists", name);
 
   classNameCString = [name UTF8String];
-  tmp = malloc(strlen(classNameCString) + 1);
-  strcpy(tmp, classNameCString);
+  len = strlen(classNameCString);
+  tmp = malloc(len + 1);
+  strncpy(tmp, classNameCString, len);
+  tmp[len] = '\0';
   classNameCString = tmp;
 
   newClass = objc_allocateClassPair(classSuperClass, classNameCString, 0);
@@ -480,15 +506,15 @@ GSObjCAddClasses(NSArray *classes)
 static BOOL behavior_debug = NO;
 
 BOOL
-GSObjCBehaviorDebug(int i)
+GSObjCBehaviorDebug(int setget)
 {
   BOOL	old = behavior_debug;
 
-  if (i == YES)
+  if (setget == 1)
     {
       behavior_debug = YES;
     }
-  else if (i == NO)
+  else if (setget == 0)
     {
       behavior_debug = NO;
     }
@@ -777,6 +803,34 @@ GSProtocolFromName(const char *name)
   return p;
 }
 
+struct objc_method_description
+GSProtocolGetMethodDescriptionRecursive(Protocol *aProtocol, SEL aSel, BOOL isRequired, BOOL isInstance)
+{
+  struct objc_method_description desc;
+
+  desc = protocol_getMethodDescription(aProtocol, aSel, isRequired, isInstance);
+  if (desc.name == NULL && desc.types == NULL)
+    {
+      Protocol **list;
+      unsigned int count;
+      list = protocol_copyProtocolList(aProtocol, &count);
+      if (list != NULL)
+        {
+          unsigned int i;
+          for (i = 0; i < count; i++)
+            {
+              desc = GSProtocolGetMethodDescriptionRecursive(list[i], aSel, isRequired, isInstance);
+              if (desc.name != NULL || desc.types != NULL)
+                {
+                  return desc;
+                }
+            }
+          free(list);
+        }
+    }
+
+  return desc;
+}
 
 void
 GSObjCAddClassBehavior(Class receiver, Class behavior)
@@ -936,10 +990,11 @@ id
 GSObjCGetVal(NSObject *self, const char *key, SEL sel,
 	       const char *type, unsigned size, int offset)
 {
+  NSMethodSignature	*sig = nil;
+
   if (sel != 0)
     {
-      NSMethodSignature	*sig = [self methodSignatureForSelector: sel];
-
+      sig = [self methodSignatureForSelector: sel];
       if ([sig numberOfArguments] != 2)
 	{
 	  [NSException raise: NSInvalidArgumentException
@@ -1295,12 +1350,49 @@ GSObjCGetVal(NSObject *self, const char *key, SEL sel,
               }
             else
               {
-	        val = [self valueForUndefinedKey:
-		  [NSString stringWithUTF8String: key]];
+                if (sel == 0)
+                  {
+		    return [NSValue valueWithBytes: ((char *)self + offset)
+					  objCType: type];
+                  }
+                else
+                  {
+		    NSInvocation	*inv;
+		    size_t		retSize;
+
+		    inv = [NSInvocation invocationWithMethodSignature: sig];
+		    [inv setSelector: sel];
+		    [inv invokeWithTarget: self];
+		    retSize = [sig methodReturnLength];
+		    {
+		      char ret[retSize];
+
+		      [inv getReturnValue: ret];
+		      return [NSValue valueWithBytes: ret objCType: type];
+		    }
+                  }
               }
             break;
 
 	  default:
+#ifdef __GNUSTEP_RUNTIME__
+	    {
+	      Class		cls;
+	      struct objc_slot	*type_slot;
+	      SEL		typed;
+	      struct objc_slot	*slot;
+
+	      cls = [self class];
+	      type_slot = objc_get_slot(cls, @selector(retain));
+	      typed = GSSelectorFromNameAndTypes(sel_getName(sel),
+		type_slot->types);
+	      slot = objc_get_slot(cls, typed);
+	      if (strcmp(slot->types, type_slot->types) == 0)
+		{
+		  return slot->method(self, typed);
+		}
+	    }
+#endif
 	    val = [self valueForUndefinedKey:
 	      [NSString stringWithUTF8String: key]];
 	}
@@ -1332,7 +1424,8 @@ void
 GSObjCSetVal(NSObject *self, const char *key, id val, SEL sel,
   const char *type, unsigned size, int offset)
 {
-  static NSNull	*null = nil;
+  static NSNull		*null = nil;
+  NSMethodSignature	*sig = nil;
 
   if (null == nil)
     {
@@ -1340,8 +1433,7 @@ GSObjCSetVal(NSObject *self, const char *key, id val, SEL sel,
     }
   if (sel != 0)
     {
-      NSMethodSignature	*sig = [self methodSignatureForSelector: sel];
-
+      sig = [self methodSignatureForSelector: sel];
       if ([sig numberOfArguments] != 3)
 	{
 	  [NSException raise: NSInvalidArgumentException
@@ -1709,8 +1801,24 @@ GSObjCSetVal(NSObject *self, const char *key, id val, SEL sel,
               }
             else
               {
-                [self setValue: val forUndefinedKey:
-		  [NSString stringWithUTF8String: key]];
+		NSUInteger	size;
+
+		NSGetSizeAndAlignment(type, &size, 0);
+                if (sel == 0)
+                  {
+		    [val getValue: ((char *)self + offset)];
+		  }
+		else
+		  {
+		    NSInvocation	*inv;
+		    char		buf[size];
+
+		    [val getValue: buf];
+		    inv = [NSInvocation invocationWithMethodSignature: sig];
+		    [inv setSelector: sel];
+		    [inv setArgument: buf atIndex: 2];
+		    [inv invokeWithTarget: self];
+		  }
               }
             break;
 
@@ -1856,11 +1964,11 @@ GSPrintf (FILE *fptr, NSString* format, ...)
 {
   static Class                  stringClass = 0;
   static NSStringEncoding       enc;
-  CREATE_AUTORELEASE_POOL(arp);
-  va_list       ap;
-  NSString      *message;
-  NSData        *data;
-  BOOL          ok = NO;
+  va_list       		ap;
+  NSAutoreleasePool		*arp = [NSAutoreleasePool new];
+  NSString      		*message;
+  NSData        		*data;
+  BOOL          		ok = NO;
 
   if (stringClass == 0)
     {
@@ -1876,7 +1984,7 @@ GSPrintf (FILE *fptr, NSString* format, ...)
     {
       data = [message dataUsingEncoding: NSUTF8StringEncoding];
     }
-  RELEASE(message);
+  [message release];
 
   if (data != nil)
     {
@@ -1887,6 +1995,84 @@ GSPrintf (FILE *fptr, NSString* format, ...)
           ok = YES;
         }
     }
-  RELEASE(arp);
+  [arp release];
   return ok;
 }
+
+#if     defined(GNUSTEP_BASE_LIBRARY)
+
+# ifndef	NDEBUG
+#   define	AADD(c, o) GSDebugAllocationAdd(c, o)
+#   define	AREM(c, o) GSDebugAllocationRemove(c, o)
+# else
+#   define	AADD(c, o) 
+#   define	AREM(c, o) 
+# endif
+
+# if	GS_WITH_GC
+
+#include	<gc/gc.h>
+
+static BOOL
+GSIsFinalizable(Class c)
+{
+  static IMP	finalizeIMP = 0;
+
+  if (0 == finalizeIMP)
+    {
+      finalizeIMP = [NSObject instanceMethodForSelector: @selector(finalize)];
+    }
+  if (class_getMethodImplementation(c, @selector(finalize)) != finalizeIMP)
+    return YES;
+  return NO;
+}
+
+static void
+GSFinalize(void* object, void* data)
+{
+  [(id)object finalize];
+  AREM(object_getClass((id)object), (id)object);
+  object_setClass((id)object, (Class)(void*)0xdeadface);
+}
+
+# endif	/* GS_WITH_GC */
+#endif	/* defined(GNUSTEP_BASE_LIBRARY) */
+
+void
+GSClassSwizzle(id instance, Class newClass)
+{
+  Class	oldClass = object_getClass(instance);
+
+  if (oldClass != newClass)
+    {
+#if     defined(GNUSTEP_BASE_LIBRARY)
+# if	GS_WITH_GC
+      /* We only do allocation counting for objects that can be
+       * finalised - for other objects we have no way of decrementing
+       * the count when the object is collected.
+       */
+      if (GSIsFinalizable(oldClass))
+	{
+	  /* Already finalizable, so we just need to do any allocation
+	   * accounting.
+	   */
+          AREM(oldClass, instance);
+          AADD(newClass, instance);
+	}
+      else if (GSIsFinalizable(newClass))
+	{
+	  /* New class is finalizable, so we must register the instance
+	   * for finalisation and do allocation acounting for it.
+	   */
+	  AADD(newClass, instance);
+	  GC_REGISTER_FINALIZER (instance, GSFinalize, NULL, NULL, NULL);
+	}
+# else
+      AREM(oldClass, instance);
+      AADD(newClass, instance);
+# endif	/* GS_WITH_GC */
+#endif	/* defined(GNUSTEP_BASE_LIBRARY) */
+      object_setClass(instance, newClass);
+    }
+}
+
