@@ -90,34 +90,38 @@
 
 - (id) notImplemented: (SEL)aSel
 {
+  char	c = (class_isMetaClass(object_getClass(self)) ? '+' : '-');
+
   [NSException
-    raise: NSGenericException
-    format: @"method %@ not implemented in %@(%s)",
-    aSel ? (id)NSStringFromSelector(aSel) : (id)@"(null)",
-    NSStringFromClass([self class]),
-    GSObjCIsInstance(self) ? "instance" : "class"];
+    raise: NSInvalidArgumentException
+    format: @"[%@%c%@] not implemented",
+    NSStringFromClass([self class]), c,
+    aSel ? (id)NSStringFromSelector(aSel) : (id)@"(null)"];
   return nil;
 }
 
 - (id) shouldNotImplement: (SEL)aSel
 {
+  char	c = (class_isMetaClass(object_getClass(self)) ? '+' : '-');
+
   [NSException
-    raise: NSGenericException
-    format: @"%@(%s) should not implement %@",
-    NSStringFromClass([self class]),
-    GSObjCIsInstance(self) ? "instance" : "class",
+    raise: NSInvalidArgumentException
+    format: @"[%@%c%@] should not be implemented",
+    NSStringFromClass([self class]), c,
     aSel ? (id)NSStringFromSelector(aSel) : (id)@"(null)"];
   return nil;
 }
 
 - (id) subclassResponsibility: (SEL)aSel
 {
+  char	c = (class_isMetaClass(object_getClass(self)) ? '+' : '-');
+
   [NSException raise: NSInvalidArgumentException
-    format: @"subclass %@(%s) should override %@",
-    NSStringFromClass([self class]),
-    GSObjCIsInstance(self) ? "instance" : "class",
+    format: @"[%@%c%@] should be overridden by subclass",
+    NSStringFromClass([self class]), c,
     aSel ? (id)NSStringFromSelector(aSel) : (id)@"(null)"];
   return nil;
 }
 
 @end
+
