@@ -29,11 +29,8 @@
 */
 
 #import "common.h"
-#if	defined(NeXT_Foundation_LIBRARY)
-#import <Foundation/Foundation.h>
-#else
+#if	!defined(NeXT_Foundation_LIBRARY)
 #import "Foundation/NSArray.h"
-#import "Foundation/NSBundle.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSError.h"
 #import "Foundation/NSException.h"
@@ -307,7 +304,8 @@ static void GSSetupEncodingTable(void)
 		    }
 		}
 	    }
-	  encTable = NSZoneMalloc(NSDefaultMallocZone(),(encTableSize+1)*sizeof(struct _strenc_ *));
+	  encTable = NSZoneMalloc(NSDefaultMallocZone(),
+	    (encTableSize+1)*sizeof(struct _strenc_ *));
 	  memset(encTable, 0, (encTableSize+1)*sizeof(struct _strenc_ *));
 
 	  /*
@@ -326,14 +324,16 @@ static void GSSetupEncodingTable(void)
 	      if (entry->iconv != 0 && *(entry->iconv) != 0)
 		{
 		  iconv_t	c;
+		  int		l;
 		  char	*lossy;
 
 		  /*
 		   * See if we can do a lossy conversion.
 		   */
-		  lossy = NSZoneMalloc(NSDefaultMallocZone(),strlen(entry->iconv) + 12);
-		  strcpy(lossy, entry->iconv);
-		  strcat(lossy, "//TRANSLIT");
+		  l = strlen(entry->iconv);
+		  lossy = NSZoneMalloc(NSDefaultMallocZone(), l + 11);
+		  strncpy(lossy, entry->iconv, l);
+		  strncpy(lossy + l, "//TRANSLIT", 11);
 		  c = iconv_open(UNICODE_ENC, entry->iconv);
 		  if (c == (iconv_t)-1)
 		    {
@@ -475,8 +475,8 @@ GSEncodingForRegistry (NSString *registry, NSString *encoding)
  *  deduced from the clocale string itself. If  clocale isn't set or
  *  no match can be found, returns GSUndefinedEncoding.
  */
-/* It would be really nice if this could be used in +defaultCStringEncoding, but
- * there are too many dependancies on other parts of the library to
+/* It would be really nice if this could be used in +defaultCStringEncoding,
+ * but there are too many dependancies on other parts of the library to
  * make this practical (even if everything possible was written in C,
  * we'd still need some way to find the Locale.encodings file).
  */
@@ -1488,7 +1488,7 @@ done:
 	}
     }
 #if	!GS_WITH_GC
-  else if (ptr != buf && (dst == 0 || ptr != *dst))
+  else if (ptr != buf && dst != 0 && ptr != *dst)
     {
       NSZoneFree(zone, ptr);
     }
@@ -2274,7 +2274,6 @@ tables:
 		 * No mapping has been found.
 		 */
 		result = NO;
-		spos = slen;
 		goto done;
 	      }
 	  }
@@ -2485,7 +2484,7 @@ iconv_start:
 	}
     }
 #if	!GS_WITH_GC
-  else if (ptr != buf && (dst == 0 || ptr != *dst))
+  else if (ptr != buf && dst != 0 && ptr != *dst)
     {
       NSZoneFree(zone, ptr);
     }
@@ -2566,8 +2565,8 @@ GSPrivateDefaultCStringEncoding()
 	  /* Take it from the system locale information.  */
           [gnustep_global_lock lock];
           strncpy(encbuf, nl_langinfo(CODESET), sizeof(encbuf)-1);
-          [gnustep_global_lock unlock];
           encbuf[sizeof(encbuf)-1] = '\0';
+          [gnustep_global_lock unlock];
           encoding = encbuf;
 
 	  /*
