@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-02-28 19:49:57 +0000 (Mon, 28 Feb 2011) $ $Revision: 32417 $
    */
 
 #import "common.h"
@@ -635,13 +635,7 @@ static Class	GSInlineArrayClass;
 
 - (id) makeImmutableCopyOnFail: (BOOL)force
 {
-#ifndef NDEBUG
-  GSDebugAllocationRemove(isa, self);
-#endif
-  isa = [GSArray class];
-#ifndef NDEBUG
-  GSDebugAllocationAdd(isa, self);
-#endif
+  GSClassSwizzle(self, [GSArray class]);
   return self;
 }
 
@@ -729,7 +723,7 @@ static Class	GSInlineArrayClass;
       index++;
     }
   _contents_array[_count] = 0;
-  RELEASE(obj);	/* Adjust array BEFORE releasing object.	*/
+  [obj release];	/* Adjust array BEFORE releasing object.	*/
   _version++;
 }
 
@@ -794,9 +788,9 @@ static Class	GSInlineArrayClass;
    *	array in case a retain or release causes an exception.
    */
   obj = _contents_array[index];
-  IF_NO_GC(RETAIN(anObject));
+  [anObject retain];
   _contents_array[index] = anObject;
-  RELEASE(obj);
+  [obj release];
   _version++;
 }
 
