@@ -26,7 +26,6 @@
 #import "Foundation/NSCoder.h"
 #import "Foundation/NSDistantObject.h"
 #import "GSInvocation.h"
-#import <objc/objc-api.h>
 #import <avcall.h>
 #import <callback.h>
 #import "callframe.h"
@@ -493,7 +492,7 @@ static IMP gs_objc_msg_forward (SEL sel)
 	  // 2.b.2 NO, we do not have it.
 	  vacallReturnTypeInfo *ret_info;
 
-	  ret_info = objc_malloc (sizeof (vacallReturnTypeInfo));
+	  ret_info = malloc(sizeof (vacallReturnTypeInfo));
 	  *ret_info = returnInfo;
 	
 	  forwarding_callback
@@ -919,10 +918,10 @@ GSInvocationCallback (void *callback_data, va_alist args)
 	{
 	  const char	*runtimeName = sel_getName(selector);
 
-	  selector = sel_get_typed_uid (runtimeName, receiverTypes);
+	  runtimeTypes = GSTypesFromSelector(selector);
 	  if (selector == 0)
 	    {
-	      selector = sel_register_typed_name (runtimeName, receiverTypes);
+	      selector = GSSelectorFromNameAndTypes(runtimeName, receiverTypes);
 	    }
 	  if (runtimeTypes != 0)
 	    {
