@@ -39,6 +39,7 @@
 #import "Foundation/NSByteOrder.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSUserDefaults.h"
+#import "GSNetwork.h"
 #import "GSPrivate.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
 
@@ -243,14 +244,14 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 - (void) dealloc
 {
-  RELEASE(address);
-  RELEASE(service);
-  RELEASE(protocol);
+  DESTROY(address);
+  DESTROY(service);
+  DESTROY(protocol);
 
   [self finalize];
 
-  RELEASE(readInfo);
-  RELEASE(writeInfo);
+  DESTROY(readInfo);
+  DESTROY(writeInfo);
   [super dealloc];
 }
 
@@ -273,6 +274,7 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
   if (gzDescriptor != 0)
     {
       gzclose(gzDescriptor);
+      gzDescriptor = 0;
     }
 #endif
   if (descriptor != -1)
@@ -919,7 +921,9 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
       return nil;
     }
 
-  if (listen(net, 256) == -1)
+  /* We try to allow a large number of connections.
+   */
+  if (listen(net, GSBACKLOG) == -1)
     {
       NSLog(@"unable to listen on port - %@", [NSError _last]);
       (void) close(net);
@@ -1023,10 +1027,10 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     {
       self = [self initWithFileDescriptor: 2 closeOnDealloc: NO];
       fh_stderr = self;
-    }
-  if (self)
-    {
-      readOK = NO;
+      if (self)
+	{
+	  readOK = NO;
+	}
     }
   return self;
 }
@@ -1041,10 +1045,10 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     {
       self = [self initWithFileDescriptor: 0 closeOnDealloc: NO];
       fh_stdin = self;
-    }
-  if (self)
-    {
-      writeOK = NO;
+      if (self)
+	{
+	  writeOK = NO;
+	}
     }
   return self;
 }
@@ -1059,10 +1063,10 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     {
       self = [self initWithFileDescriptor: 1 closeOnDealloc: NO];
       fh_stdout = self;
-    }
-  if (self)
-    {
-      readOK = NO;
+      if (self)
+	{
+	  readOK = NO;
+	}
     }
   return self;
 }
@@ -1810,9 +1814,9 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
   if ([writeInfo count] > 0)
     {
-      NSMutableDictionary*	info = [writeInfo objectAtIndex: 0];
+      NSMutableDictionary	*info = [writeInfo objectAtIndex: 0];
 
-      modes=(NSArray*)[info objectForKey: NSFileHandleNotificationMonitorModes];
+      modes = [info objectForKey: NSFileHandleNotificationMonitorModes];
     }
 
   if (modes && [modes count])
@@ -2170,7 +2174,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 - (NSString*) socketLocalAddress
 {
   NSString	*str = nil;
-  struct sockaddr_in sin;
+  struct sockaddr_in sin = { 0 };
   unsigned	size = sizeof(sin);
 
   if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == -1)
@@ -2187,7 +2191,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 - (NSString*) socketLocalService
 {
   NSString	*str = nil;
-  struct sockaddr_in sin;
+  struct sockaddr_in sin = { 0 };
   unsigned	size = sizeof(sin);
 
   if (getsockname(descriptor, (struct sockaddr*)&sin, &size) == -1)
