@@ -610,10 +610,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  NSString	*enc;
 	  NSString	*len;
 	  int		status;
-	  float		ver;
 
 	  info = [document headerNamed: @"http"];
-	  ver = [[info value] floatValue];
 	  status = [[info objectForKey: NSHTTPPropertyStatusCodeKey] intValue];
 	  len = [[document headerNamed: @"content-length"] value];
 	  enc = [[document headerNamed: @"content-transfer-encoding"] value];
@@ -630,6 +628,10 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	    {
 	      complete = NO;	// Read chunked body data
 	    }
+	  else if (nil != len && [len intValue] == 0)
+	    {
+	      complete = YES;	// content-length explicitly zero
+	    }
 	  if (complete == NO && [d length] == 0)
 	    {
 	      complete = YES;	// Had EOF ... terminate
@@ -1016,6 +1018,9 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       [nc removeObserver: self name: nil object: sock];
       if ([sock sslConnect] == NO)
 	{
+	  if (debug)
+	    NSLog(@"%@ %p %s Failed to make ssl connect",
+	      NSStringFromSelector(_cmd), self, keepalive?"K":"");
 	  [self endLoadInBackground];
 	  [self backgroundLoadDidFailWithReason:
 	    @"Failed to make ssl connect"];
