@@ -24,7 +24,6 @@
 
 #import "common.h"
 #import "GSNetServices.h"
-#define	EXPOSE_NSNetServices_IVARS	1
 #import "Foundation/NSNetServices.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSEnumerator.h"
@@ -1163,7 +1162,8 @@ static void DNSSD_API
 	    struct sockaddr_in	ip4;
 	    
 	    // oogly
-	    sprintf(rdb, "%d.%d.%d.%d", rd[0], rd[1], rd[2], rd[3]);
+	    snprintf(rdb, sizeof(rdb),
+	      "%d.%d.%d.%d", rd[0], rd[1], rd[2], rd[3]);
 	    LOG(@"Found IPv4 <%s> on port %d", rdb, ntohs(service->port));
 	    
 	    length = sizeof (struct sockaddr_in);
@@ -1184,11 +1184,12 @@ static void DNSSD_API
 	    struct sockaddr_in6	ip6;
 	    
 	    // Even more oogly
-	    sprintf(rdb, "%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x",
-			 rd[0], rd[1], rd[2], rd[3],
-			 rd[4], rd[5], rd[6], rd[7],
-			 rd[8], rd[9], rd[10], rd[11],
-			 rd[12], rd[13], rd[14], rd[15]);
+	    snprintf(rdb, sizeof(rdb),
+	      "%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x:%x%x",
+	      rd[0], rd[1], rd[2], rd[3],
+	      rd[4], rd[5], rd[6], rd[7],
+	      rd[8], rd[9], rd[10], rd[11],
+	      rd[12], rd[13], rd[14], rd[15]);
 	    LOG(@"Found IPv6 <%s> on port %d", rdb, ntohs(service->port));
 	    
 	    length = sizeof (struct sockaddr_in6);
@@ -1524,8 +1525,6 @@ static void DNSSD_API
 		  break;
 		}
 	      
-	      strcat(key, "\0");
-	      
 	      if ([[values objectAtIndex: i] isKindOfClass: [NSString class]])
 		{
 		  char	value[256];
@@ -2296,14 +2295,13 @@ static void DNSSD_API
 - (void)      netService: (NSNetService *) sender
   didUpdateTXTRecordData: (NSData *) data
 {
-  id _delegate = [self delegate];
+  id delegate = [self delegate];
   INTERNALTRACE;
   
-  if ([_delegate respondsToSelector:
+  if ([delegate respondsToSelector:
     @selector(netService:didUpdateTXTRecordData:)])
     {
-      [_delegate   netService: sender
-       didUpdateTXTRecordData: data];
+      [delegate netService: sender didUpdateTXTRecordData: data];
     }
 }
 
