@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSAutoreleasePool class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-24 06:24:05 +0000 (Thu, 24 Mar 2011) $ $Revision: 32697 $
    */
 
 #import "common.h"
@@ -34,6 +34,132 @@
 #import "Foundation/NSException.h"
 #import "Foundation/NSThread.h"
 
+#if	GS_WITH_GC
+
+@implementation NSAutoreleasePool
+
+static NSAutoreleasePool	*pool = nil;
+
++ (void) initialize
+{
+  pool = NSAllocateObject(self, 0, NSDefaultMallocZone());
+  return;
+}
+
++ (id) allocWithZone: (NSZone*)zone
+{
+  return pool;
+}
+
++ (id) new
+{
+  return pool;
+}
+
+- (id) init
+{
+  return self;
+}
+
+- (unsigned) autoreleaseCount
+{
+  return 0;
+}
+
+- (unsigned) autoreleaseCountForObject: (id)anObject
+{
+  return 0;
+}
+
++ (unsigned) autoreleaseCountForObject: (id)anObject
+{
+  return 0;
+}
+
++ (id) currentPool
+{
+  return pool;
+}
+
++ (void) addObject: (id)anObj
+{
+  return;
+}
+
+- (void) addObject: (id)anObj
+{
+  return;
+}
+
+- (void) drain
+{
+  static NSGarbageCollector	*collector = nil;
+  static SEL			sel;
+  static IMP			imp;
+
+  if (collector == nil)
+    {
+      collector = [NSGarbageCollector defaultCollector];
+      sel = @selector(collectIfNeeded);
+      imp = [collector methodForSelector: sel];
+    }
+  (*imp)(collector, sel);
+}
+
+- (id) retain
+{
+  [NSException raise: NSGenericException
+	       format: @"Don't call `-retain' on a NSAutoreleasePool"];
+  return self;
+}
+
+- (oneway void) release
+{
+  return;
+}
+
+- (void) dealloc
+{
+  GSNOSUPERDEALLOC;
+  return;
+}
+
+- (void) emptyPool
+{
+  return;
+}
+
+- (id) autorelease
+{
+  [NSException raise: NSGenericException
+	       format: @"Don't call `-autorelease' on a NSAutoreleasePool"];
+  return self;
+}
+
++ (void) _endThread: (NSThread*)thread
+{
+  return;
+}
+
++ (void) enableRelease: (BOOL)enable
+{
+  return;
+}
+
++ (void) freeCache
+{
+  return;
+}
+
++ (void) setPoolCountThreshhold: (unsigned)c
+{
+  return;
+}
+
+@end
+
+#else
+
 /* When this is `NO', autoreleased objects are never actually recorded
    in an NSAutoreleasePool, and are not sent a `release' message.
    Thus memory for objects use grows, and grows, and... */
@@ -111,22 +237,15 @@ pop_pool_from_cache (struct autorelease_thread_vars *tv)
   return tv->pool_cache[--(tv->pool_cache_count)];
 }
 
-static SEL	releaseSel = 0;
-
 
 @implementation NSAutoreleasePool
 
-static IMP	allocImp;
-static IMP	initImp;
-
 + (void) initialize
 {
-  if (self == [NSAutoreleasePool class])
-    {
-      releaseSel = @selector(release);
-      allocImp = [self methodForSelector: @selector(allocWithZone:)];
-      initImp = [self instanceMethodForSelector: @selector(init)];
-    }
+  /* Do nothing here which might interact with ther classes since this
+   * is called by [NSObject+initialize] !
+   */
+  return;
 }
 
 + (id) allocWithZone: (NSZone*)zone
@@ -140,7 +259,18 @@ static IMP	initImp;
 
 + (id) new
 {
-  id arp = (*allocImp)(self, @selector(allocWithZone:), NSDefaultMallocZone());
+  static IMP	allocImp = 0;
+  static IMP	initImp = 0;
+  id		arp;
+
+  if (0 == allocImp)
+    {
+      allocImp
+	= [NSAutoreleasePool methodForSelector: @selector(allocWithZone:)];
+      initImp
+	= [NSAutoreleasePool instanceMethodForSelector: @selector(init)];
+    }
+  arp = (*allocImp)(self, @selector(allocWithZone:), NSDefaultMallocZone());
   return (*initImp)(arp, @selector(init));
 }
 
@@ -169,7 +299,10 @@ static IMP	initImp;
       _released = _released_head;
     }
 
-  /* Install ourselves as the current pool. */
+  /* Install ourselves as the current pool.
+   * The only other place where the parent/child linked list is modified
+   * should be in -dealloc
+   */
   {
     struct autorelease_thread_vars *tv = ARP_THREAD_VARS;
     _parent = tv->current_pool;
@@ -235,8 +368,8 @@ static IMP	initImp;
   pool = t->_autorelease_vars.current_pool;
   if (pool == nil && t->_active == NO)
     {
-      [self new];	// Don't leak while exiting thread.
-      pool = t->_autorelease_vars.current_pool;
+      // Don't leak while exiting thread.
+      pool = t->_autorelease_vars.current_pool = [self new];
     }
   if (pool != nil)
     {
@@ -306,21 +439,7 @@ static IMP	initImp;
 
 - (void) drain
 {
-#if	GS_WITH_GC
-  static NSGarbageCollector	*collector = nil;
-  static SEL			sel;
-  static IMP			imp;
-
-  if (collector == nil)
-    {
-      collector = [NSGarbageCollector defaultCollector];
-      sel = @selector(collectIfNeeded);
-      imp = [collector methodForSelector: sel];
-    }
-  (*imp)(collector, sel);
-#else
-  DESTROY(self);
-#endif
+  [self release];
 }
 
 - (id) retain
@@ -341,10 +460,11 @@ static IMP	initImp;
 
   [self emptyPool];
 
-  /*
-   * Remove self from the linked list of pools in use.
-   * We already know that we have deallocated our child (if any),
+  /* Remove self from the linked list of pools in use.
+   * We already know that we have deallocated any child (in -emptyPool),
    * but we may have a parent which needs to know we have gone.
+   * The only other place where the parent/child linked list is modified
+   * should be in -init
    */
   if (tv->current_pool == self)
     {
@@ -355,7 +475,6 @@ static IMP	initImp;
       _parent->_child = nil;
       _parent = nil;
     }
-  _child = nil;
 
   /* Don't deallocate ourself, just save us for later use. */
   push_pool_to_cache (tv, self);
@@ -429,16 +548,17 @@ static IMP	initImp;
 		{
                   /* If anObject was an instance, c is it's class.
                    * If anObject was a class, c is its metaclass.
-                   * Either way, get_imp() should get the appropriate pointer.
+                   * Either way, we should get the appropriate pointer.
                    * If anObject is a proxy to something,
                    * the +instanceMethodForSelector: and -methodForSelector:
-                   * methods may not exist, but get_imp() will return the
+                   * methods may not exist, but this will return the
                    * address of the forwarding method if necessary.
                    */
-		  imps[hash] = get_imp(c, @selector(release));
+		  imps[hash]
+		    = class_getMethodImplementation(c, @selector(release));
 		  classes[hash] = c;
 		}
-	      (imps[hash])(anObject, releaseSel);
+	      (imps[hash])(anObject, @selector(release));
 	    }
 	  _released_count -= released->count;
 	  released->count = 0;
@@ -515,4 +635,4 @@ static IMP	initImp;
 }
 
 @end
-
+#endif
