@@ -28,7 +28,7 @@
 
 
    <title>NSBundle class reference</title>
-   $Date: 2010-08-21 09:03:32 +0100 (Sat, 21 Aug 2010) $ $Revision: 31191 $
+   $Date: 2011-04-04 11:57:49 +0100 (Mon, 04 Apr 2011) $ $Revision: 32769 $
 */
 
 #import "common.h"
@@ -54,6 +54,11 @@
 
 #import "GSPrivate.h"
 
+/* Constants */
+NSString * const NSBundleDidLoadNotification = @"NSBundleDidLoadNotification";
+NSString * const NSShowNonLocalizedStrings = @"NSShowNonLocalizedStrings";
+NSString * const NSLoadedClasses = @"NSLoadedClasses";
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -154,78 +159,91 @@ static NSString	*library_combo =
 static NSString*
 AbsolutePathOfExecutable(NSString *path, BOOL atLaunch)
 {
-  NSFileManager	*mgr = manager();
-  NSDictionary	*env;
-  NSString	*pathlist;
-  NSString	*prefix;
-  id		patharr;
+  if (NO == [path isAbsolutePath])
+    {
+      NSFileManager	*mgr = manager();
+      NSDictionary	*env;
+      NSString		*pathlist;
+      NSString		*prefix;
+      id		patharr;
+      NSString		*result = nil;
 
-  path = [path stringByStandardizingPath];
-  if ([path isAbsolutePath])
-    {
-      return path;
-    }
+      env = [[NSProcessInfo processInfo] environment];
+      pathlist = [env objectForKey:@"PATH"];
 
-  env = [[NSProcessInfo processInfo] environment];
-  pathlist = [env objectForKey:@"PATH"];
-
-/* Windows 2000 and perhaps others have "Path" not "PATH" */
-  if (pathlist == nil)
-    {
-      pathlist = [env objectForKey:@"Path"];
-    }
-#if defined(__MINGW__)
-  patharr = [pathlist componentsSeparatedByString:@";"];
-#else
-  patharr = [pathlist componentsSeparatedByString:@":"];
-#endif
-  /* Add . if not already in path */
-  if ([patharr indexOfObject: @"."] == NSNotFound)
-    {
-      patharr = AUTORELEASE([patharr mutableCopy]);
-      [patharr addObject: @"."];
-    }
-  patharr = [patharr objectEnumerator];
-  while ((prefix = [patharr nextObject]))
-    {
-      if ([prefix isEqual:@"."])
+    /* Windows 2000 and perhaps others have "Path" not "PATH" */
+      if (pathlist == nil)
 	{
-	  if (atLaunch == YES)
+	  pathlist = [env objectForKey:@"Path"];
+	}
+#if defined(__MINGW__)
+      patharr = [pathlist componentsSeparatedByString:@";"];
+#else
+      patharr = [pathlist componentsSeparatedByString:@":"];
+#endif
+      /* Add . if not already in path */
+      if ([patharr indexOfObject: @"."] == NSNotFound)
+	{
+	  patharr = AUTORELEASE([patharr mutableCopy]);
+	  [patharr addObject: @"."];
+	}
+      patharr = [patharr objectEnumerator];
+      while (nil != (prefix = [patharr nextObject]))
+	{
+	  if ([prefix isEqual:@"."])
 	    {
-	      prefix = _launchDirectory;
+	      if (atLaunch == YES)
+		{
+		  prefix = _launchDirectory;
+		}
+	      else
+		{
+		  prefix = [mgr currentDirectoryPath];
+		}
 	    }
+	  prefix = [prefix stringByAppendingPathComponent: path];
+	  if ([mgr isExecutableFileAtPath: prefix])
+	    {
+	      result = [prefix stringByStandardizingPath];
+	      break;
+	    }
+#if defined(__WIN32__)
 	  else
 	    {
-	      prefix = [mgr currentDirectoryPath];
-	    }
-	}
-      prefix = [prefix stringByAppendingPathComponent: path];
-      if ([mgr isExecutableFileAtPath: prefix])
-	{
-	  return [prefix stringByStandardizingPath];
-	}
-#if defined(__WIN32__)
-      {
-	NSString	*ext = [path pathExtension];
+	      NSString	*ext = [path pathExtension];
 
-	/* Also add common executable extensions on windows */
-	if (ext == nil || [ext length] == 0)
-	  {
-	    NSString *wpath;
-	    wpath = [prefix stringByAppendingPathExtension: @"exe"];
-	    if ([mgr isExecutableFileAtPath: wpath])
-	      return [wpath stringByStandardizingPath];
-	    wpath = [prefix stringByAppendingPathExtension: @"com"];
-	    if ([mgr isExecutableFileAtPath: wpath])
-	      return [wpath stringByStandardizingPath];
-	    wpath = [prefix stringByAppendingPathExtension: @"cmd"];
-	    if ([mgr isExecutableFileAtPath: wpath])
-	      return [wpath stringByStandardizingPath];
-	  }
-	}
+	      /* Also add common executable extensions on windows */
+	      if (ext == nil || [ext length] == 0)
+		{
+		  NSString *wpath;
+
+		  wpath = [prefix stringByAppendingPathExtension: @"exe"];
+		  if ([mgr isExecutableFileAtPath: wpath])
+		    {
+		      result = [wpath stringByStandardizingPath];
+		      break;
+		    }
+		  wpath = [prefix stringByAppendingPathExtension: @"com"];
+		  if ([mgr isExecutableFileAtPath: wpath])
+		    {
+		      result = [wpath stringByStandardizingPath];
+		      break;
+		    }
+		  wpath = [prefix stringByAppendingPathExtension: @"cmd"];
+		  if ([mgr isExecutableFileAtPath: wpath])
+		    {
+		      result = [wpath stringByStandardizingPath];
+		      break;
+		    }
+		}
+	    }
 #endif
+	}
+      path = result;
     }
-  return nil;
+  path = [path stringByResolvingSymlinksInPath];
+  path = [path stringByStandardizingPath];
+  return path;
 }
 
 /*
@@ -242,7 +260,7 @@ GSPrivateExecutablePath()
       [load_lock lock];
       if (beenHere == NO)
 	{
-#ifdef PROCFS_EXE_LINK
+#if	defined(PROCFS_EXE_LINK)
 	  executablePath = [manager()
 	    pathContentOfSymbolicLinkAtPath:
               [NSString stringWithUTF8String: PROCFS_EXE_LINK]];
@@ -264,6 +282,9 @@ GSPrivateExecutablePath()
 	    {
 	      executablePath
 		= [[[NSProcessInfo processInfo] arguments] objectAtIndex: 0];
+	    }
+	  if (NO == [executablePath isAbsolutePath])
+	    {
 	      executablePath = AbsolutePathOfExecutable(executablePath, YES);
 	    }
 
@@ -512,21 +533,24 @@ _find_main_bundle_for_tool(NSString *toolName)
   NSString	**fmClasses;
   NSString	*bundlePath = nil;
   unsigned int	len;
+  const char *frameworkClassName;
 
   if (frameworkClass == Nil)
     {
       return;
     }
 
-  len = strlen (frameworkClass->name);
+  frameworkClassName = class_getName(frameworkClass);
+
+  len = strlen (frameworkClassName);
 
   if (len > 12 * sizeof(char)
-      && !strncmp ("NSFramework_", frameworkClass->name, 12))
+      && !strncmp ("NSFramework_", frameworkClassName, 12))
     {
       /* The name of the framework.  */
       NSString *name;
 
-      name = [NSString stringWithUTF8String: &frameworkClass->name[12]];
+      name = [NSString stringWithUTF8String: &frameworkClassName[12]];
       /* Important - gnustep-make mangles framework names to encode
        * them as ObjC class names.  Here we need to demangle them.  We
        * apply the reverse transformations in the reverse order.
@@ -778,6 +802,7 @@ _find_main_bundle_for_tool(NSString *toolName)
 static void
 _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 {
+  const char *className;
   NSCAssert(_loadingBundle, NSInternalInconsistencyException);
   NSCAssert(_loadingFrameworks, NSInternalInconsistencyException);
 
@@ -786,11 +811,12 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
     {
       return;
     }
+  className = class_getName(theClass);
 
   /* Don't store the internal NSFramework_xxx class into the list of
      bundle classes, but store the linked frameworks in _loadingFrameworks  */
-  if (strlen (theClass->name) > 12   &&  !strncmp ("NSFramework_",
-						   theClass->name, 12))
+  if (strlen (className) > 12   &&  !strncmp ("NSFramework_",
+						   className, 12))
     {
       if (_currentFrameworkName)
 	{
@@ -798,7 +824,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 
 	  frameworkName = [_currentFrameworkName cString];
 
-	  if (!strcmp(theClass->name, frameworkName))
+	  if (!strcmp(className, frameworkName))
 	    return;
 	}
 
@@ -819,9 +845,14 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 {
   if (self == [NSBundle class])
     {
-      NSDictionary *env;
-      NSString	   *str;
+      extern const char	*GSPathHandling(const char *);
+      const char	*mode;
+      NSDictionary	*env;
+      NSString		*str;
 
+      /* Ensure we do 'right' path handling while initializing core paths.
+       */
+      mode = GSPathHandling("right");
       _emptyTable = RETAIN([NSDictionary dictionary]);
 
       /* Initialise manager here so it's thread-safe.
@@ -868,20 +899,21 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
       handle = objc_open_main_module(stderr);
       printf("%08x\n", handle);
 #endif
-#if NeXT_RUNTIME
+#if NeXT_RUNTIME || defined(__GNUSTEP_RUNTIME__) || defined(__GNU_LIBOBJC__)
       {
 	int i, numClasses = 0, newNumClasses = objc_getClassList(NULL, 0);
 	Class *classes = NULL;
-	while (numClasses < newNumClasses) {
-	  numClasses = newNumClasses;
-	  classes = objc_realloc(classes, sizeof(Class) * numClasses);
-	  newNumClasses = objc_getClassList(classes, numClasses);
-	}
+	while (numClasses < newNumClasses)
+	  {
+	    numClasses = newNumClasses;
+	    classes = realloc(classes, sizeof(Class) * numClasses);
+	    newNumClasses = objc_getClassList(classes, numClasses);
+	  }
 	for (i = 0; i < numClasses; i++)
 	  {
 	    [self _addFrameworkFromClass: classes[i]];
 	  }
-	objc_free(classes);
+	free(classes);
       }
 #else
       {
@@ -890,10 +922,11 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
 	
 	while ((class = objc_next_class(&state)))
 	  {
-	    unsigned int len = strlen (class->name);
+	    const char *className = class_getName(class);
+	    unsigned int len = strlen (className);
 	    
 	    if (len > sizeof("NSFramework_")
-		&& !strncmp("NSFramework_", class->name, 12))
+		&& !strncmp("NSFramework_", className, 12))
 	      {
 		[self _addFrameworkFromClass: class];
 	      }
@@ -907,6 +940,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
       objc_close_main_module(handle);
       _loadingBundle = nil;
 #endif
+      GSPathHandling(mode);
     }
 }
 
@@ -1208,7 +1242,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
   if (bundle == nil)
     {
       /* Is it in the main bundle or a library? */
-      if (class_is_class(aClass))
+      if (!class_isMetaClass(aClass))
         {
 	  NSString	*lib;
 
@@ -1295,6 +1329,9 @@ IF_NO_GC(
    * so we can manipulate it without having to worry about
    * details like that throughout the code.
    */
+
+  /* 1. make path absolute.
+   */
   if ([path isAbsolutePath] == NO)
     {
       NSWarnMLog(@"NSBundle -initWithPath: requires absolute path names, "
@@ -1334,12 +1371,15 @@ IF_NO_GC(
 #endif
     }
 
-  /*
-   * Standardize the path so we can be sure that cache lookup is consistent.
+  /* 2. Expand any symbolic links.
+   */
+  path = [path stringByResolvingSymlinksInPath];
+
+  /* 3. Standardize so we can be sure that cache lookup is consistent.
    */
   path = [path stringByStandardizingPath];
 
-  /* Check if we were already initialized for this directory */
+  /* check if we were already initialized for this directory */
   [load_lock lock];
   if (_bundles)
     {
@@ -1670,7 +1710,8 @@ IF_NO_GC(
   NSEnumerator		*enumerate;
 
   array = [NSMutableArray arrayWithCapacity: 8];
-  languages = [NSUserDefaults userLanguages];
+  languages = [[NSUserDefaults standardUserDefaults]
+    stringArrayForKey: @"NSLanguages"];
 
   primary = [rootPath stringByAppendingPathComponent: @"Resources"];
   [array addObject: _bundle_resource_path(primary, subPath, nil)];
@@ -1889,9 +1930,9 @@ IF_NO_GC(
 		  inDirectory: (NSString*)subPath
 	      forLocalization: (NSString*)localizationName
 {
-  CREATE_AUTORELEASE_POOL(arp);
-  NSString	*result = nil;
-  NSArray	*array;
+  NSAutoreleasePool	*arp = [NSAutoreleasePool new];
+  NSString		*result = nil;
+  NSArray		*array;
 
   if ([ext length] == 0)
     {
@@ -1922,14 +1963,16 @@ IF_NO_GC(
 	    }
 	}
     }
-  IF_NO_GC([result retain]; DESTROY(arp);)
-  return AUTORELEASE(result);
+  [result retain];
+  [arp release];
+  return [result autorelease];
 }
 
 + (NSArray *) preferredLocalizationsFromArray: (NSArray *)localizationsArray
 {
   return [self preferredLocalizationsFromArray: localizationsArray
-	         forPreferences: [NSUserDefaults userLanguages]];
+    forPreferences: [[NSUserDefaults standardUserDefaults]
+    stringArrayForKey: @"NSLanguages"]];
 }
 
 + (NSArray *) preferredLocalizationsFromArray: (NSArray *)localizationsArray
@@ -1981,8 +2024,10 @@ IF_NO_GC(
 	  dict = [[NSDictionary alloc] initWithContentsOfFile: path];
 	}
     }
-  if (dict == nil)
-    dict = [self infoDictionary];
+  if (nil == [dict autorelease])
+    {
+      dict = [self infoDictionary];
+    }
   return dict;
 }
 
@@ -2095,11 +2140,11 @@ IF_NO_GC(
               if (tableContent != nil)
                 {
                   NSWarnMLog (@"Localisation file %@ not in portable encoding"
-                              @" so I'm using the default encoding for the current"
-                              @" system, which may not display messages correctly.\n"
-                              @"The file should be ASCII (using \\U escapes for unicode"
-                              @" characters) or Unicode (UTF16 or UTF8) with a leading "
-                              @"byte-order-marker.\n", tablePath);
+		    @" so I'm using the default encoding for the current"
+		    @" system, which may not display messages correctly.\n"
+		    @"The file should be ASCII (using \\U escapes for unicode"
+		    @" characters) or Unicode (UTF16 or UTF8) with a leading "
+		    @"byte-order-marker.\n", tablePath);
                 }
             }
           if (tableContent == nil)
@@ -2381,6 +2426,42 @@ IF_NO_GC(
 	    }
 	}
     }
+#elif defined(__APPLE__)
+  /* A .dylib is usually of the form 'libxxx.maj.min.sub.dylib',
+   * but GNUstep-make installs them with 'libxxx.dylib.maj.min.sub'.
+   * For maximum compatibility with support both forms here.
+   */
+  if ([[libraryName pathExtension] isEqual: @"dylib"])
+    {
+      NSString	*s = [libraryName stringByDeletingPathExtension];
+      NSArray	*a = [s componentsSeparatedByString: @"."];
+
+      if ([a count] > 1)
+	{
+	  libraryName = [a objectAtIndex: 0];
+	  if (interfaceVersion == nil && [a count] >= 3)
+	    {
+	      interfaceVersion = [NSString stringWithFormat: @"%@.%@",
+		[a objectAtIndex: 1], [a objectAtIndex: 2]];
+	    }
+	}
+    }
+  else
+    {
+      r = [libraryName rangeOfString: @".dylib."];
+      if (r.length > 0)
+	{
+	  NSString *s = [libraryName substringFromIndex: NSMaxRange(r)];
+	  NSArray  *a = [s componentsSeparatedByString: @"."];
+
+	  libraryName = [libraryName substringToIndex: r.location];
+	  if (interfaceVersion == nil && [a count] >= 2)
+	    {
+	      interfaceVersion = [NSString stringWithFormat: @"%@.%@",
+		[a objectAtIndex: 0], [a objectAtIndex: 1]];
+	    }
+	}
+    }
 #else
   /* A .so is usually of the form 'libxxx.so.maj.min.sub'
    * so we can extract the version info and use it.
