@@ -23,7 +23,7 @@
  * License along with this library; if not, write to the Free
 
    <title>NSCallBacks class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-02-19 19:42:42 +0000 (Sat, 19 Feb 2011) $ $Revision: 32234 $
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02111 USA. */
 
@@ -93,7 +93,7 @@ void
 _NS_owned_void_p_release(void *table, void *p)
 {
   if (p != 0)
-    objc_free(p);
+    free(p);
   return;
 }
 
