@@ -104,7 +104,7 @@ typedef GSIMapNode_t *GSIMapNode;
 #define	GSI_MAP_ENUMERATOR	NSMapEnumerator
 
 #if	GS_WITH_GC
-#include	<gc_typed.h>
+#include	<gc/gc_typed.h>
 static GC_descr	nodeSS = 0;
 static GC_descr	nodeSW = 0;
 static GC_descr	nodeWS = 0;
@@ -525,8 +525,10 @@ NSEnumerateMapTable(NSMapTable *table)
   else
     {
       NSMapEnumerator	v = {0, 0, 0};
+      NSEnumerator	*e = [[table keyEnumerator] retain];
 
-      v.node = (void*)[[table keyEnumerator] retain];
+      v.node = (void*)e;
+      GS_CONSUMED(e)
       v.bucket = (unsigned long)(uintptr_t)table;
       return v;
     }
@@ -1232,7 +1234,8 @@ const NSMapTableValueCallBacks NSOwnedPointerMapValueCallBacks =
 
 - (id) initWithCoder: (NSCoder*)aCoder
 {
-  return [self subclassResponsibility: _cmd];
+  [self subclassResponsibility: _cmd];
+  return nil;
 }
 
 - (id) initWithKeyPointerFunctions: (NSPointerFunctions*)keyFunctions
