@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDate class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-09 10:24:18 +0000 (Wed, 09 Mar 2011) $ $Revision: 32507 $
    */
 
 #import "common.h"
@@ -54,8 +54,6 @@ const NSTimeInterval NSTimeIntervalSince1970 = 978307200.0;
 
 
 
-extern NSTimeInterval	GSTimeNow();	// In NSCalendarDate.m
-
 static BOOL	debug = NO;
 static Class	abstractClass = nil;
 static Class	concreteClass = nil;
@@ -160,7 +158,7 @@ otherTime(NSDate* other)
  */
 + (NSTimeInterval) timeIntervalSinceReferenceDate
 {
-  return GSTimeNow();
+  return GSPrivateTimeNow();
 }
 
 /**
@@ -169,7 +167,7 @@ otherTime(NSDate* other)
 + (id) date
 {
   return AUTORELEASE([[self allocWithZone: NSDefaultMallocZone()]
-	initWithTimeIntervalSinceReferenceDate: GSTimeNow()]);
+	initWithTimeIntervalSinceReferenceDate: GSPrivateTimeNow()]);
 }
 
 /**
@@ -469,7 +467,8 @@ otherTime(NSDate* other)
 		      modYear += adjective;
 		      hadYear = YES;
 		    }
-		  else if ([tmp caseInsensitiveCompare: mname] == NSOrderedSame)
+		  else if (mname != nil
+		    && [tmp caseInsensitiveCompare: mname] == NSOrderedSame)
 		    {
 		      modMonth += adjective;
 		      hadMonth = YES;
@@ -674,7 +673,7 @@ otherTime(NSDate* other)
 	{
 	  BOOL	mustSkip = YES;
 
-	  while ((dtoIndex < [dto length]) && (mustSkip == YES))
+	  while ((dtoIndex < length) && (mustSkip == YES))
 	    {
 	      switch ([dto characterAtIndex: dtoIndex])
 		{
@@ -716,7 +715,7 @@ otherTime(NSDate* other)
 		    break;
 		}
 	    }
-	  if (dtoIndex >= [dto length])
+	  if (dtoIndex >= length)
 	    {
 	      if (debug)
 		{
@@ -983,7 +982,7 @@ otherTime(NSDate* other)
 + (id) distantFuture
 {
   if (_distantFuture == nil)
-    return [GSDateFuture allocWithZone: 0];
+    _distantFuture = [GSDateFuture allocWithZone: 0];
   return _distantFuture;
 }
 
@@ -994,7 +993,7 @@ otherTime(NSDate* other)
 + (id) distantPast
 {
   if (_distantPast == nil)
-    return [GSDatePast allocWithZone: 0];
+    _distantPast = [GSDatePast allocWithZone: 0];
   return _distantPast;
 }
 
@@ -1059,7 +1058,7 @@ otherTime(NSDate* other)
  */
 - (id) init
 {
-  return [self initWithTimeIntervalSinceReferenceDate: GSTimeNow()];
+  return [self initWithTimeIntervalSinceReferenceDate: GSPrivateTimeNow()];
 }
 
 /**
@@ -1110,7 +1109,7 @@ otherTime(NSDate* other)
 {
   // Get the current time, add the secs and init thyself
   return [self initWithTimeIntervalSinceReferenceDate:
-    GSTimeNow() + secsToBeAdded];
+    GSPrivateTimeNow() + secsToBeAdded];
 }
 
 /**
@@ -1142,7 +1141,7 @@ otherTime(NSDate* other)
 				   timeZone: (NSTimeZone*)timeZone
 {
   NSCalendarDate *d = [calendarClass alloc];
-  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
+  d = [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
   [d setCalendarFormat: formatString];
   [d setTimeZone: timeZone];
   return AUTORELEASE(d);
@@ -1157,10 +1156,10 @@ otherTime(NSDate* other)
   // Easiest to just have NSCalendarDate do the work for us
   NSString *s;
   NSCalendarDate *d = [calendarClass alloc];
-  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
-  s = [d description];
+  d = [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
+  s = [[d description] retain];
   RELEASE(d);
-  return s;
+  return [s autorelease];
 }
 
 /**
@@ -1176,7 +1175,7 @@ otherTime(NSDate* other)
   NSCalendarDate *d = [calendarClass alloc];
   id f;
 
-  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
+  d = [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
   if (!format)
     f = [d calendarFormat];
   else
@@ -1184,24 +1183,24 @@ otherTime(NSDate* other)
   if (aTimeZone)
     [d setTimeZone: aTimeZone];
 
-  s = [d descriptionWithCalendarFormat: f locale: l];
+  s = [[d descriptionWithCalendarFormat: f locale: l] retain];
   RELEASE(d);
-  return s;
+  return [s autorelease];
 }
 
 /**
  * Returns a string representation of the receiver formatted according
  * to the default format string and time zone, but using the given locale.
  */
-- (NSString *) descriptionWithLocale: (NSDictionary *)locale
+- (NSString *) descriptionWithLocale: (id)locale
 {
   // Easiest to just have NSCalendarDate do the work for us
   NSString *s;
   NSCalendarDate *d = [calendarClass alloc];
-  [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
-  s = [d descriptionWithLocale: locale];
+  d = [d initWithTimeIntervalSinceReferenceDate: otherTime(self)];
+  s = [[d descriptionWithLocale: locale] retain];
   RELEASE(d);
-  return s;
+  return [s autorelease];
 }
 
 /**
@@ -1247,7 +1246,7 @@ otherTime(NSDate* other)
  */
 - (NSTimeInterval) timeIntervalSinceNow
 {
-  return otherTime(self) - GSTimeNow();
+  return otherTime(self) - GSPrivateTimeNow();
 }
 
 /**
@@ -1303,7 +1302,7 @@ otherTime(NSDate* other)
   return self;
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   return (unsigned)[self timeIntervalSinceReferenceDate];
 }
@@ -1415,7 +1414,7 @@ otherTime(NSDate* other)
 
 - (NSTimeInterval) timeIntervalSinceNow
 {
-  return _seconds_since_ref - GSTimeNow();
+  return _seconds_since_ref - GSPrivateTimeNow();
 }
 
 - (NSTimeInterval) timeIntervalSinceReferenceDate
@@ -1459,7 +1458,7 @@ otherTime(NSDate* other)
   return self;
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   return (unsigned)_seconds_since_ref;
 }
