@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDebug utilities reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-07 15:34:06 +0000 (Mon, 07 Mar 2011) $ $Revision: 32485 $
    */
 
 #import "common.h"
@@ -33,6 +33,7 @@
 #import "GNUstepBase/GSLock.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSData.h"
+#import "Foundation/NSDictionary.h"
 #import "Foundation/NSLock.h"
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSNotificationQueue.h"
@@ -493,7 +494,7 @@ _GSDebugAllocationList(BOOL difference)
 	}
       if (val != 0)
 	{
-	  pos += 11 + strlen(the_table[i].class->name);
+	  pos += 22 + strlen(class_getName(the_table[i].class));
 	}
     }
   if (pos == 0)
@@ -539,7 +540,8 @@ _GSDebugAllocationList(BOOL difference)
 
 	  if (val != 0)
 	    {
-	      sprintf(&buf[pos], "%d\t%s\n", val, the_table[i].class->name);
+	      snprintf(&buf[pos], siz - pos, "%d\t%s\n",
+		val, class_getName(the_table[i].class));
 	      pos += strlen(&buf[pos]);
 	    }
 	}
@@ -587,7 +589,7 @@ _GSDebugAllocationListAll(void)
 
       if (val != 0)
 	{
-	  pos += 11 + strlen(the_table[i].class->name);
+	  pos += 22 + strlen(class_getName(the_table[i].class));
 	}
     }
   if (pos == 0)
@@ -619,7 +621,8 @@ _GSDebugAllocationListAll(void)
 
 	  if (val != 0)
 	    {
-	      sprintf(&buf[pos], "%d\t%s\n", val, the_table[i].class->name);
+	      snprintf(&buf[pos], siz - pos, "%d\t%s\n",
+		val, class_getName(the_table[i].class));
 	      pos += strlen(&buf[pos]);
 	    }
 	}
@@ -682,7 +685,7 @@ _GSDebugAllocationRemove(Class c, id o)
 		    }
 		}
 	      [uniqueLock unlock];
-	      RELEASE(tag);
+	      [tag release];
 	      return;
 	    }
 	}
@@ -831,11 +834,15 @@ GSDebugAllocationListRecordedObjects(Class c)
   return answer;
 }
 
+#if	!defined(HAVE_BUILTIN_EXTRACT_RETURN_ADDRESS)
+# define	__builtin_extract_return_address(X)	X
+#endif
 
 #define _NS_FRAME_HACK(a) \
 case a: env->addr = __builtin_frame_address(a + 1); break;
 #define _NS_RETURN_HACK(a) \
-case a: env->addr = __builtin_return_address(a + 1); break;
+case a: env->addr = (__builtin_frame_address(a + 1) ? \
+__builtin_extract_return_address(__builtin_return_address(a + 1)) : 0); break;
 
 /*
  * The following horrible signal handling code is a workaround for the fact
@@ -878,13 +885,14 @@ static jbuf_type *
 jbuf()
 {
   NSMutableData	*d;
+  NSMutableDictionary	*dict;
 
-  d = [[[NSThread currentThread] threadDictionary] objectForKey: @"GSjbuf"];
+  dict = [[NSThread currentThread] threadDictionary];
+  d = [dict objectForKey: @"GSjbuf"];
   if (d == nil)
     {
       d = [[NSMutableData alloc] initWithLength: sizeof(jbuf_type)];
-      [[[NSThread currentThread] threadDictionary] setObject: d
-						      forKey: @"GSjbuf"];
+      [dict setObject: d forKey: @"GSjbuf"];
       RELEASE(d);
     }
   return (jbuf_type*)[d mutableBytes];
@@ -1086,7 +1094,7 @@ NSMutableArray *
 GSPrivateStackAddresses(void)
 {
   NSMutableArray        *stack;
-  NSAutoreleasePool     *pool;
+  NSAutoreleasePool	*pool;
 
 #if HAVE_BACKTRACE
   void                  *addresses[1024];
@@ -1180,7 +1188,7 @@ GSPrivateStackAddresses(void)
       signal(SIGBUS, env->bus);
     }
 #endif
-  RELEASE(pool);
+  [pool release];
   return stack;
 }
 
