@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSInvocation class reference</title>
-   $Date: 2010-06-17 20:09:26 +0100 (Thu, 17 Jun 2010) $ $Revision: 30773 $
+   $Date: 2011-03-09 10:24:18 +0000 (Wed, 09 Mar 2011) $ $Revision: 32507 $
    */
 
 #import "common.h"
@@ -399,16 +399,11 @@ _arg_addr(NSInvocation *inv, int index)
  */
 - (void) getReturnValue: (void*)buffer
 {
-  const char	*type;
-
   if (_validReturn == NO)
     {
       [NSException raise: NSGenericException
 		  format: @"getReturnValue with no value set"];
     }
-
-  type = [_sig methodReturnType];
-
   if (*_inf[0].type != _C_VOID)
     {
       memcpy(buffer, _retval, _inf[0].size);
@@ -481,10 +476,13 @@ _arg_addr(NSInvocation *inv, int index)
 		}
 	      else
 		{
+		  int	len;
 		  char	*tmp;
 
-		  tmp = NSZoneMalloc(NSDefaultMallocZone(), strlen(newstr)+1);
-		  strcpy(tmp, newstr);
+		  len = strlen(newstr);
+		  tmp = NSZoneMalloc(NSDefaultMallocZone(), len + 1);
+		  strncpy(tmp, newstr, len);
+		  tmp[len] = '\0';
 		  _set_arg(self, index, tmp);
 		}
 	      if (oldstr != 0)
@@ -615,9 +613,12 @@ _arg_addr(NSInvocation *inv, int index)
 	      if (str != 0)
 	        {
 		  char  *tmp;
+		  int	len;
 
-		  tmp = NSZoneMalloc(NSDefaultMallocZone(), strlen(str)+1);
-		  strcpy(tmp, str);
+		  len = strlen(str);
+		  tmp = NSZoneMalloc(NSDefaultMallocZone(), len + 1);
+		  strncpy(tmp, str, len);
+		  tmp[len] = '\0';
 		  _set_arg(self, i-1, &tmp);
 		}
 	    }
@@ -773,18 +774,6 @@ _arg_addr(NSInvocation *inv, int index)
  * create invocations.
  */
 @implementation NSInvocation (MacroSetup)
-/**
- * Internal use.<br />
- * Initialises the receiver with a known selector and argument list
- * as supplied to the forward:: method by the ObjectiveC runtime
- * when it is unable to locate an implementation for the selector
- * in a class.
- */
-- (id) initWithArgframe: (arglist_t)frame selector: (SEL)aSelector
-{
-  [self subclassResponsibility: _cmd];
-  return nil;
-}
 
 /** <init /><override-subclass />
  * Initialised an invocation instance which can be used to send messages to
@@ -804,11 +793,6 @@ _arg_addr(NSInvocation *inv, int index)
  * Provides a return frame that the ObjectiveC runtime can use to
  * return the result of an invocation to a calling function.
  */
-- (void*) returnFrame: (arglist_t)argFrame
-{
-  [self subclassResponsibility: _cmd];
-  return NULL;
-}
 
 + (id) _newProxyForInvocation: (id)target
 {
@@ -839,10 +823,6 @@ _arg_addr(NSInvocation *inv, int index)
 #warning Using dummy NSInvocation implementation.  It is strongly recommended that you use libffi.
 @implementation GSDummyInvocation
 
-- (id) initWithArgframe: (arglist_t)frame selector: (SEL)aSelector
-{
-  return self;
-}
 
 /*
  *	This is the de_signated initialiser.
@@ -866,10 +846,6 @@ _arg_addr(NSInvocation *inv, int index)
     }
 }
 
-- (void*) returnFrame: (arglist_t)argFrame
-{
-  return 0;
-}
 @end
 #endif
 
@@ -886,19 +862,6 @@ _arg_addr(NSInvocation *inv, int index)
 {
   return invocation;
 }
-- (retval_t) forward: (SEL)aSel : (arglist_t)argFrame
-{
-  NSInvocation	*inv;
-
-  if (aSel == 0)
-    [NSException raise: NSInvalidArgumentException
-		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
-
-  inv = AUTORELEASE([[NSInvocation alloc] initWithArgframe: argFrame
-						  selector: aSel]);
-  [self forwardInvocation: inv];
-  return [inv returnFrame: argFrame];
-}
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
   invocation = anInvocation;
