@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-09 10:42:16 +0000 (Wed, 09 Mar 2011) $ $Revision: 32512 $
 */
 
 #import "common.h"
@@ -47,7 +47,7 @@
 /*
  * IMPLEMENTATION NOTES
  *
- * Originally, I wanted to do KVO via a proxy, with isa swizzling
+ * Originally, I wanted to do KVO via a proxy, with class pointer swizzling
  * to turn the original instance into an instance of the proxy class.
  * However, I couldn't figure a way to get decent performance out of
  * this model, as every message to the instance would have to be
@@ -57,7 +57,7 @@
  * So, instead I arrived at the mechanism of creating a subclass of
  * each class being observed, with a few subclass methods overriding
  * those of the original, but most remaining the same.
- * The same isa swizzling technique was used to convert between the
+ * The same class pointer swizzling technique was used to convert between the
  * original class and the superclass.
  * This subclass basically overrides several standard methods with
  * those from a template class, and then overrides any setter methods
@@ -77,19 +77,25 @@ static NSMapTable       *dependentKeyTable;
 static Class		baseClass;
 static id               null;
 
-static inline void setup()
+static inline void
+setup()
 {
-  if (kvoLock == nil)
+  if (nil == kvoLock)
     {
-      kvoLock = [GSLazyRecursiveLock new];
-      null = [[NSNull null] retain];
-      classTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
-	NSNonOwnedPointerMapValueCallBacks, 128);
-      infoTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
-	NSNonOwnedPointerMapValueCallBacks, 1024);
-      dependentKeyTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
-          NSOwnedPointerMapValueCallBacks, 128);
-      baseClass = NSClassFromString(@"GSKVOBase");
+      [gnustep_global_lock lock];
+      if (nil == kvoLock)
+	{
+	  kvoLock = [GSLazyRecursiveLock new];
+	  null = [[NSNull null] retain];
+	  classTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
+	    NSNonOwnedPointerMapValueCallBacks, 128);
+	  infoTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
+	    NSNonOwnedPointerMapValueCallBacks, 1024);
+	  dependentKeyTable = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
+	      NSOwnedPointerMapValueCallBacks, 128);
+	  baseClass = NSClassFromString(@"GSKVOBase");
+	}
+      [gnustep_global_lock unlock];
     }
 }
 /*
@@ -192,11 +198,6 @@ static inline void setup()
   NSString                              *keyPathToForward;
 }
 
-+ (id) forwarderWithKeyPath: (NSString *)keyPath
-                   ofObject: (id)object
-                 withTarget: (id)aTarget
-                    context: (void *)context;
-
 - (id) initWithKeyPath: (NSString *)keyPath
               ofObject: (id)object
             withTarget: (id)aTarget
@@ -211,7 +212,7 @@ static inline void setup()
 {
   // Turn off KVO for self ... then call the real dealloc implementation.
   [self setObservationInfo: nil];
-  isa = [self class];
+  object_setClass(self, [self class]);
   [self dealloc];
   GSNOSUPERDEALLOC;
 }
@@ -313,6 +314,7 @@ static NSString *newKey(SEL _cmd)
   unsigned	len = strlen(name);
   NSString	*key;
   unsigned	i;
+  NSCAssert(len > 0, @"Invalid selector name!");
 
   if (*name == '_')
     {
@@ -359,7 +361,7 @@ static NSString *newKey(SEL _cmd)
       u = uni_tolower(u);
       tmp = [[NSString alloc] initWithCharacters: &u length: 1];
       [m replaceCharactersInRange: NSMakeRange(0, 1) withString: tmp];
-      RELEASE(tmp);
+      [tmp release];
       key = m;
     }
   return key;
@@ -372,7 +374,6 @@ replacementForClass(Class c)
   GSKVOReplacement *r;
 
   setup();
-
   [kvoLock lock];
   r = (GSKVOReplacement*)NSMapGet(classTable, (void*)c);
   if (r == nil)
@@ -455,6 +456,7 @@ replacementForClass(Class c)
       tmp = [[NSString alloc] initWithCharacters: &u length: 1];
       a[0] = [NSString stringWithFormat: @"set%@%@:", tmp, suffix];
       a[1] = [NSString stringWithFormat: @"_set%@%@:", tmp, suffix];
+      [tmp release];
       for (i = 0; i < 2; i++)
         {
           /*
@@ -552,7 +554,7 @@ replacementForClass(Class c)
                   {
                     imp = [[GSKVOSetter class]
                       instanceMethodForSelector: @selector(setterRect:)];
-                  }                           
+                  }
                 else
                   {
                     imp = 0;
@@ -611,12 +613,12 @@ replacementForClass(Class c)
                     }
                   NSEndHashTableEnumeration(&dependentKeyEnum);
                }
-              NSEndMapTableEnumeration(&enumerator); 
+              NSEndMapTableEnumeration(&enumerator);
             }
 
           if (!found)
             {
-              NSDebugLLog(@"KVC", @"class %@ not KVC complient for %@",
+              NSDebugLLog(@"KVC", @"class %@ not KVC compliant for %@",
 		original, aKey);
               /*
               [NSException raise: NSInvalidArgumentException
@@ -957,7 +959,7 @@ replacementForClass(Class c)
   observations = [NSMutableArray new];
   return self;
 }
- 
+
 - (void) notifyForKey: (NSString *)aKey ofInstance: (id)instance prior: (BOOL)f
 {
   unsigned      count;
@@ -989,6 +991,10 @@ replacementForClass(Class c)
       newValue = null;
     }
 
+  /* Retain self so that we won't be deallocated during the
+   * notification process.
+   */
+  [self retain];
   count = [observations count];
   while (count-- > 0)
     {
@@ -1026,6 +1032,7 @@ replacementForClass(Class c)
   [oldValue release];
   [change setObject: newValue forKey: NSKeyValueChangeNewKey];
   [newValue release];
+  [self release];
 }
 @end
 
@@ -1178,7 +1185,7 @@ replacementForClass(Class c)
 }
 
 /*
- * removes the observer 
+ * removes the observer
  */
 - (void) removeObserver: (NSObject*)anObserver forKeyPath: (NSString*)aPath
 {
@@ -1251,17 +1258,6 @@ replacementForClass(Class c)
 
 @implementation NSKeyValueObservationForwarder
 
-+ (id) forwarderWithKeyPath: (NSString *)keyPath
-                   ofObject: (id)object
-                 withTarget: (id)aTarget
-                    context: (void *)context
-{
-  return [[self alloc] initWithKeyPath: keyPath
-                              ofObject: object
-                            withTarget: aTarget
-                               context: context];
-}
-
 - (id) initWithKeyPath: (NSString *)keyPath
               ofObject: (id)object
             withTarget: (id)aTarget
@@ -1291,11 +1287,11 @@ replacementForClass(Class c)
   dot = [remainingKeyPath rangeOfString: @"."];
   if (dot.location != NSNotFound)
     {
-      child = [NSKeyValueObservationForwarder
-        forwarderWithKeyPath: remainingKeyPath
-                    ofObject: [object valueForKey: keyForUpdate]
-                  withTarget: self
-                     context: NULL];
+      child = [[NSKeyValueObservationForwarder alloc]
+        initWithKeyPath: remainingKeyPath
+	       ofObject: [object valueForKey: keyForUpdate]
+	     withTarget: self
+		context: NULL];
       observedObjectForForwarding = nil;
     }
   else
@@ -1325,7 +1321,7 @@ replacementForClass(Class c)
     }
   if (observedObjectForForwarding)
     {
-      [observedObjectForForwarding removeObserver: self forKeyPath: 
+      [observedObjectForForwarding removeObserver: self forKeyPath:
         keyForForwarding];
     }
   DESTROY(self);
@@ -1345,7 +1341,7 @@ replacementForClass(Class c)
                          change: (NSDictionary *)change
                         context: (void *)context
 {
-  if (anObject == observedObjectForUpdate) 
+  if (anObject == observedObjectForUpdate)
     {
       [self keyPathChanged: nil];
     }
@@ -1379,8 +1375,8 @@ replacementForClass(Class c)
     {
       NSMutableDictionary *change;
 
-      change = [NSMutableDictionary dictionaryWithObject: 
-                                        [NSNumber numberWithInt: 1] 
+      change = [NSMutableDictionary dictionaryWithObject:
+                                        [NSNumber numberWithInt: 1]
                                     forKey:  NSKeyValueChangeKindKey];
 
       if (observedObjectForForwarding != nil)
@@ -1389,7 +1385,7 @@ replacementForClass(Class c)
 
           oldValue
             = [observedObjectForForwarding valueForKey: keyForForwarding];
-          [observedObjectForForwarding removeObserver: self forKeyPath: 
+          [observedObjectForForwarding removeObserver: self forKeyPath:
                                            keyForForwarding];
           if (oldValue)
             {
@@ -1469,7 +1465,7 @@ replacementForClass(Class c)
     {
       info = [[GSKVOInfo alloc] initWithInstance: self];
       [self setObservationInfo: info];
-      isa = [r replacement];
+      object_setClass(self, [r replacement]);
     }
 
   /*
@@ -1478,11 +1474,11 @@ replacementForClass(Class c)
   dot = [aPath rangeOfString:@"."];
   if (dot.location != NSNotFound)
     {
-      forwarder = [NSKeyValueObservationForwarder
-        forwarderWithKeyPath: aPath
-                    ofObject: self
-                  withTarget: anObserver
-                     context: aContext];
+      forwarder = [[NSKeyValueObservationForwarder alloc]
+        initWithKeyPath: aPath
+	       ofObject: self
+	     withTarget: anObserver
+		context: aContext];
       [info addObserver: anObserver
              forKeyPath: aPath
                 options: options
@@ -1519,7 +1515,7 @@ replacementForClass(Class c)
        * The instance is no longer being observed ... so we can
        * turn off key-value-observing for it.
        */
-      isa = [self class];
+      object_setClass(self, [self class]);
       IF_NO_GC(AUTORELEASE(info);)
       [self setObservationInfo: nil];
     }
@@ -1661,7 +1657,7 @@ replacementForClass(Class c)
       if (pathInfo->recursion++ == 0)
         {
           id    old = [pathInfo->change objectForKey: NSKeyValueChangeNewKey];
-          
+
           if (old != nil)
             {
               /* We have set a value for this key already, so the value
