@@ -24,10 +24,13 @@
    Boston, MA 02111 USA.
 
    <title>NSMethodSignature class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-07 15:34:06 +0000 (Mon, 07 Mar 2011) $ $Revision: 32485 $
    */
 
 #import "common.h"
+#ifndef __GNU_LIBOBJC__
+#include <objc/encoding.h>
+#endif
 #define	EXPOSE_NSMethodSignature_IVARS	1
 
 #import "Foundation/NSMethodSignature.h"
@@ -388,22 +391,30 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
       const char	*q;
       char		*args;
       char		*ret;
+      char		*end;
+      char		*ptr;
+      int		alen;
+      int		blen;
+      int		rlen;
 
 /* In case we have been given a method encoding string without offsets,
  * we attempt to generate the frame size and offsets in a new copy of
  * the types string.
  */
-      ret = alloca((strlen(t)+1)*16);
+      blen = (strlen(t) + 1) * 16;	// Total buffer length
+      ret = alloca(blen);
+      end = ret + blen;
 
       /* Copy the return type (including qualifiers) with ehough room
        * after it to store the frame size.
        */
       p = t;
       p = objc_skip_typespec (p);
-      strncpy(ret, t, p - t);
-      ret[p - t] = '\0';
-      args = ret + (p - t) + 10;
-      *args = '\0';
+      rlen = p - t;
+      strncpy(ret, t, rlen);
+      ret[rlen] = '\0';
+      ptr = args = ret + rlen + 10;	// Allow room for a decimal integer
+      *ptr = '\0';
 
       /* Skip to the first arg type, taking note of where the qualifiers start.
        * Assume that casting _argFrameLength to int will not lose information.
@@ -418,18 +429,22 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
 	  _numArgs++;
 	  size = objc_promoted_size (p);
 	  p = objc_skip_typespec (p);
-	  strncat(args, q, p - q);
-	  sprintf(args + strlen(args), "%d", (int)_argFrameLength);
+	  memcpy(ptr, q, p - q);
+	  ptr += (p - q);
+	  snprintf(ptr, end - ptr, "%d", (int)_argFrameLength);
+	  ptr += strlen(ptr);
 	  _argFrameLength += size;
 	  p = skip_offset (p);
 	  q = p;
 	  p = objc_skip_type_qualifiers (p);
 	}
-      sprintf(ret + strlen(ret), "%d", (int)_argFrameLength);
-      _methodTypes = NSZoneMalloc(NSDefaultMallocZone(),
-	strlen(args) + strlen(ret) + 1);
-      strcpy((char*)_methodTypes, ret);
-      strcat((char*)_methodTypes, args);
+      alen = ptr - args;
+      sprintf(ret + rlen, "%d", (int)_argFrameLength);
+
+      _methodTypes = NSZoneMalloc(NSDefaultMallocZone(), alen + rlen + 1);
+      strncpy((char*)_methodTypes, ret, rlen);
+      strncpy(((char*)_methodTypes) + rlen, args, alen);
+      ((char*)_methodTypes)[alen + rlen] = '\0';
     }
   return self;
 }
@@ -449,6 +464,7 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
   if (_inf == 0)
     {
       [self methodInfo];
+      NSAssert(0 != _inf, @"Initialising failed");
     }
   return _inf[index+1];
 }
@@ -468,6 +484,7 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
   if (_inf == 0)
     {
       [self methodInfo];
+      NSAssert(0 != _inf, @"Initialising failed");
     }
   return _inf[index+1].qtype;
 }
@@ -477,6 +494,7 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
   if (_inf == 0)
     {
       [self methodInfo];
+      NSAssert(0 != _inf, @"Initialising failed");
     }
   return (_inf[0].qual & _F_ONEWAY) ? YES : NO;
 }
@@ -486,6 +504,7 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
   if (_inf == 0)
     {
       [self methodInfo];
+      NSAssert(0 != _inf, @"Initialising failed");
     }
   return _inf[0].size;
 }
@@ -495,6 +514,7 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
   if (_inf == 0)
     {
       [self methodInfo];
+      NSAssert(0 != _inf, @"Initialising failed");
     }
   return _inf[0].qtype;
 }
@@ -520,7 +540,7 @@ next_arg(const char *typePtr, NSArgumentInfo *info, char *outTypes)
     {
       return NO;
     }
-  if (((NSMethodSignature *)other)->isa != isa)
+  if (object_getClass(other) != object_getClass(self))
     {
       return NO;
     }
