@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSNotificationCenter class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-29 09:16:02 +0100 (Tue, 29 Mar 2011) $ $Revision: 32731 $
 */
 
 #import "common.h"
@@ -207,13 +207,13 @@ static void obsFree(Observation *o);
 #define GSI_MAP_RETAIN_VAL(M, X)
 #define GSI_MAP_RELEASE_VAL(M, X)
 
-#define GSI_MAP_KTYPES GSUNION_OBJ|GSUNION_INT
+#define GSI_MAP_KTYPES GSUNION_OBJ|GSUNION_NSINT
 #define GSI_MAP_VTYPES GSUNION_PTR
 #define GSI_MAP_VEXTRA Observation*
 #define	GSI_MAP_EXTRA	void*
 
 #if	GS_WITH_GC
-#include	<gc_typed.h>
+#include	<gc/gc_typed.h>
 static GC_descr	nodeDesc;	// Type descriptor for map node.
 #define	GSI_MAP_NODES(M, X) \
 (GSIMapNode)GC_calloc_explicitly_typed(X, sizeof(GSIMapNode_t), nodeDesc)
@@ -790,6 +790,7 @@ static NSNotificationCenter *default_center = nil;
 	   */
 	  name = [name copyWithZone: NSDefaultMallocZone()];
 	  GSIMapAddPair(NAMED, (GSIMapKey)(id)name, (GSIMapVal)(void*)m);
+	  GS_CONSUMED(name)
 	}
       else
 	{
