@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObjCRuntime class reference</title>
-   $Date: 2010-08-21 09:03:32 +0100 (Sat, 21 Aug 2010) $ $Revision: 31191 $
+   $Date: 2011-02-07 22:53:35 +0000 (Mon, 07 Feb 2011) $ $Revision: 32012 $
    */
 
 #import "common.h"
@@ -37,7 +37,7 @@ NSString *
 NSStringFromProtocol(Protocol *aProtocol)
 {
   if (aProtocol != (Protocol*)0)
-    return [NSString stringWithUTF8String: (const char*)[aProtocol name]];
+    return [NSString stringWithUTF8String: protocol_getName(aProtocol)];
   return nil;
 }
 
@@ -95,7 +95,8 @@ NSSelectorFromString(NSString *aSelectorName)
 
 /**
  * Returns the class whose name is supplied in the
- * aClassName argument, or 0 if a nil string is supplied.
+ * aClassName argument, or Nil if a nil string is supplied.
+ * If no such class has been loaded, the function returns Nil.
  */
 Class
 NSClassFromString(NSString *aClassName)
