@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObject class reference</title>
-   $Date: 2010-09-08 10:00:34 +0100 (Wed, 08 Sep 2010) $ $Revision: 31249 $
+   $Date: 2011-04-14 13:36:06 +0100 (Thu, 14 Apr 2011) $ $Revision: 32861 $
    */
 
 /* On some versions of mingw we need to work around bad function declarations
@@ -57,17 +57,17 @@
 #ifdef	HAVE_SYS_SIGNAL_H
 #include	<sys/signal.h>
 #endif
-#ifdef __FreeBSD__
+
+#if __GNUC__ >= 4
+#if defined(__FreeBSD__)
 #include <fenv.h>
 #endif
+#endif // __GNUC__
 
+#define	IN_NSOBJECT_M	1
 #import "GSPrivate.h"
 
 
-#ifndef NeXT_RUNTIME
-extern BOOL __objc_responds_to(id, SEL);
-#endif
-
 /* When this is `YES', every call to release/autorelease, checks to
    make sure isn't being set up to release itself too many times.
    This does not need mutex protection. */
@@ -84,13 +84,13 @@ static IMP autorelease_imp;
 
 #if GS_WITH_GC
 
-#include	<gc.h>
-#include	<gc_typed.h>
+#include	<gc/gc.h>
+#include	<gc/gc_typed.h>
+
+#endif
 
 static SEL finalize_sel;
 static IMP finalize_imp;
-#endif
-
 static Class	NSConstantStringClass;
 
 @class	NSDataMalloc;
@@ -101,7 +101,6 @@ static Class	NSConstantStringClass;
   Class	isa;
 }
 - (Class) class;
-- (retval_t) forward:(SEL)aSel :(arglist_t)argFrame;
 - (void) forwardInvocation: (NSInvocation*)anInvocation;
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector;
 @end
@@ -123,15 +122,16 @@ BOOL	NSZombieEnabled = NO;
 BOOL	NSDeallocateZombies = NO;
 
 @class	NSZombie;
-static Class		zombieClass;
-static NSMapTable	*zombieMap;
+static Class		zombieClass = Nil;
+static NSMapTable	*zombieMap = 0;
 
 #if	!GS_WITH_GC
 static void GSMakeZombie(NSObject *o)
 {
-  Class	c = ((id)o)->class_pointer;
+  Class		c;
 
-  ((id)o)->class_pointer = zombieClass;
+  c = object_getClass(o);
+  object_setClass(o, zombieClass);
   if (NSDeallocateZombies == NO)
     {
       [allocationLock lock];
@@ -153,13 +153,13 @@ static void GSLogZombie(id o, SEL sel)
     }
   if (c == 0)
     {
-      NSLog(@"Deallocated object (%p) sent %@",
-	o, NSStringFromSelector(sel));
+      NSLog(@"*** -[??? %@]: message sent to deallocated instance %p",
+	NSStringFromSelector(sel), o);
     }
   else
     {
-      NSLog(@"Deallocated %@ (%p) sent %@",
-	c, o, NSStringFromSelector(sel));
+      NSLog(@"*** -[%@ %@]: message sent to deallocated instance %p",
+	c, NSStringFromSelector(sel), o);
     }
   if (GSPrivateEnvironmentFlag("CRASH_ON_ZOMBIE", NO) == YES)
     {
@@ -318,7 +318,9 @@ GSAtomicIncrement(gsatomic_t X)
   int tmp;
 
   __asm__ __volatile__ (
+#if !defined(__mips64__)
     "   .set  mips2  \n"
+#endif
     "0: ll    %0, %1 \n"
     "   addiu %0, 1  \n"
     "   sc    %0, %1 \n"
@@ -333,7 +335,9 @@ GSAtomicDecrement(gsatomic_t X)
   int tmp;
 
   __asm__ __volatile__ (
+#if !defined(__mips64__)
     "   .set  mips2  \n"
+#endif
     "0: ll    %0, %1 \n"
     "   addiu %0, -1 \n"
     "   sc    %0, %1 \n"
@@ -550,6 +554,41 @@ NSIncrementExtraRefCount(id anObject)
 #define	AREM(c, o) 
 #endif
 
+static SEL cxx_construct, cxx_destruct;
+
+/**
+ * Calls the C++ constructors for this object, starting with the ones declared
+ * in aClass.  The compiler generates two methods on Objective-C++ classes that
+ * static instances of C++ classes as ivars.  These are -.cxx_construct and
+ * -.cxx_destruct.  The -.cxx_construct methods must be called in order from
+ *  the root class to all subclasses, to ensure that subclass ivars are
+ *  initialised after superclass ones.  This must be done in reverse for
+ *  destruction.  
+ *
+ *  This function first calls itself recursively on the superclass, to get the
+ *  IMP for the constructor function in the superclass.  It then compares the
+ *  construct method for this class with the one that's already been called,
+ *  and calls it if it's new.
+ */
+static IMP
+callCXXConstructors(Class aClass, id anObject)
+{
+  IMP constructor = 0;
+
+  if (class_respondsToSelector(aClass, cxx_construct))
+    {
+      IMP calledConstructor =
+        callCXXConstructors(class_getSuperclass(aClass), anObject);
+      constructor = class_getMethodImplementation(aClass, cxx_construct);
+      if (calledConstructor != constructor)
+        {
+          constructor(anObject, cxx_construct);
+        }
+    }
+  return constructor;
+}
+
+
 /*
  *	Now do conditional compilation of memory allocation functions
  *	depending on what information (if any) we are storing before
@@ -570,14 +609,15 @@ static void
 GSFinalize(void* object, void* data)
 {
   [(id)object finalize];
-  AREM(((id)object)->class_pointer, (id)object);
-  ((id)object)->class_pointer = (void*)0xdeadface;
+  AREM(object_getClass((id)object), (id)object);
+  object_setClass((id)object, (Class)(void*)0xdeadface);
 }
 
 static BOOL
 GSIsFinalizable(Class c)
 {
-  if (get_imp(c, finalize_sel) != finalize_imp)
+  if (class_getMethodImplementation(c, finalize_sel) != finalize_imp
+    && class_respondsToSelector(c, finalize_sel))
     return YES;
   return NO;
 }
@@ -589,7 +629,7 @@ NSAllocateObject(Class aClass, NSUInteger extraBytes, NSZone *zone)
   int	size;
   GC_descr	gc_type;
 
-  NSCAssert((CLS_ISCLASS(aClass)), @"Bad class for new object");
+  NSCAssert((!class_isMetaClass(aClass)), @"Bad class for new object");
   gc_type = (GC_descr)aClass->gc_object_type;
   size = class_getInstanceSize(aClass) + extraBytes;
   if (size % sizeof(void*) != 0)
@@ -613,8 +653,27 @@ NSAllocateObject(Class aClass, NSUInteger extraBytes, NSZone *zone)
 
   if (new != nil)
     {
-      new->class_pointer = aClass;
-      if (GSIsFinalizable(aClass))
+      object_setClass(new, aClass);
+
+      /* Don't bother doing this in a thread-safe way, because
+       * the cost of locking will be a lot more than the cost
+       * of doing the same call in two threads.
+       * The returned selector will persist and the runtime will
+       * ensure that both calls return the same selector, so we
+       * don't need to bother doing it ourselves.
+       */
+      if (0 == cxx_construct)
+	{
+	  cxx_construct = sel_registerName(".cxx_construct");
+	  cxx_destruct = sel_registerName(".cxx_destruct");
+	}
+      callCXXConstructors(aClass, new);
+
+      /* We only need to finalize this object if it implements its a
+       * -finalize method or has C++ destructors.
+       */
+      if (GSIsFinalizable(aClass)
+        || class_respondsToSelector(aClass, cxx_destruct))
 	{
 	  /* We only do allocation counting for objects that can be
 	   * finalised - for other objects we have no way of decrementing
@@ -649,7 +708,7 @@ NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
   id	new;
   int	size;
 
-  NSCAssert((CLS_ISCLASS(aClass)), @"Bad class for new object");
+  NSCAssert((!class_isMetaClass(aClass)), @"Bad class for new object");
   size = class_getInstanceSize(aClass) + extraBytes + sizeof(struct obj_layout);
   if (zone == 0)
     {
@@ -661,21 +720,41 @@ NSAllocateObject (Class aClass, NSUInteger extraBytes, NSZone *zone)
       memset (new, 0, size);
       ((obj)new)->zone = zone;
       new = (id)&((obj)new)[1];
-      new->class_pointer = aClass;
+      object_setClass(new, aClass);
       AADD(aClass, new);
     }
+
+  /* Don't bother doing this in a thread-safe way, because the cost of locking
+   * will be a lot more than the cost of doing the same call in two threads.
+   * The returned selector will persist and the runtime will ensure that both
+   * calls return the same selector, so we don't need to bother doing it
+   * ourselves.
+   */
+  if (0 == cxx_construct)
+    {
+      cxx_construct = sel_registerName(".cxx_construct");
+      cxx_destruct = sel_registerName(".cxx_destruct");
+    }
+  callCXXConstructors(aClass, new);
+
   return new;
 }
 
 inline void
 NSDeallocateObject(id anObject)
 {
-  if ((anObject!=nil) && CLS_ISCLASS(((id)anObject)->class_pointer))
+  Class aClass = object_getClass(anObject);
+
+  if ((anObject != nil) && !class_isMetaClass(aClass))
     {
       obj	o = &((obj)anObject)[-1];
       NSZone	*z = o->zone;
 
-      AREM(((id)anObject)->class_pointer, (id)anObject);
+      /* Call the default finalizer to handle C++ destructors.
+       */
+      (*finalize_imp)(anObject, finalize_sel);
+
+      AREM(aClass, (id)anObject);
       if (NSZombieEnabled == YES)
 	{
 	  GSMakeZombie(anObject);
@@ -686,7 +765,7 @@ NSDeallocateObject(id anObject)
 	}
       else
 	{
-	  ((id)anObject)->class_pointer = (void*) 0xdeadface;
+	  object_setClass((id)anObject, (Class)(void*)0xdeadface);
 	  NSZoneFree(z, o);
 	}
     }
@@ -695,42 +774,6 @@ NSDeallocateObject(id anObject)
 
 #endif	/* GS_WITH_GC */
 
-
-void
-GSPrivateSwizzle(id o, Class c)
-{
-  if ((Class)o->class_pointer != c)
-    {
-#if	GS_WITH_GC
-      /* We only do allocation counting for objects that can be
-       * finalised - for other objects we have no way of decrementing
-       * the count when the object is collected.
-       */
-      if (GSIsFinalizable(o->class_pointer))
-	{
-	  /* Already finalizable, so we just need to do any allocation
-	   * accounting.
-	   */
-          AREM(o->class_pointer, o);
-          AADD(c, o);
-	}
-      else if (GSIsFinalizable(c))
-	{
-	  /* New class is finalizable, so we must register the instance
-	   * for finalisation and do allocation acounting for it.
-	   */
-	  AADD(c, o);
-	  GC_REGISTER_FINALIZER (o, GSFinalize, NULL, NULL, NULL);
-	}
-#else
-      AREM(o->class_pointer, o);
-      AADD(c, o);
-#endif	/* GS_WITH_GC */
-      o->class_pointer = c;
-    }
-}
-
-
 BOOL
 NSShouldRetainWithZone (NSObject *anObject, NSZone *requestedZone)
 {
@@ -743,101 +786,6 @@ NSShouldRetainWithZone (NSObject *anObject, NSZone *requestedZone)
 }
 
 
-
-/* FIXME ... the following code is a hack for the gnu runtime only
- */
-struct objc_method_description_list {
-  int count;
-  struct objc_method_description list[1];
-};
-
-/* Must have same layout as ivars of Protocol class
- */
-struct protocol_class {
-  Class	isa;
-  char	*protocol_name;
-  struct objc_protocol_list *protocol_list;
-  struct objc_method_description_list *instance_methods;
-  struct objc_method_description_list *class_methods;
-};
-
-struct objc_method_description *
-GSDescriptionForInstanceMethod(Protocol *self, SEL aSel)
-{
-  struct protocol_class *pcl = (struct protocol_class*)self;
-  int i;
-  struct objc_protocol_list	*p_list;
-  const char			*name = sel_getName(aSel);
-  struct objc_method_description *result;
-
-  if (pcl->instance_methods != 0)
-    {
-      for (i = 0; i < pcl->instance_methods->count; i++)
-	{
-	  if (!strcmp ((char*)pcl->instance_methods->list[i].name, name))
-	    return &(pcl->instance_methods->list[i]);
-	}
-    }
-  for (p_list = pcl->protocol_list; p_list != 0; p_list = p_list->next)
-    {
-      for (i = 0; i < p_list->count; i++)
-	{
-	  result = GSDescriptionForInstanceMethod(p_list->list[i], aSel);
-	  if (result)
-	    {
-	      return result;
-	    }
-	}
-    }
-
-  return NULL;
-}
-
-struct objc_method_description *
-GSDescriptionForClassMethod(Protocol *self, SEL aSel)
-{
-  struct protocol_class *pcl = (struct protocol_class*)self;
-  int i;
-  struct objc_protocol_list	*p_list;
-  const char			*name = sel_getName(aSel);
-  struct objc_method_description *result;
-
-  if (pcl->class_methods != 0)
-    {
-      for (i = 0; i < pcl->class_methods->count; i++)
-	{
-	  if (!strcmp ((char*)pcl->class_methods->list[i].name, name))
-	    return &(pcl->class_methods->list[i]);
-	}
-    }
-  for (p_list = pcl->protocol_list; p_list != 0; p_list = p_list->next)
-    {
-      for (i = 0; i < p_list->count; i++)
-	{
-	  result = GSDescriptionForClassMethod(p_list->list[i], aSel);
-	  if (result)
-	    {
-	      return result;
-	    }
-	}
-    }
-
-  return NULL;
-}
-
-@implementation	Protocol (Fixup)
-
-- (struct objc_method_description *) descriptionForInstanceMethod:(SEL)aSel
-{
-  return GSDescriptionForInstanceMethod(self, aSel);
-}
-
-- (struct objc_method_description *) descriptionForClassMethod:(SEL)aSel;
-{
-  return GSDescriptionForClassMethod(self, aSel);
-}
-
-@end
 
 /**
  * <p>
@@ -925,7 +873,7 @@ static void
 GSGarbageCollectorLog(char *msg, GC_word arg)
 {
   char	buf[strlen(msg)+1024];
-  sprintf(buf, msg, (unsigned long)arg);
+  snprintf(buf, sizeof(buf), msg, (unsigned long)arg);
   fprintf(stderr, "Garbage collector: %s", buf);
 }
 #endif
@@ -934,19 +882,12 @@ GSGarbageCollectorLog(char *msg, GC_word arg)
  * Semi-private function in libobjc2 that initialises the classes used for
  * blocks.
  */
-#ifndef __MINGW__
-BOOL 
-objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
-#endif
+extern BOOL 
+objc_create_block_classes_as_subclasses_of(Class super);
 
-+ (void)load
++ (void) load
 {
-#ifndef __MINGW__
-  /* When NSObject is loaded, register it as the superclass of the block
-   * classes */
-  if (objc_create_block_classes_as_subclasses_of)
-    objc_create_block_classes_as_subclasses_of(self);
-#endif
+  objc_create_block_classes_as_subclasses_of(self);
 }
 
 + (void) initialize
@@ -1011,10 +952,8 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 #endif /* SIGPIPE */
 #endif /* __MINGW__ */
 
-#if	GS_WITH_GC
       finalize_sel = @selector(finalize);
-      finalize_imp = get_imp(self, finalize_sel);
-#endif
+      finalize_imp = class_getMethodImplementation(self, finalize_sel);
 
 #if (defined(__FreeBSD__) || defined(__OpenBSD__)) && defined(__i386__)
       // Manipulate the FPU to add the exception mask. (Fixes SIGFPE
@@ -1033,29 +972,58 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 #  endif
 #endif
 
+
 #ifdef HAVE_LOCALE_H
-      GSSetLocaleC(LC_ALL, "");		// Set up locale from environment.
+      /* Set up locale from environment.
+       * This function should not use any ObjC code since important
+       * classes are not yet initialized.
+       */
+      GSSetLocaleC(LC_ALL, "");
 #endif
 
-      // Create the global lock
+      /* Create the global lock.
+       * NB. Ths is one of the first things we do ... setting up a new lock
+       * must not call any other Objective-C classes and must not involve
+       * any use of the autorelease system.
+       */
       gnustep_global_lock = [NSRecursiveLock new];
 
-      // Behavior debugging
+      /* Behavior debugging ... enable with environment variable if needed.
+       */
       GSObjCBehaviorDebug(GSPrivateEnvironmentFlag("GNUSTEP_BEHAVIOR_DEBUG",
 	GSObjCBehaviorDebug(-1)));
 
-      // Zombie management stuff.
-      zombieMap = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
-	NSNonOwnedPointerMapValueCallBacks, 0);
-      zombieClass = [NSZombie class];
-      NSZombieEnabled = GSPrivateEnvironmentFlag("NSZombieEnabled", NO);
-      NSDeallocateZombies = GSPrivateEnvironmentFlag("NSDeallocateZombies", NO);
-
+      /* Set up the autorelease system ... we must do this before using any
+       * other class whose +initialize might autorelease something.
+       */
       autorelease_class = [NSAutoreleasePool class];
       autorelease_sel = @selector(addObject:);
       autorelease_imp = [autorelease_class methodForSelector: autorelease_sel];
+
+      /* Make sure the constant string class works and set up well-known
+       * string constants etc.
+       */
       NSConstantStringClass = [NSString constantStringClass];
       GSPrivateBuildStrings();
+
+      /* Determine zombie management flags and set up a map to store
+       * information about zombie objects.
+       */
+      NSZombieEnabled = GSPrivateEnvironmentFlag("NSZombieEnabled", NO);
+      NSDeallocateZombies = GSPrivateEnvironmentFlag("NSDeallocateZombies", NO);
+      zombieMap = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
+	NSNonOwnedPointerMapValueCallBacks, 0);
+
+      /* We need to cache the zombie class.
+       * We can't call +class because NSZombie doesn't have that method.
+       * We can't use NSClassFromString() because that would use an NSString
+       * object, and that class hasn't been initialized yet ...
+       */
+      zombieClass = objc_lookUpClass("NSZombie");
+
+      /* Now that we have a workign autorelease system and working string
+       * classes we are able to set up notifications.
+       */
       [[NSNotificationCenter defaultCenter]
 	addObserver: self
 	   selector: @selector(_becomeMultiThreaded:)
@@ -1083,10 +1051,9 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  * <p>
  *   Memory for an instance of the receiver is allocated; a
  *   pointer to this newly created instance is returned.  All
- *   instance variables are set to 0 except the
- *   <code>isa</code> pointer which is set to point to the
- *   object class.  No initialization of the instance is
- *   performed: it is your responsibility to initialize the
+ *   instance variables are set to 0.  No initialization of the
+ *   instance is performed apart from setup to be an instance of
+ *   the correct class: it is your responsibility to initialize the
  *   instance by calling an appropriate <code>init</code>
  *   method.  If you are not using the garbage collector, it is
  *   also your responsibility to make sure the returned
@@ -1194,15 +1161,14 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 
 /**
  * Returns the class of which the receiver is an instance.<br />
- * The default implementation returns the private <code>isa</code>
- * instance variable of NSObject, which is used to store a pointer
- * to the objects class.<br />
- * NB.  When NSZombie is enabled (see NSDebug.h) this pointer is
- * changed upon object deallocation.
+ * The default implementation returns the actual class that the
+ * receiver is an instance of.<br />
+ * NB.  When NSZombie is enabled (see NSDebug.h) this is changed
+ * to be the NSZombie class upon object deallocation.
  */
 - (Class) class
 {
-  return object_get_class(self);
+  return object_getClass(self);
 }
 
 /**
@@ -1315,6 +1281,30 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 
 - (void) finalize
 {
+  Class	destructorClass = object_getClass(self);
+  IMP	destructor = 0;
+
+  /* C++ destructors must be called in the opposite order to their
+   * creators, so start at the leaf class and then go up the tree until we
+   * get to the root class.  As a small optimisation, we don't bother
+   * visiting any classes that don't have an implementation of this method
+   * (including one inherited from a superclass).
+   *
+   * Care must be taken not to call inherited .cxx_destruct methods.
+   */
+  while (class_respondsToSelector(destructorClass, cxx_destruct))
+    {
+      IMP newDestructor;
+
+      newDestructor
+	= class_getMethodImplementation(destructorClass, cxx_destruct);
+      if (newDestructor != destructor)
+	{
+	  newDestructor(self, cxx_destruct);
+	  destructor = newDestructor;
+	}
+      destructorClass = class_getSuperclass(destructorClass);
+    }
   return;
 }
 
@@ -1382,7 +1372,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 	}
       return NO;
     }
-  return __objc_responds_to((id)&self, aSelector);
+  return class_respondsToSelector(self, aSelector) ? YES : NO;
 }
 
 /**
@@ -1390,7 +1380,26 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  */
 + (BOOL) conformsToProtocol: (Protocol*)aProtocol
 {
+#ifdef __GNU_LIBOBJC__
+  Class c;
+
+  /* Iterate over the current class and all the superclasses.  */
+  for (c = self; c != Nil; c = class_getSuperclass (c))
+    {
+      if (class_conformsToProtocol(c, aProtocol))
+	{
+	  return YES;
+	}
+    }
+
+  return NO;
+#else
+  /* libobjc2 and ObjectiveC2/ have an implementation of
+     class_conformsToProtocol() which automatically looks up the
+     protocol in superclasses (unlike the Apple and GNU Objective-C
+     runtime ones).  */
   return class_conformsToProtocol(self, aProtocol);
+#endif
 }
 
 /**
@@ -1399,7 +1408,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  */
 - (BOOL) conformsToProtocol: (Protocol*)aProtocol
 {
-  return class_conformsToProtocol([self class], aProtocol);
+  return [[self class] conformsToProtocol: aProtocol];
 }
 
 /**
@@ -1414,9 +1423,10 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
     [NSException raise: NSInvalidArgumentException
 		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
   /*
-   *	Since 'self' is an class, get_imp() will get the instance method.
+   * Since 'self' is an class, class_getMethodImplementation() will get
+   * the instance method.
    */
-  return get_imp((Class)self, aSelector);
+  return class_getMethodImplementation((Class)self, aSelector);
 }
 
 /**
@@ -1431,11 +1441,11 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
   /*
    *	If 'self' is an instance, object_getClass() will get the class,
-   *	and get_imp() will get the instance method.
+   *	and class_getMethodImplementation() will get the instance method.
    *	If 'self' is a class, object_getClass() will get the meta-class,
-   *	and get_imp() will get the class method.
+   *	and class_getMethodImplementation() will get the class method.
    */
-  return get_imp(object_getClass(self), aSelector);
+  return class_getMethodImplementation(object_getClass(self), aSelector);
 }
 
 /**
@@ -1453,9 +1463,10 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
 
   mth = GSGetMethod(self, aSelector, YES, YES);
-  if (mth == 0)
+  if (0 == mth)
     return nil;
-  return [NSMethodSignature signatureWithObjCTypes:mth->method_types];
+  return [NSMethodSignature
+    signatureWithObjCTypes: method_getTypeEncoding(mth)];
 }
 
 /**
@@ -1465,26 +1476,28 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  */
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
 {
-  const char		*types;
-  struct objc_method	*mth;
-  Class			c;
+  const char	*types = NULL;
+  Class		c;
+  unsigned int	count;
+  Protocol	**protocols;
 
   if (0 == aSelector)
     {
       return nil;
     }
 
-  c = (GSObjCIsInstance(self) ? object_getClass(self) : (Class)self);
-  mth = GSGetMethod(c, aSelector, GSObjCIsInstance(self), YES);
+  c = object_getClass(self);
 
-  if (mth == 0)
+  /* Do a fast lookup to see if the method is implemented at all.  If it isn't,
+   * we can give up without doing a very expensive linear search through every
+   * method list in the class hierarchy.
+   */
+  if (!class_respondsToSelector(c, aSelector))
     {
       return nil; // Method not implemented
     }
-  types = mth->method_types;
 
-  /*
-   * If there are protocols that this class conforms to,
+  /* If there are protocols that this class conforms to,
    * the method may be listed in a protocol with more
    * detailed type information than in the class itself
    * and we must therefore use the information from the
@@ -1493,37 +1506,50 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
    * used by the Distributed Objects system, which the
    * runtime does not maintain in classes.
    */
-  if (c->protocols != 0)
+  protocols = class_copyProtocolList(c, &count);
+  if (NULL != protocols)
     {
-      struct objc_protocol_list	*protocols = c->protocols;
-      BOOL			found = NO;
+      struct objc_method_description mth;
+      int i;
 
-      while (found == NO && protocols != 0)
-	{
-	  NSUInteger	i = 0;
+      for (i = 0 ; i < count ; i++)
+        {
+          mth = GSProtocolGetMethodDescriptionRecursive(protocols[i],
+	    aSelector, YES, YES);
+          if (NULL == mth.types)
+            {
+              // Search for class method
+              mth = GSProtocolGetMethodDescriptionRecursive(protocols[i],
+		aSelector, YES, NO);
+              // FIXME: We should probably search optional methods here too.
+            }
 
-	  while (found == NO && i < protocols->count)
+          if (NULL != mth.types)
 	    {
-	      Protocol				*p;
-	      struct objc_method_description	*pmth;
-
-	      p = protocols->list[i++];
-	      if (c == (Class)self)
-		{
-		  pmth = [p descriptionForClassMethod: aSelector];
-		}
-	      else
-		{
-		  pmth = [p descriptionForInstanceMethod: aSelector];
-		}
-	      if (pmth != 0)
-		{
-		  types = pmth->types;
-		  found = YES;
-		}
+	      break;
 	    }
-	  protocols = protocols->next;
+        }
+      free(protocols);
+    }
+
+  if (types == 0)
+    {
+#ifdef __GNUSTEP_RUNTIME__
+      struct objc_slot	*objc_get_slot(Class cls, SEL selector);
+      struct objc_slot	*slot = objc_get_slot(object_getClass(self), aSelector);
+      types = slot->types;
+#else
+      struct objc_method *mth;
+      if (GSObjCIsInstance(self))
+	{
+	  mth = GSGetMethod(object_getClass(self), aSelector, YES, YES);
 	}
+      else
+	{
+	  mth = GSGetMethod((Class)self, aSelector, NO, YES);
+	}
+      types = method_getTypeEncoding (mth);
+#endif
     }
 
   if (types == 0)
@@ -1558,11 +1584,8 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  */
 + (void) poseAsClass: (Class)aClassObject
 {
-  class_pose_as(self, aClassObject);
-  /*
-   *	We may have replaced a class in the cache, or may have replaced one
-   *	which had cached methods, so we must rebuild the cache.
-   */
+  [NSException raise: NSInternalInconsistencyException
+              format: @"Class posing is not supported"];
 }
 
 /**
@@ -1578,20 +1601,6 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 	       aSelector ? sel_getName(aSelector) : "(null)"];
 }
 
-- (retval_t) forward: (SEL)aSel : (arglist_t)argFrame
-{
-  NSInvocation *inv;
-
-  if (aSel == 0)
-    [NSException raise: NSInvalidArgumentException
-		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
-
-  inv = AUTORELEASE([[NSInvocation alloc] initWithArgframe: argFrame
-						  selector: aSel]);
-  [self forwardInvocation: inv];
-  return [inv returnFrame: argFrame];
-}
-
 /**
  * This method is called automatically to handle a message sent to
  * the receiver for which the receivers class has no method.<br />
@@ -1783,7 +1792,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  */
 - (BOOL) isMemberOfClass: (Class)aClass
 {
-  return (object_getClass(self) == aClass) ? YES : NO;
+  return ([self class] == aClass) ? YES : NO;
 }
 
 /**
@@ -1819,7 +1828,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
     [NSException raise: NSInvalidArgumentException
 		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
 
-  msg = get_imp(object_getClass(self), aSelector);
+  msg = class_getMethodImplementation(object_getClass(self), aSelector);
   if (!msg)
     {
       [NSException raise: NSGenericException
@@ -1844,7 +1853,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
     [NSException raise: NSInvalidArgumentException
 		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
 
-  msg = get_imp(object_getClass(self), aSelector);
+  msg = class_getMethodImplementation(object_getClass(self), aSelector);
   if (!msg)
     {
       [NSException raise: NSGenericException
@@ -1872,7 +1881,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
     [NSException raise: NSInvalidArgumentException
 		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
 
-  msg = get_imp(object_getClass(self), aSelector);
+  msg = class_getMethodImplementation(object_getClass(self), aSelector);
   if (!msg)
     {
       [NSException raise: NSGenericException
@@ -1936,7 +1945,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
       return NO;
     }
 
-  return __objc_responds_to(self, aSelector);
+  return class_respondsToSelector(object_getClass(self), aSelector) ? YES : NO;
 }
 
 /**
@@ -2050,7 +2059,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
     [NSException raise: NSInvalidArgumentException
 	        format: @"%s +setVersion: may not set a negative version",
 			GSClassNameFromObject(self)];
-  class_set_version(self, aVersion);
+  class_setVersion(self, aVersion);
   return self;
 }
 
@@ -2061,7 +2070,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  */
 + (NSInteger) version
 {
-  return class_get_version(self);
+  return class_getVersion(self);
 }
 
 - (id) autoContentAccessingProxy
@@ -2088,21 +2097,19 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  */
 - (id) error: (const char *)aString, ...
 {
-#if !defined(NeXT_RUNTIME) && !defined(__GNUSTEP_RUNTIME__)
 #define FMT "error: %s (%s)\n%s\n"
   char fmt[(strlen((char*)FMT)+strlen((char*)GSClassNameFromObject(self))
             +((aString!=NULL)?strlen((char*)aString):0)+8)];
   va_list ap;
 
-  sprintf(fmt, FMT, GSClassNameFromObject(self),
-                    GSObjCIsInstance(self)?"instance":"class",
-                    (aString!=NULL)?aString:"");
+  snprintf(fmt, sizeof(fmt), FMT, GSClassNameFromObject(self),
+    GSObjCIsInstance(self) ? "instance" : "class",
+    (aString != NULL) ? aString : "");
   va_start(ap, aString);
-  /* xxx What should `code' argument be?  Current 0. */
-  objc_verror (self, 0, fmt, ap);
+  vfprintf (stderr, fmt, ap);
+  abort ();
   va_end(ap);
 #undef FMT
-#endif
   return nil;
 }
 
@@ -2143,15 +2150,6 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
   return [self conformsToProtocol: aProtocol];
 }
 
-- (retval_t) performv: (SEL)aSel :(arglist_t)argFrame
-{
-  if (aSel == 0)
-    [NSException raise: NSInvalidArgumentException
-		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
-
-  return objc_msg_sendv(self, aSel, argFrame);
-}
-
 + (IMP) instanceMethodFor: (SEL)aSel
 {
   return [self instanceMethodForSelector:aSel];
@@ -2166,9 +2164,10 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
 
   mth = GSGetMethod(self, aSelector, YES, YES);
-  if (mth == 0)
+  if (0 == mth)
     return nil;
-  return [NSMethodSignature signatureWithObjCTypes:mth->method_types];
+  return [NSMethodSignature
+    signatureWithObjCTypes: method_getTypeEncoding(mth)];
 }
 
 - (IMP) methodFor: (SEL)aSel
@@ -2241,12 +2240,12 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  * the -description method and discards the locale
  * information.
  */
-- (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
+- (NSString*) descriptionWithLocale: (id)aLocale
 {
   return [self description];
 }
 
-+ (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
++ (NSString*) descriptionWithLocale: (id)aLocale
 {
   return [self description];
 }
@@ -2256,13 +2255,13 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
  * the -descriptionWithLocale: method and discards the
  * level information.
  */
-- (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
+- (NSString*) descriptionWithLocale: (id)aLocale
 			     indent: (NSUInteger)level
 {
   return [self descriptionWithLocale: aLocale];
 }
 
-+ (NSString*) descriptionWithLocale: (NSDictionary*)aLocale
++ (NSString*) descriptionWithLocale: (id)aLocale
 			     indent: (NSUInteger)level
 {
   return [self descriptionWithLocale: aLocale];
@@ -2316,7 +2315,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 + (NSInteger) streamVersion: (void*)aStream
 {
   GSOnceMLog(@"[NSObject+streamVersion:] is deprecated ... do not use");
-  return class_get_version (self);
+  return class_getVersion (self);
 }
 - (id) read: (void*)aStream
 {
@@ -2341,21 +2340,12 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 @implementation	NSZombie
 - (Class) class
 {
-  return (Class)isa;
+  return object_getClass(self);
 }
 - (Class) originalClass
 {
   return NSMapGet(zombieMap, (void*)self);
 }
-- (retval_t) forward:(SEL)aSel :(arglist_t)argFrame
-{
-  if (aSel == 0)
-    [NSException raise: NSInvalidArgumentException
-		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
-
-  GSLogZombie(self, aSel);
-  return 0;
-}
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
   NSUInteger	size = [[anInvocation methodSignature] methodReturnLength];
@@ -2415,3 +2405,4 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
   return [object methodSignatureForSelector: aSelector];
 }
 @end
+
