@@ -1,5 +1,5 @@
 /* Implementation of filesystem & path-related functions for GNUstep
-   Copyright (C) 1996-2004 Free Software Foundation, Inc.
+   Copyright (C) 1996-2010 Free Software Foundation, Inc.
 
    Written by:  Andrew Kachites McCallum <address@hidden>
    Created: May 1996
@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPathUtilities function reference</title>
-   $Date: 2010-08-21 09:03:32 +0100 (Sat, 21 Aug 2010) $ $Revision: 31191 $
+   $Date: 2011-04-13 15:34:39 +0100 (Wed, 13 Apr 2011) $ $Revision: 32841 $
    */
 
 /**
@@ -218,6 +218,8 @@ static NSString *gnustepUserDirDocumentation = nil;
 static NSString *gnustepUserDirDocumentationInfo = nil;
 static NSString *gnustepUserDirDocumentationMan = nil;
 
+static NSString	*uninstalled = nil;
+
 static BOOL ParseConfigurationFile(NSString *name, NSMutableDictionary *dict,
   NSString *userName);
 
@@ -663,6 +665,81 @@ static void ExtractValuesFromConfig(NSDictionary *config)
   ASSIGN_DEFAULT_PATH(gnustepLocalUsersDir, @GNUSTEP_TARGET_LOCAL_USERS_DIR);
 }
 
+static void
+addDefaults(NSString *defs, NSMutableDictionary *conf)
+{
+  if ([MGR() isReadableFileAtPath: defs] == YES)
+    {
+      NSDictionary      *d;
+      NSDictionary      *attributes;
+
+      attributes = [MGR() fileAttributesAtPath: defs
+				  traverseLink: YES];
+      if (([attributes filePosixPermissions]
+	& (0022 & ATTRMASK)) != 0)
+	{
+#if defined(__MINGW__)
+	  fprintf(stderr,
+	    "The file '%S' is writable by someone other than"
+	    " its owner (permissions 0%lo).\nIgnoring it.\n",
+	    [defs fileSystemRepresentation],
+	    (long)[attributes filePosixPermissions]);
+#else
+	  fprintf(stderr,
+	    "The file '%s' is writable by someone other than"
+	    " its owner (permissions 0%lo).\nIgnoring it.\n",
+	    [defs fileSystemRepresentation],
+	    (long)[attributes filePosixPermissions]);
+#endif
+	  d = nil;
+	}
+      else
+	{
+	  d = [NSDictionary dictionaryWithContentsOfFile: defs];
+	}
+
+      if (d != nil)
+	{
+	  NSEnumerator  *enumerator;
+	  NSString      *key;
+	  id            extra;
+
+	  extra = [conf objectForKey: @"GNUSTEP_EXTRA"];
+	  if ([extra isKindOfClass: [NSString class]])
+	    {
+	      extra = [extra componentsSeparatedByString: @","];
+	    }
+	  extra = [extra mutableCopy];
+	  if (extra == nil)
+	    {
+	      extra = [NSMutableArray new];
+	    }
+	  enumerator = [d keyEnumerator];
+	  while ((key = [enumerator nextObject]) != nil)
+	    {
+	      if ([conf objectForKey: key] == nil)
+		{
+		  [extra addObject: key];
+		}
+	      else
+		{
+		  fprintf(stderr, "Value for key '%s' in '%s' replaces"
+		    " earlier setting.\n", [key UTF8String], [defs UTF8String]);
+		}
+	    }
+	  [conf addEntriesFromDictionary: d];
+	  if ([extra count] > 0)
+	    {
+	      NSArray   *c = [extra copy];
+
+	      [conf setObject: c forKey: @"GNUSTEP_EXTRA"];
+	      RELEASE(c);
+	    }
+	  RELEASE(extra);
+	}
+    }
+}
+
 NSMutableDictionary*
 GNUstepConfig(NSDictionary *newConfig)
 {
@@ -763,82 +840,39 @@ GNUstepConfig(NSDictionary *newConfig)
 		}
 	      else
 		{
+		  NSEnumerator	*e;
                   NSString      *defs;
+                  NSString      *path;
 
 		  gnustepConfigPath
 		    = RETAIN([file stringByDeletingLastPathComponent]);
 		  ParseConfigurationFile(file, conf, nil);
 
+		  if (nil != [conf objectForKey: @"GNUSTEP_EXTRA"])
+		    {
+		      NSLog(@"Warning: use of GNUSTEP_EXTRA in your GNUstep.conf file is deprecated.  Please use a GlobalDefaults.plist instead.\n");
+		    }
+		  /* Merge in any values from property lists in the 
+		   * GlobalDefaults directory.
+		   */
+                  path = [gnustepConfigPath stringByAppendingPathComponent:
+                    @"GlobalDefaults"];
+		  e = [[MGR() directoryContentsAtPath: path] objectEnumerator];
+		  while ((defs = [e nextObject]) != nil)
+		    {
+		      if ([[defs pathExtension] isEqualToString: @"plist"])
+			{
+			  defs = [path stringByAppendingPathComponent: defs];
+			  addDefaults(defs, conf);
+			}
+		    }
+
+		  /* And merge in value from GloablDefaults.plist
+		   */
                   defs = [gnustepConfigPath stringByAppendingPathComponent:
                     @"GlobalDefaults.plist"];
-                  if ([MGR() isReadableFileAtPath: defs] == YES)
-                    {
-                      NSDictionary      *d;
-                      NSDictionary      *attributes;
 
-                      attributes = [MGR() fileAttributesAtPath: defs
-                                                  traverseLink: YES];
-                      if (([attributes filePosixPermissions]
-                        & (0022 & ATTRMASK)) != 0)
-                        {
-#if defined(__MINGW__)
-                          fprintf(stderr,
-                            "The file '%S' is writable by someone other than"
-                            " its owner (permissions 0%lo).\nIgnoring it.\n",
-                            [defs fileSystemRepresentation],
-                            (long)[attributes filePosixPermissions]);
-#else
-                          fprintf(stderr,
-                            "The file '%s' is writable by someone other than"
-                            " its owner (permissions 0%lo).\nIgnoring it.\n",
-                            [defs fileSystemRepresentation],
-                            (long)[attributes filePosixPermissions]);
-#endif
-                          d = nil;
-                        }
-                      else
-                        {
-                          d = [NSDictionary dictionaryWithContentsOfFile: defs];
-                        }
-
-                      if (d != nil)
-                        {
-                          NSEnumerator  *enumerator;
-                          NSString      *key;
-                          id            extra;
-
-                          extra = [conf objectForKey: @"GNUSTEP_EXTRA"];
-                          extra = [extra componentsSeparatedByString: @","];
-                          extra = [extra mutableCopy];
-                          if (extra == nil)
-                            {
-                              extra = [NSMutableArray new];
-                            }
-                          enumerator = [d keyEnumerator];
-                          while ((key = [enumerator nextObject]) != nil)
-                            {
-                              if ([conf objectForKey: key] == nil)
-                                {
-                                  [extra addObject: key];
-                                }
-                              else
-                                {
-                                  fprintf(stderr, "Key '%s' in '%s' duplicates"
-                                    " key in %s\n", [key UTF8String],
-                                    [defs UTF8String], [file UTF8String]);
-                                }
-                            }
-                          [conf addEntriesFromDictionary: d];
-                          if ([extra count] > 0)
-                            {
-                              NSArray   *c = [extra copy];
-
-                              [conf setObject: c forKey: @"GNUSTEP_EXTRA"];
-                              RELEASE(c);
-                            }
-                          RELEASE(extra);
-                        }
-                    }
+		  addDefaults(defs, conf);
 		}
 	    }
 	  else
@@ -941,6 +975,8 @@ static void InitialisePathUtilities(void)
       NSMutableDictionary	*config;
 
       [gnustep_global_lock lock];
+      ASSIGNCOPY(uninstalled, [[[NSProcessInfo processInfo] environment]
+	objectForKey: @"GNUSTEP_UNINSTALLED_LIBRARY_DIRECTORY"]);
       gnustepUserName = [NSUserName() copy];
 #if defined(__MINGW__)
       {
@@ -1368,8 +1404,8 @@ ParseConfigurationFile(NSString *fileName, NSMutableDictionary *dict,
   if (wantVal == YES)
     {
       [dict setObject: @"" forKey: key];
-      DESTROY(key);
     }
+  DESTROY(key);
   NSZoneFree(NSDefaultMallocZone(), src);
   NSZoneFree(NSDefaultMallocZone(), dst);
   return YES;
@@ -1479,7 +1515,8 @@ NSUserName(void)
 
       [gnustep_global_lock lock];
       pwent = getpwuid (uid);
-      strcpy(buf, pwent->pw_name);
+      strncpy(buf, pwent->pw_name, sizeof(buf) - 1);
+      buf[sizeof(buf) - 1] = '\0';
       [gnustep_global_lock unlock];
       loginName = buf;
 #endif /* HAVE_GETPWUID */
@@ -1507,7 +1544,13 @@ NSHomeDirectory(void)
 }
 
 /**
- * Returns loginName's home directory as an NSString object.
+ * Returns loginName's home directory as an NSString object.<br />
+ * On most systems this returns the directory specified in the system's
+ * password file (ar at least whatever is returned by the standard
+ * operating system API for retrieving password file records),
+ * but on ms-windows this is determined by examining the
+ * HOMEPATH and HOMEDRIVE environment variables or (if those don't make
+ * sense) the USERPROFILE environment variable.
  */
 /* NOTE FOR DEVELOPERS.
  * If you change the behavior of this method you must also change
@@ -1595,15 +1638,21 @@ NSFullUserName(void)
 {
   if (theFullUserName == nil)
     {
-      NSString	*userName = nil;
+      NSString	*userName = NSUserName();
 #if defined(__MINGW__)
       struct _USER_INFO_2	*userInfo;
 
-      if (NetUserGetInfo(NULL, (unichar*)[NSUserName() cStringUsingEncoding:
+      if (NetUserGetInfo(NULL, (unichar*)[userName cStringUsingEncoding:
 	NSUnicodeStringEncoding], 2, (LPBYTE*)&userInfo) == 0)
 	{
-	  userName = [NSString stringWithCharacters: userInfo->usri2_full_name
-	    length: wcslen(userInfo->usri2_full_name)];
+	  int	length = wcslen(userInfo->usri2_full_name);
+
+	  if (length > 0)
+	    {
+	      userName = [NSString
+		stringWithCharacters: userInfo->usri2_full_name
+		length: length];
+	    }
 	}
 #else
 #ifdef  HAVE_PWD_H
@@ -1612,27 +1661,28 @@ NSFullUserName(void)
       struct passwd *p;
       char buf[BUFSIZ*10];
 
-      if (getpwnam_r([NSUserName() cString], &pw, buf, sizeof(buf), &p) == 0)
+      if (getpwnam_r([userName cString], &pw, buf, sizeof(buf), &p) == 0)
         {
-          userName = [NSString stringWithUTF8String: pw.pw_gecos];
+	  if (*pw.pw_gecos)
+	    {
+              userName = [NSString stringWithUTF8String: pw.pw_gecos];
+	    }
         }
 #else
 #if     defined(HAVE_GETPWNAM)
       struct passwd	*pw;
 
       [gnustep_global_lock lock];
-      pw = getpwnam([NSUserName() cString]);
-      userName = [NSString stringWithUTF8String: pw->pw_gecos];
+      pw = getpwnam([userName cString]);
+      if (pw->pw_gecos)
+	{
+          userName = [NSString stringWithUTF8String: pw->pw_gecos];
+        }
       [gnustep_global_lock lock];
 #endif /* HAVE_GETPWNAM */
 #endif /* HAVE_GETPWNAM_R */
 #endif /* HAVE_PWD_H */
 #endif /* defined(__Win32__) else */
-      if (userName == nil)
-        {
-          NSLog(@"Warning: NSFullUserName not implemented\n");
-          userName = NSUserName();
-        }
       ASSIGN(theFullUserName, userName);
     }
   return theFullUserName;
@@ -1942,7 +1992,7 @@ if (domainMask & mask) \
 
       case NSDesktopDirectory:
 	{
-	  ADD_PATH(NSUserDomainMask, gnustepUserLibrary, @"Desktop");
+	  ADD_PATH(NSUserDomainMask, gnustepUserHome, @"Desktop");
 	}
 	break;
 
@@ -1986,6 +2036,12 @@ if (domainMask & mask) \
 
       case NSLibraryDirectory:
 	{
+	  /* We allow the environment to be used to specify the location of
+	   * the resources of uninstalled libraries so that we can run tests
+	   * on software prior to installation.
+	   */
+	  ADD_PLATFORM_PATH(NSAllDomainsMask, uninstalled);
+
 	  ADD_PLATFORM_PATH(NSUserDomainMask, gnustepUserLibrary);
 	  ADD_PLATFORM_PATH(NSLocalDomainMask, gnustepLocalLibrary);
 	  ADD_PLATFORM_PATH(NSNetworkDomainMask, gnustepNetworkLibrary);
@@ -2028,19 +2084,18 @@ if (domainMask & mask) \
 
       case NSDocumentDirectory:
 	{
-	  ADD_PATH(NSUserDomainMask, gnustepUserLibrary, @"Document");
-	  ADD_PATH(NSLocalDomainMask, gnustepLocalLibrary, @"Document");
-	  ADD_PATH(NSNetworkDomainMask, gnustepNetworkLibrary, @"Document");
-	  ADD_PATH(NSSystemDomainMask, gnustepSystemLibrary, @"Document");
+	  /* this is relative to the user home annd for the user domain only
+	   * verified on Macintosh 
+	   * despite the name it is Documents and not Document....
+	   */
+	  ADD_PATH(NSUserDomainMask, gnustepUserHome, @"Documents");
 	}
 	break;
 
       case NSDownloadsDirectory:
 	{
-	  ADD_PATH(NSUserDomainMask, gnustepUserLibrary, @"Downloads");
-	  ADD_PATH(NSLocalDomainMask, gnustepLocalLibrary, @"Downloads");
-	  ADD_PATH(NSNetworkDomainMask, gnustepNetworkLibrary, @"Downloads");
-	  ADD_PATH(NSSystemDomainMask, gnustepSystemLibrary, @"Downloads");
+	  /* Be consistent with NSDocumentDirectory */
+	  ADD_PATH(NSUserDomainMask, gnustepUserHome, @"Downloads");
 	}
 	break;
 
