@@ -30,7 +30,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPortCoder class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-29 10:40:14 +0100 (Tue, 29 Mar 2011) $ $Revision: 32733 $
    */
 
 #import "common.h"
@@ -49,18 +49,18 @@
 /*
  *	Setup for inline operation of pointer map tables.
  */
-#define	GSI_MAP_KTYPES	GSUNION_PTR | GSUNION_OBJ | GSUNION_CLS | GSUNION_INT
-#define	GSI_MAP_VTYPES	GSUNION_PTR | GSUNION_OBJ | GSUNION_CLS | GSUNION_INT
+#define	GSI_MAP_KTYPES	GSUNION_PTR | GSUNION_OBJ | GSUNION_CLS | GSUNION_NSINT
+#define	GSI_MAP_VTYPES	GSUNION_PTR | GSUNION_OBJ | GSUNION_CLS | GSUNION_NSINT
 #define	GSI_MAP_RETAIN_KEY(M, X)	
 #define	GSI_MAP_RELEASE_KEY(M, X)	
 #define	GSI_MAP_RETAIN_VAL(M, X)	
 #define	GSI_MAP_RELEASE_VAL(M, X)	
-#define	GSI_MAP_HASH(M, X)	((X).uint)
+#define	GSI_MAP_HASH(M, X)	((X).nsu)
 #define	GSI_MAP_EQUAL(M, X,Y)	((X).ptr == (Y).ptr)
 #define	GSI_MAP_NOCLEAN	1
 
 #if	GS_WITH_GC
-#include	<gc_typed.h>
+#include	<gc/gc_typed.h>
 static GC_descr	nodeDesc;	// Type descriptor for map node.
 #define	GSI_MAP_NODES(M, X) \
 (GSIMapNode)GC_calloc_explicitly_typed(X, sizeof(GSIMapNode_t), nodeDesc)
@@ -71,8 +71,8 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 /*
  *	Setup for inline operation of arrays.
  */
-#define	GSI_ARRAY_RETAIN(A, X)	
-#define	GSI_ARRAY_RELEASE(A, X)	
+#define	GSI_ARRAY_NO_RETAIN	1
+#define	GSI_ARRAY_NO_RELEASE	1
 #define	GSI_ARRAY_TYPES	GSUNION_OBJ|GSUNION_SEL|GSUNION_PTR
 
 #include "GNUstepBase/GSIArray.h"
@@ -132,13 +132,13 @@ typeToName1(char type)
 
 	  if (bufptr == buf1)
 	    {
-		bufptr = buf2;
+	      bufptr = buf2;
 	    }
 	  else
 	    {
 	      bufptr = buf1;
 	    }
-	  sprintf(bufptr, "unknown type info - 0x%x", type);
+	  snprintf(bufptr, 32, "unknown type info - 0x%x", type);
 	  return bufptr;
 	}
     }
@@ -177,13 +177,13 @@ typeToName2(char type)
 
 	  if (bufptr == buf1)
 	    {
-		bufptr = buf2;
+	      bufptr = buf2;
 	    }
 	  else
 	    {
 	      bufptr = buf1;
 	    }
-	  sprintf(bufptr, "unknown type info - 0x%x", type);
+	  snprintf(bufptr, 32, "unknown type info - 0x%x", type);
 	  return bufptr;
 	}
     }
@@ -350,6 +350,14 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 {
   if (self == [NSPortCoder class])
     {
+#if	GS_WITH_GC
+      /* We create a typed memory descriptor for map nodes.
+       */
+      GC_word	w[GC_BITMAP_SIZE(GSIMapNode_t)] = {0};
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, key));
+      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, value));
+      nodeDesc = GC_make_descriptor(w, GC_WORD_LEN(GSIMapNode_t));
+#endif
       connectionClass = [NSConnection class];
       mutableArrayClass = [NSMutableArray class];
       mutableDataClass = [NSMutableDataMalloc class];
@@ -365,14 +373,6 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
       _eTagImp = [mutableDataClass instanceMethodForSelector: eTagSel];
       _xRefImp = [mutableDataClass instanceMethodForSelector: xRefSel];
       mutableDictionaryClass = [NSMutableDictionary class];
-#if	GS_WITH_GC
-      /* We create a typed memory descriptor for map nodes.
-       */
-      GC_word	w[GC_BITMAP_SIZE(GSIMapNode_t)] = {0};
-      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, key));
-      GC_set_bit(w, GC_WORD_OFFSET(GSIMapNode_t, value));
-      nodeDesc = GC_make_descriptor(w, GC_WORD_LEN(GSIMapNode_t));
-#endif
     }
 }
 
@@ -425,7 +425,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 }
 
 - (void) decodeArrayOfObjCType: (const char*)type
-			 count: (unsigned)expected
+			 count: (NSUInteger)expected
 			    at: (void*)buf
 {
   unsigned int	i;
@@ -616,6 +616,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 		      obj = rep;
 		      GSIArraySetItemAtIndex(_objAry, (GSIArrayItem)obj, xref);
 		    }
+		  GS_CONSUMED(rep)
 		}
 	    }
 	  *(id*)address = obj;
@@ -723,6 +724,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	       */
 	      address = &dummy;
 	      (*_dTagImp)(_src, dTagSel, &info, &xref, &_cursor);
+	      GS_CONSUMED(classInfo)
 	    }
 	  if (info != _GSC_NONE)
 	    {
@@ -870,6 +872,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	  else
 	    {
 	      char	*tmp;
+	      int	len;
 
 	      if (xref != GSIArrayCount(_ptrAry))
 		{
@@ -877,9 +880,10 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 			      format: @"extra string crossref - %d", xref];
 		}
 	      (*_dDesImp)(_src, dDesSel, &tmp, @encode(char*), &_cursor, nil);
-	      *(void**)address = GSAutoreleasedBuffer(strlen(tmp)+1);
+	      len = strlen(tmp);
+	      *(void**)address = GSAutoreleasedBuffer(len + 1);
 	      GSIArrayAddItem(_ptrAry, (GSIArrayItem)*(void**)address);
-	      strcpy(*(char**)address, tmp);
+	      memcpy(*(char**)address, tmp, len + 1);
 	      NSZoneFree(NSDefaultMallocZone(), tmp);
 	    }
 	  return;
@@ -1032,7 +1036,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 #else
 	  val = GSSwapBigI128ToHost(val);
 #if	GS_HAVE_I64
-	  bigval = *(uint64_t*)&val;
+	  bigval = *(uint64_t*)(void*)&val;
 #else
 	  bigval = *(uint32_t*)&val;
 #endif
@@ -1089,7 +1093,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 }
 
 - (void) encodeArrayOfObjCType: (const char*)type
-			 count: (unsigned)count
+			 count: (NSUInteger)count
 			    at: (const void*)buf
 {
   unsigned	i;
@@ -1211,7 +1215,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	  return;
 	}
 
-      GSIMapAddPair(_cIdMap, (GSIMapKey)anObject, (GSIMapVal)0);
+      GSIMapAddPair(_cIdMap, (GSIMapKey)anObject, (GSIMapVal)(NSUInteger)0);
     }
   else if (anObject == nil)
     {
@@ -1288,13 +1292,14 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	       *	and add it to the map of unconditionay encoded ones.
 	       */
 	      GSIMapRemoveKey(_cIdMap, (GSIMapKey)anObject);
-	      GSIMapAddPair(_uIdMap, (GSIMapKey)anObject, (GSIMapVal)0);
+	      GSIMapAddPair(_uIdMap,
+		(GSIMapKey)anObject, (GSIMapVal)(NSUInteger)0);
 	      [anObject encodeWithCoder: self];
 	    }
 	  return;
 	}
 
-      if (node == 0 || node->value.uint == 0)
+      if (node == 0 || node->value.nsu == 0)
 	{
 	  Class	cls;
 	  id	obj;
@@ -1302,11 +1307,11 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	  if (node == 0)
 	    {
 	      node = GSIMapAddPair(_uIdMap,
-		(GSIMapKey)anObject, (GSIMapVal)++_xRefO);
+		(GSIMapKey)anObject, (GSIMapVal)(NSUInteger)++_xRefO);
 	    }
 	  else
 	    {
-	      node->value.uint = ++_xRefO;
+	      node->value.nsu = ++_xRefO;
 	    }
 
 	  obj = [anObject replacementObjectForPortCoder: self];
@@ -1316,20 +1321,20 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	       *	If the object we have been given is actually a class,
 	       *	we encode it as a special case.
 	       */
-	      (*_xRefImp)(_dst, xRefSel, _GSC_CID, node->value.uint);
+	      (*_xRefImp)(_dst, xRefSel, _GSC_CID, node->value.nsu);
 	      (*_eValImp)(self, eValSel, @encode(Class), &obj);
 	    }
 	  else
 	    {
 	      cls = [obj classForPortCoder];
-	      (*_xRefImp)(_dst, xRefSel, _GSC_ID, node->value.uint);
+	      (*_xRefImp)(_dst, xRefSel, _GSC_ID, node->value.nsu);
 	      (*_eValImp)(self, eValSel, @encode(Class), &cls);
 	      [obj encodeWithCoder: self];
 	    }
 	}
       else
 	{
-	  (*_xRefImp)(_dst, xRefSel, _GSC_ID | _GSC_XREF, node->value.uint);
+	  (*_xRefImp)(_dst, xRefSel, _GSC_ID | _GSC_XREF, node->value.nsu);
 	}
     }
 }
@@ -1455,13 +1460,13 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 		if (node == 0)
 		  {
 		    GSIMapAddPair(_ptrMap,
-			(GSIMapKey)*(void**)buf, (GSIMapVal)0);
+		      (GSIMapKey)*(void**)buf, (GSIMapVal)(NSUInteger)0);
 		    type++;
 		    buf = *(char**)buf;
 		    (*_eValImp)(self, eValSel, type, buf);
 		  }
 	      }
-	    else if (node == 0 || node->value.uint == 0)
+	    else if (node == 0 || node->value.nsu == 0)
 	      {
 		/*
 		 *	Second pass, unwritten pointer - write it.
@@ -1469,13 +1474,13 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 		if (node == 0)
 		  {
 		    node = GSIMapAddPair(_ptrMap,
-			(GSIMapKey)*(void**)buf, (GSIMapVal)++_xRefP);
+		      (GSIMapKey)*(void**)buf, (GSIMapVal)(NSUInteger)++_xRefP);
 		  }
 		else
 		  {
-		    node->value.uint = ++_xRefP;
+		    node->value.nsu = ++_xRefP;
 		  }
-		(*_xRefImp)(_dst, xRefSel, _GSC_PTR, node->value.uint);
+		(*_xRefImp)(_dst, xRefSel, _GSC_PTR, node->value.nsu);
 		type++;
 		buf = *(char**)buf;
 		(*_eValImp)(self, eValSel, type, buf);
@@ -1486,7 +1491,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 		 *	Second pass, write a cross-reference number.
 		 */
 		(*_xRefImp)(_dst, xRefSel, _GSC_PTR|_GSC_XREF,
-		  node->value.uint);
+		  node->value.nsu);
 	      }
 	  }
 	return;
@@ -1520,7 +1525,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	    if (node != 0)
 	      {
 		(*_xRefImp)(_dst, xRefSel, _GSC_CLASS | _GSC_XREF,
-		  node->value.uint);
+		  node->value.nsu);
 		return;
 	      }
 	    while (done == NO)
@@ -1535,11 +1540,11 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 				format: @"negative class version"];
 		  }
 		node = GSIMapAddPair(_clsMap,
-			(GSIMapKey)(void*)c, (GSIMapVal)++_xRefC);
+		  (GSIMapKey)(void*)c, (GSIMapVal)(NSUInteger)++_xRefC);
 		/*
 		 *	Encode tag and crossref number.
 		 */
-		(*_xRefImp)(_dst, xRefSel, _GSC_CLASS, node->value.uint);
+		(*_xRefImp)(_dst, xRefSel, _GSC_CLASS, node->value.nsu);
 		/*
 		 *	Encode class, and version.
 		 */
@@ -1585,8 +1590,8 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	    if (node == 0)
 	      {
 		node = GSIMapAddPair(_ptrMap,
-			(GSIMapKey)(void*)s, (GSIMapVal)++_xRefP);
-		(*_xRefImp)(_dst, xRefSel, _GSC_SEL, node->value.uint);
+		  (GSIMapKey)(void*)s, (GSIMapVal)(NSUInteger)++_xRefP);
+		(*_xRefImp)(_dst, xRefSel, _GSC_SEL, node->value.nsu);
 		/*
 		 *	Encode selector.
 		 */
@@ -1595,7 +1600,7 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	    else
 	      {
 		(*_xRefImp)(_dst, xRefSel, _GSC_SEL|_GSC_XREF,
-		  node->value.uint);
+		  node->value.nsu);
 	      }
 	  }
 	return;
@@ -1616,14 +1621,14 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
 	    if (node == 0)
 	      {
 		node = GSIMapAddPair(_ptrMap,
-			(GSIMapKey)*(char**)buf, (GSIMapVal)++_xRefP);
-		(*_xRefImp)(_dst, xRefSel, _GSC_CHARPTR, node->value.uint);
+		  (GSIMapKey)*(char**)buf, (GSIMapVal)(NSUInteger)++_xRefP);
+		(*_xRefImp)(_dst, xRefSel, _GSC_CHARPTR, node->value.nsu);
 		(*_eSerImp)(_dst, eSerSel, buf, type, nil);
 	      }
 	    else
 	      {
 		(*_xRefImp)(_dst, xRefSel, _GSC_CHARPTR|_GSC_XREF,
-		  node->value.uint);
+		  node->value.nsu);
 	      }
 	  }
 	return;
@@ -1995,7 +2000,8 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
   char		header[headerLength+1];
   unsigned	dataLength = [_dst length];
 
-  sprintf(header, "%s%08x:%08x:%08x:%08x:", PREFIX, v, cc, oc, pc);
+  snprintf(header, sizeof(header), "%s%08x:%08x:%08x:%08x:",
+    PREFIX, v, cc, oc, pc);
 
   if (locationInData + headerLength <= dataLength)
     {
@@ -2042,10 +2048,10 @@ static IMP	_xRefImp;	/* Serialize a crossref.	*/
     {
       proxyClass = [NSDistantObject class];
       /*
-       * use get_imp() because NSDistantObject doesn't implement
-       * methodForSelector:
+       * Use class_getMethodImplementation() because NSDistantObject
+       * doesn't implement methodForSelector:
        */
-      proxyImp = get_imp(object_getClass((id)proxyClass),
+      proxyImp = class_getMethodImplementation(object_getClass((id)proxyClass),
 	@selector(proxyWithLocal:connection:));
     }
 
