@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProcessInfo class reference</title>
-   $Date: 2010-06-10 11:13:51 +0100 (Thu, 10 Jun 2010) $ $Revision: 30649 $
+   $Date: 2011-03-17 22:33:31 +0000 (Thu, 17 Mar 2011) $ $Revision: 32638 $
 */
 
 /*************************************************************************
@@ -101,9 +101,12 @@
 #undef id
 #endif
 
+#if defined(__APPLE__) && !GS_FAKE_MAIN
+#include <crt_externs.h>
+#endif
+
 #import "GNUstepBase/GSConfig.h"
 #import "Foundation/NSArray.h"
-#import "Foundation/NSBundle.h"
 #import "Foundation/NSSet.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSDate.h"
@@ -226,19 +229,22 @@ static NSMutableSet	*mySet = nil;
 static void
 _gnu_process_args(int argc, char *argv[], char *env[])
 {
-  CREATE_AUTORELEASE_POOL(arp);
+  NSAutoreleasePool *arp = [NSAutoreleasePool new];
   NSString	*arg0 = nil;
   int i;
 
   if (_gnu_arg_zero != 0)
     {
-      objc_free(_gnu_arg_zero);
+      free(_gnu_arg_zero);
     }
 
   if (argv != 0 && argv[0] != 0)
     {
-      _gnu_arg_zero = (char*)objc_malloc(strlen(argv[0]) + 1);
-      strcpy(_gnu_arg_zero, argv[0]);
+      int	len;
+
+      len = strlen(argv[0]) + 1;
+      _gnu_arg_zero = (char*)malloc(len);
+      memcpy(_gnu_arg_zero, argv[0], len);
       arg0 = [[NSString alloc] initWithCString: _gnu_arg_zero];
     }
   else
@@ -247,12 +253,13 @@ _gnu_process_args(int argc, char *argv[], char *env[])
       unichar	*buffer;
       int	buffer_size = 0;
       int	needed_size = 0;
+      int	len;
       const char	*tmp;
 
       while (needed_size == buffer_size)
 	{
           buffer_size = buffer_size + 256;
-          buffer = (unichar*)objc_malloc(buffer_size * sizeof(unichar));
+          buffer = (unichar*)malloc(buffer_size * sizeof(unichar));
           needed_size = GetModuleFileNameW(NULL, buffer, buffer_size);
           if (needed_size < buffer_size)
 	    {
@@ -269,12 +276,13 @@ _gnu_process_args(int argc, char *argv[], char *env[])
 	    }
           else
 	    {
-              objc_free(buffer);
+              free(buffer);
 	    }
 	}
       tmp = [arg0 cStringUsingEncoding: [NSString defaultCStringEncoding]];
-      _gnu_arg_zero = (char*)objc_malloc(strlen(tmp) + 1);
-      strcpy(_gnu_arg_zero, tmp);
+      len = strlen(tmp) + 1;
+      _gnu_arg_zero = (char*)malloc(len);
+      memcpy(_gnu_arg_zero, tmp, len);
 #else
       fprintf(stderr, "Error: for some reason, argv not properly set up "
 	      "during GNUstep base initialization\n");
@@ -424,7 +432,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
 	      {
 		char	buf[len+2];
 
-		strcpy(buf, env[i]);
+		memcpy(buf, env[i], len + 1);
 		cp = &buf[cp - env[i]];
 		*cp++ = '\0';
 		[keys addObject:
@@ -444,7 +452,7 @@ _gnu_process_args(int argc, char *argv[], char *env[])
   IF_NO_GC(RELEASE(arp));
 }
 
-#if !GS_FAKE_MAIN && ((defined(HAVE_PROCFS)  || defined(HAVE_KVM_ENV) || defined(HAVE_PROCFS_PSINFO)) && (defined(HAVE_LOAD_METHOD)))
+#if !GS_FAKE_MAIN && ((defined(HAVE_PROCFS)  || defined(HAVE_KVM_ENV) || defined(HAVE_PROCFS_PSINFO) || defined(__APPLE__)) && (defined(HAVE_LOAD_METHOD)))
 /*
  * We have to save program arguments and environment before main () is
  * executed, because main () could modify their values before we get a
@@ -512,7 +520,7 @@ static char	**_gnu_noobjc_env = NULL;
   /* copy the environment strings */
   for (count = 0; vectors[count]; count++)
     ;
-  _gnu_noobjc_env = (char**)objc_malloc(sizeof(char*) * (count + 1));
+  _gnu_noobjc_env = (char**)malloc(sizeof(char*) * (count + 1));
   if (!_gnu_noobjc_env)
     goto malloc_error;
   for (i = 0; i < count; i++)
@@ -540,7 +548,7 @@ static char	**_gnu_noobjc_env = NULL;
   for (_gnu_noobjc_argc = 0; vectors[_gnu_noobjc_argc]; _gnu_noobjc_argc++)
     ;
   _gnu_noobjc_argv
-    = (char**)objc_malloc(sizeof(char*) * (_gnu_noobjc_argc + 1));
+    = (char**)malloc(sizeof(char*) * (_gnu_noobjc_argc + 1));
   if (!_gnu_noobjc_argv)
     goto malloc_error;
   for (i = 0; i < _gnu_noobjc_argc; i++)
@@ -560,8 +568,8 @@ static char	**_gnu_noobjc_env = NULL;
   int i, count;
   
   // Read commandline
-  proc_file_name = (char*)objc_malloc(sizeof(char) * 2048);
-  sprintf(proc_file_name, "/proc/%d/psinfo", (int)getpid());
+  proc_file_name = (char*)malloc(2048);
+  snprintf(proc_file_name, 2048, "/proc/%d/psinfo", (int)getpid());
   
   ifp = fopen(proc_file_name, "r");
   if (ifp == NULL)
@@ -586,7 +594,7 @@ static char	**_gnu_noobjc_env = NULL;
   /* copy the environment strings */
   for (count = 0; vectors[count]; count++)
     ;
-  _gnu_noobjc_env = (char**)objc_malloc(sizeof(char*) * (count + 1));
+  _gnu_noobjc_env = (char**)malloc(sizeof(char*) * (count + 1));
   if (!_gnu_noobjc_env)
     goto malloc_error;
   for (i = 0; i < count; i++)
@@ -608,7 +616,7 @@ static char	**_gnu_noobjc_env = NULL;
   for (_gnu_noobjc_argc = 0; vectors[_gnu_noobjc_argc]; _gnu_noobjc_argc++)
     ;
   _gnu_noobjc_argv
-    = (char**)objc_malloc(sizeof(char*) * (_gnu_noobjc_argc + 1));
+    = (char**)malloc(sizeof(char*) * (_gnu_noobjc_argc + 1));
   if (!_gnu_noobjc_argv)
     goto malloc_error;
   for (i = 0; i < _gnu_noobjc_argc; i++)
@@ -619,6 +627,45 @@ static char	**_gnu_noobjc_env = NULL;
     }
   _gnu_noobjc_argv[i] = NULL;
 
+  return;
+#elif defined(__APPLE__)
+  /*
+   * Darwin/Mac OS X provides indirect access to command line arguments and
+   * the environment with functions defined in the C runtime system.
+   */
+  int i, n;
+  int argc = *_NSGetArgc();
+  char **argv = *_NSGetArgv();
+  char **environ = *_NSGetEnviron();
+
+  /* copy environment */
+  n = 0;
+  while (environ[n] != NULL)
+    n++;
+  _gnu_noobjc_env = (char **)malloc(sizeof(char *) * (n + 1));
+  if (_gnu_noobjc_env == NULL)
+    goto malloc_error;
+  for (i = 0; i < n; i++)
+    {
+      _gnu_noobjc_env[i] = (char *)strdup(environ[i]);
+      if (_gnu_noobjc_env[i] == NULL)
+	goto malloc_error;
+    }
+  _gnu_noobjc_env[i] = NULL;
+
+  /* copy arguments */
+  _gnu_noobjc_argc = argc;
+  _gnu_noobjc_argv = (char **)malloc(sizeof(char *) * (argc + 1));
+  if (_gnu_noobjc_argv == NULL)
+    goto malloc_error;
+  for (i = 0; i < argc; i++)
+    {
+      _gnu_noobjc_argv[i] = (char *)strdup(argv[i]);
+      if (_gnu_noobjc_argv[i] == NULL)
+	goto malloc_error;
+    }
+  _gnu_noobjc_argv[i] = NULL;
+
   return;
 #else /* !HAVE_KVM_ENV (i.e. HAVE_PROCFS).  */
   /*
@@ -657,7 +704,7 @@ static char	**_gnu_noobjc_env = NULL;
   while (environ[c] != NULL)
     c++;
   env_terms = c;
-  _gnu_noobjc_env = (char**)objc_malloc(sizeof(char*) * (env_terms + 1));
+  _gnu_noobjc_env = (char**)malloc(sizeof(char*) * (env_terms + 1));
   if (_gnu_noobjc_env == NULL)
     goto malloc_error;
   for (c = 0; c < env_terms; c++)
@@ -669,8 +716,8 @@ static char	**_gnu_noobjc_env = NULL;
   _gnu_noobjc_env[c] = NULL;
 
   // Read commandline
-  proc_file_name = (char *)objc_malloc(sizeof(char) * 2048);
-  sprintf(proc_file_name, "/proc/%d/cmdline", (int)getpid());
+  proc_file_name = (char *)malloc(2048);
+  snprintf(proc_file_name, 2048, "/proc/%d/cmdline", (int)getpid());
 
   /*
    * We read the /proc file thrice.
@@ -702,8 +749,7 @@ static char	**_gnu_noobjc_env = NULL;
    * Now _gnu_noobcj_argc is the number of arguments;
    * allocate memory accordingly.
    */
-  _gnu_noobjc_argv
-    = (char **)objc_malloc((sizeof(char *)) * (_gnu_noobjc_argc + 1));
+  _gnu_noobjc_argv = (char **)malloc((sizeof(char *)) * (_gnu_noobjc_argc + 1));
   if (_gnu_noobjc_argv == NULL)
     goto malloc_error;
 
@@ -712,7 +758,7 @@ static char	**_gnu_noobjc_env = NULL;
   //freopen(proc_file_name, "r", ifp);
   if (ifp == NULL)
     {
-      objc_free(_gnu_noobjc_argv);
+      free(_gnu_noobjc_argv);
       goto proc_fs_error;
     }
   argument = 0;
@@ -724,7 +770,7 @@ static char	**_gnu_noobjc_env = NULL;
       if ((c == EOF) || (c == 0)) // End of a parameter
 	{
 	  _gnu_noobjc_argv[argument]
-	    = (char*)objc_malloc((sizeof(char))*length);
+	    = (char*)malloc((sizeof(char))*length);
 	  if (_gnu_noobjc_argv[argument] == NULL)
 	    goto malloc_error;
 	  argument++;
@@ -739,8 +785,8 @@ static char	**_gnu_noobjc_env = NULL;
   if (ifp == NULL)
     {
       for (c = 0; c < _gnu_noobjc_argc; c++)
-	objc_free(_gnu_noobjc_argv[c]);
-      objc_free(_gnu_noobjc_argv);
+	free(_gnu_noobjc_argv[c]);
+      free(_gnu_noobjc_argv);
       goto proc_fs_error;
     }
   argument = 0;
@@ -772,7 +818,7 @@ static char	**_gnu_noobjc_env = NULL;
     }
   _gnu_noobjc_argv[argument] = NULL;
   fclose(ifp);
-  objc_free(proc_file_name);
+  free(proc_file_name);
   return;
 
  proc_fs_error:
@@ -792,7 +838,7 @@ static char	**_gnu_noobjc_env = NULL;
 #ifdef HAVE_PROGRAM_INVOCATION_NAME
   fprintf(stderr, "We try to go on anyway; but the program will ignore any argument which were passed to it.\n");
   _gnu_noobjc_argc = 1;
-  _gnu_noobjc_argv = objc_malloc(sizeof(char *) * 2);
+  _gnu_noobjc_argv = malloc(sizeof(char *) * 2);
   if (_gnu_noobjc_argv == NULL)
     goto malloc_error;
   _gnu_noobjc_argv[0] = strdup(program_invocation_name);
@@ -810,7 +856,7 @@ static char	**_gnu_noobjc_env = NULL;
 #endif /* HAVE_PROGRAM_INVOCATION_NAME */
 #endif /* !HAVE_KVM_ENV (e.g. HAVE_PROCFS) */
  malloc_error:
-  fprintf(stderr, "objc_malloc() error when starting gnustep-base.\n");
+  fprintf(stderr, "malloc() error when starting gnustep-base.\n");
   fprintf(stderr, "Free some memory and then re-run the program.\n");
   abort();
 }
@@ -823,19 +869,19 @@ _gnu_noobjc_free_vars(void)
   p = _gnu_noobjc_argv;
   while (*p)
     {
-      objc_free(*p);
+      free(*p);
       p++;
     }
-  objc_free(_gnu_noobjc_argv);
+  free(_gnu_noobjc_argv);
   _gnu_noobjc_argv = 0;
 
   p = _gnu_noobjc_env;
   while (*p)
     {
-      objc_free(*p);
+      free(*p);
       p++;
     }
-  objc_free(_gnu_noobjc_env);
+  free(_gnu_noobjc_env);
   _gnu_noobjc_env = 0;
 }
 
@@ -1013,10 +1059,8 @@ int main(int argc, char *argv[], char *env[])
   [gnustep_global_lock lock];
   if (host == nil)
     {
-      extern NSTimeInterval GSTimeNow(void);
-
       pid = [self processIdentifier];
-      start = (unsigned long)GSTimeNow();
+      start = (unsigned long)GSPrivateTimeNow();
       host = [[self hostName] stringByReplacingString: @"." withString: @"_"];
       IF_NO_GC(RETAIN(host);)
     }
