@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSScanner class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-09-09 22:41:40 +0100 (Thu, 09 Sep 2010) $ $Revision: 31278 $
 */
 
 #import "common.h"
@@ -1146,7 +1146,7 @@ typedef GSString	*ivars;
 {
   NSScanner	*n = [[self class] allocWithZone: zone];
 
-  [n initWithString: _string];
+  n = [n initWithString: _string];
   [n setCharactersToBeSkipped: _charactersToBeSkipped];
   [n setLocale: _locale];
   [n setScanLocation: _scanLocation];
