@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTimeZone class reference</title>
-   $Date: 2010-06-16 11:47:12 +0100 (Wed, 16 Jun 2010) $ $Revision: 30766 $
+   $Date: 2011-04-04 21:54:26 +0100 (Mon, 04 Apr 2011) $ $Revision: 32778 $
  */
 
 /* Use the system time zones if available. In other cases, use an
@@ -92,13 +92,13 @@
 #import "Foundation/NSLock.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSUserDefaults.h"
-#import "Foundation/NSBundle.h"
 #import "Foundation/NSMapTable.h"
 #import "Foundation/NSThread.h"
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSPortCoder.h"
 #import "Foundation/NSTimeZone.h"
 #import "Foundation/NSByteOrder.h"
+#import "Foundation/NSLocale.h"
 #import "GNUstepBase/GSConfig.h"
 #import "GNUstepBase/NSObject+GNUstepBase.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
@@ -111,6 +111,10 @@
 #include "nstzfile.h"
 #endif
 
+#if defined(HAVE_UNICODE_UCAL_H)
+#include <unicode/ucal.h>
+#endif
+
 NSString * const NSSystemTimeZoneDidChangeNotification
   = @"NSSystemTimeZoneDidChangeNotification";
 
@@ -152,6 +156,51 @@ NSString * const NSSystemTimeZoneDidChangeNotification
 #define POSIX_TZONES     @"posix/"
 #endif
 
+#define BUFFER_SIZE 512
+
+#if GS_USE_ICU == 1
+static inline int
+_NSToICUTZDisplayStyle(NSTimeZoneNameStyle style)
+{
+  switch (style)
+    {
+      case NSTimeZoneNameStyleStandard:
+        return UCAL_STANDARD;
+      case NSTimeZoneNameStyleShortStandard:
+        return UCAL_SHORT_STANDARD;
+      case NSTimeZoneNameStyleDaylightSaving:
+        return UCAL_DST;
+      case NSTimeZoneNameStyleShortDaylightSaving:
+        return UCAL_SHORT_DST;
+      default:
+        return -1;
+    }
+}
+
+static inline UCalendar *
+ICUCalendarSetup (NSTimeZone *tz, NSLocale *locale)
+{
+  NSString *tzStr;
+  int32_t tzLen;
+  const char *cLocale;
+  UChar tzName[BUFFER_SIZE];
+  UCalendar *cal;
+  UErrorCode err = U_ZERO_ERROR;
+  
+  tzStr = [tz name];
+  if ((tzLen = [tzStr length]) > BUFFER_SIZE)
+    tzLen = BUFFER_SIZE;
+  [tzStr getCharacters: tzName range: NSMakeRange(0, tzLen)];
+  cLocale = [[locale localeIdentifier] UTF8String];
+  
+  cal = ucal_open(tzName, tzLen, cLocale, UCAL_TRADITIONAL, &err);
+  if (U_FAILURE(err))
+    return NULL;
+  
+  return cal;
+}
+#endif
+
 /* Possible location of system time zone files */
 static NSString *tzdir = nil;
 
@@ -259,7 +308,7 @@ decode (const void *ptr)
   result = (result << 8) | *p++;
   result = (result << 8) | *p++;
   result = (result << 8) | *p++;
-  result = (result << 8) | *p++;
+  result = (result << 8) | *p;
   return result;
 #endif /* defined(WORDS_BIGENDIAN) && SIZEOF_INT == 4 */
 }
@@ -692,7 +741,7 @@ static NSMapTable	*absolutes = 0;
 	      int	m = i % 60;
 	      char	buf[9];
 
-	      sprintf(buf, "GMT%c%02d%02d", s, h, m);
+	      snprintf(buf, sizeof(buf), "GMT%c%02d%02d", s, h, m);
 	      name = [[NSString alloc] initWithUTF8String: buf];
 	    }
 	  else
@@ -944,7 +993,7 @@ static NSMapTable	*absolutes = 0;
     }
   if (abbreviationDictionary == nil)
     {
-      CREATE_AUTORELEASE_POOL(pool);
+      NSAutoreleasePool	*pool = [NSAutoreleasePool new];
       NSString		*path;
 
       path = _time_zone_path (ABBREV_DICT, @"plist");
@@ -989,7 +1038,7 @@ static NSMapTable	*absolutes = 0;
 	  [md makeImmutableCopyOnFail: NO];
 	  abbreviationDictionary = md;
 	}
-      RELEASE(pool);
+      [pool release];
     }
   if (zone_mutex != nil)
     {
@@ -1018,7 +1067,7 @@ static NSMapTable	*absolutes = 0;
     }
   if (abbreviationMap == nil)
     {
-      CREATE_AUTORELEASE_POOL(pool);
+      NSAutoreleasePool		*pool = [NSAutoreleasePool new];
       NSMutableDictionary	*md;
       NSMutableArray		*ma;
       NSString			*the_name;
@@ -1135,7 +1184,7 @@ static NSMapTable	*absolutes = 0;
 
       [md makeImmutableCopyOnFail: NO];
       abbreviationMap = RETAIN(md); 
-      RELEASE(pool);
+      [pool release];
     }
   if (zone_mutex != nil)
     {
@@ -1668,9 +1717,10 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
     }
   if (regionsArray == nil)
     {
-      CREATE_AUTORELEASE_POOL(pool);
-      int index, i;
-      char name[80];
+      NSAutoreleasePool	*pool = [NSAutoreleasePool new];
+      int		index;
+      int		i;
+      char		name[80];
       FILE		*fp;
       NSMutableArray	*temp_array[24];
       NSString		*path;
@@ -1790,7 +1840,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
 	    }
 	}
       regionsArray = [[NSArray alloc] initWithObjects: temp_array count: 24];
-      RELEASE(pool);
+      [pool release];
     }
   if (zone_mutex != nil)
     {
@@ -2071,7 +2121,25 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
 
 - (NSTimeInterval) daylightSavingTimeOffsetForDate: (NSDate *)aDate
 {
+#if GS_USE_ICU == 1
+  NSTimeInterval result;
+  UCalendar *cal;
+  UErrorCode err = U_ZERO_ERROR;
+  
+  cal = ICUCalendarSetup (self, nil);
+  if (cal == NULL)
+    return 0.0;
+  
+  ucal_setMillis (cal, ([aDate timeIntervalSince1970] * 1000.0), &err);
+  result = (double)ucal_get (cal, UCAL_DST_OFFSET, &err) / 1000.0;
+  if (U_FAILURE(err))
+    result = 0.0;
+  ucal_close (cal);
+  
+  return result;
+#else
   return 0.0;   // FIXME
+#endif
 }
 
 - (NSDate *) nextDaylightSavingTimeTransitionAfterDate: (NSDate *)aDate
@@ -2081,18 +2149,44 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
 
 - (NSTimeInterval) daylightSavingTimeOffset
 {
-  return 0.0;   // FIXME
+  return [self daylightSavingTimeOffsetForDate: [NSDate date]];
 }
 
 - (NSDate *) nextDaylightSavingTimeTransition
 {
-  return nil;   // FIXME;
+  return [self nextDaylightSavingTimeTransitionAfterDate: [NSDate date]];
 }
 
 - (NSString *)localizedName: (NSTimeZoneNameStyle)style
                      locale: (NSLocale *)locale
 {
+#if GS_USE_ICU == 1
+  UChar *result;
+  const char *cLocale;
+  int32_t len;
+  UCalendar *cal;
+  UErrorCode err = U_ZERO_ERROR;
+  
+  cal = ICUCalendarSetup (self, locale);
+  if (cal == NULL)
+    return nil;
+  
+  cLocale = [[locale localeIdentifier] UTF8String];
+  result = NSZoneMalloc ([self zone], BUFFER_SIZE * sizeof(UChar));
+  len = ucal_getTimeZoneDisplayName (cal, _NSToICUTZDisplayStyle(style),
+    cLocale, result, BUFFER_SIZE, &err);
+  if (len > BUFFER_SIZE)
+    {
+      result = NSZoneRealloc ([self zone], result, len * sizeof(UChar));
+      ucal_getTimeZoneDisplayName (cal, _NSToICUTZDisplayStyle(style),
+        cLocale, result, len, &err);
+    }
+  
+  return AUTORELEASE([[NSString alloc] initWithCharactersNoCopy: result
+    length: len freeWhenDone: YES]);
+#else
   return nil;   // FIXME;
+#endif
 }
                     
 @end
@@ -2291,8 +2385,10 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
 
 - (id) initWithName: (NSString*)name data: (NSData*)data
 {
-  HKEY     regDirKey;
-  BOOL     isNT = NO, regFound=NO;
+  HKEY	regDirKey;
+  BOOL	isNT = NO;
+  BOOL	regFound=NO;
+  BOOL	tzFound = NO;
 
   /* Open the key in the local machine hive where
    * the time zone data is stored. */
@@ -2302,8 +2398,8 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
     KEY_READ,
     &regDirKey))
     {
-      isNT=YES;
-      regFound=YES;
+      isNT = YES;
+      regFound = YES;
     }
   else
     {
@@ -2313,7 +2409,7 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
           KEY_READ,
           &regDirKey))
         {
-          regFound=YES;
+          regFound = YES;
         }
     }
 
@@ -2326,7 +2422,7 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
       DWORD    cbName;                   // size of name string 
       wchar_t  achClass[MAX_PATH] = L""; // buffer for class name 
       DWORD    cchClassName = MAX_PATH;  // size of class string 
-      DWORD    cSubKeys=0;               // number of subkeys 
+      DWORD    cSubKeys = 0;             // number of subkeys 
       DWORD    cbMaxSubKey;              // longest subkey size 
       DWORD    cchMaxClass;              // longest class string 
       DWORD    cValues;                  // number of values for key 
@@ -2335,7 +2431,6 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
       DWORD    cbSecurityDescriptor;     // size of security descriptor 
       FILETIME ftLastWriteTime;          // last write time 
       DWORD     i, retCode;
-      BOOL	tzFound = NO;
 		
       /* Get the class name and the value count. */
       retCode = RegQueryInfoKeyW(
@@ -2478,7 +2573,10 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
         }
       RegCloseKey(regDirKey);
     }
-    
+  if (NO == tzFound)
+    {
+      DESTROY(self);
+    }
   return self;
 }
 
@@ -2801,9 +2899,9 @@ newDetailInZoneForType(GSTimeZone *zone, TypeInfo *type)
 		      format: @"TZ_MAGIC is incorrect"];
 	}
 #endif
-      n_trans = GSSwapBigI32ToHost(*(int32_t*)header->tzh_timecnt);
-      n_types = GSSwapBigI32ToHost(*(int32_t*)header->tzh_typecnt);
-      charcnt = GSSwapBigI32ToHost(*(int32_t*)header->tzh_charcnt);
+      n_trans = GSSwapBigI32ToHost(*(int32_t*)(void*)header->tzh_timecnt);
+      n_types = GSSwapBigI32ToHost(*(int32_t*)(void*)header->tzh_typecnt);
+      charcnt = GSSwapBigI32ToHost(*(int32_t*)(void*)header->tzh_charcnt);
 
       i = pos;
       i += sizeof(int32_t)*n_trans;
