@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSURL class reference</title>
-   $Date: 2010-07-03 07:08:28 +0100 (Sat, 03 Jul 2010) $ $Revision: 30914 $
+   $Date: 2011-03-07 11:34:17 +0000 (Mon, 07 Mar 2011) $ $Revision: 32483 $
 */
 
 /*
@@ -40,6 +40,7 @@ function may be incorrect
 #define	EXPOSE_NSURL_IVARS	1
 #import "Foundation/NSArray.h"
 #import "Foundation/NSCoder.h"
+#import "Foundation/NSData.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSFileManager.h"
@@ -54,6 +55,73 @@ function may be incorrect
 NSString * const NSURLErrorDomain = @"NSURLErrorDomain";
 NSString * const NSErrorFailingURLStringKey = @"NSErrorFailingURLStringKey";
 
+@interface	NSString (NSURLPrivate)
+- (NSString*) _stringByAddingPercentEscapes;
+@end
+
+@implementation	NSString (NSURLPrivate)
+/* Like the normal percent escape method, but with additional characters
+ * escaped.
+ */
+- (NSString*) _stringByAddingPercentEscapes
+{
+  NSData	*data = [self dataUsingEncoding: NSUTF8StringEncoding];
+  NSString	*s = nil;
+
+  if (data != nil)
+    {
+      unsigned char	*src = (unsigned char*)[data bytes];
+      unsigned int	slen = [data length];
+      unsigned char	*dst;
+      unsigned int	spos = 0;
+      unsigned int	dpos = 0;
+
+      dst = (unsigned char*)NSZoneMalloc(NSDefaultMallocZone(), slen * 3);
+      while (spos < slen)
+	{
+	  unsigned char	c = src[spos++];
+	  unsigned int	hi;
+	  unsigned int	lo;
+
+	  if (c <= 32
+	    || c > 126
+	    || c == 34
+	    || c == 35
+	    || c == 37
+	    || c == 59
+	    || c == 60
+	    || c == 62
+	    || c == 63
+	    || c == 91
+	    || c == 92
+	    || c == 93
+	    || c == 94
+	    || c == 96
+	    || c == 123
+	    || c == 124
+	    || c == 125)
+	    {
+	      dst[dpos++] = '%';
+	      hi = (c & 0xf0) >> 4;
+	      dst[dpos++] = (hi > 9) ? 'A' + hi - 10 : '0' + hi;
+	      lo = (c & 0x0f);
+	      dst[dpos++] = (lo > 9) ? 'A' + lo - 10 : '0' + lo;
+	    }
+	  else
+	    {
+	      dst[dpos++] = c;
+	    }
+	}
+      s = [[NSString alloc] initWithBytes: dst
+				   length: dpos
+				 encoding: NSASCIIStringEncoding];
+      NSZoneFree(NSDefaultMallocZone(), dst);
+      IF_NO_GC([s autorelease];)
+    }
+  return s;
+}
+@end
+
 /*
  * Structure describing a URL.
  * All the char* fields may be NULL pointers, except path, which
@@ -94,9 +162,11 @@ static char *unescape(const char *from, char * to);
  */
 static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 {
+  const char	*rpath;
   char		*buf;
   char		*ptr;
   char		*tmp;
+  int		l;
   unsigned int	len = 1;
 
   if (rel->scheme != 0)
@@ -121,8 +191,13 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
     }
   if (rel->path != 0)
     {
-      len += strlen(rel->path) + 1;	// path
+      rpath = rel->path;
     }
+  else
+    {
+      rpath = "";
+    }
+  len += strlen(rpath) + 1;	// path
   if (base != 0 && base->path != 0)
     {
       len += strlen(base->path) + 1;	// path
@@ -148,8 +223,9 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 
   if (rel->scheme != 0)
     {
-      strcpy(ptr, rel->scheme);
-      ptr = &ptr[strlen(ptr)];
+      l = strlen(rel->scheme);
+      memcpy(ptr, rel->scheme, l);
+      ptr += l;
       *ptr++ = ':';
     }
   if (rel->isGeneric == YES
@@ -161,14 +237,16 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 	{
 	  if (rel->user != 0)
 	    {
-	      strcpy(ptr, rel->user);
-	      ptr = &ptr[strlen(ptr)];
+	      l = strlen(rel->user);
+	      memcpy(ptr, rel->user, l);
+	      ptr += l;
 	    }
 	  if (rel->password != 0)
 	    {
 	      *ptr++ = ':';
-	      strcpy(ptr, rel->password);
-	      ptr = &ptr[strlen(ptr)];
+	      l = strlen(rel->password);
+	      memcpy(ptr, rel->password, l);
+	      ptr += l;
 	    }
 	  if (rel->host != 0 || rel->port != 0)
 	    {
@@ -177,14 +255,16 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 	}
       if (rel->host != 0)
 	{
-	  strcpy(ptr, rel->host);
-	  ptr = &ptr[strlen(ptr)];
+	  l = strlen(rel->host);
+	  memcpy(ptr, rel->host, l);
+	  ptr += l;
 	}
       if (rel->port != 0)
 	{
 	  *ptr++ = ':';
-	  strcpy(ptr, rel->port);
-	  ptr = &ptr[strlen(ptr)];
+	  l = strlen(rel->port);
+	  memcpy(ptr, rel->port, l);
+	  ptr += l;
 	}
     }
 
@@ -199,19 +279,25 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 	{
 	  *tmp++ = '/';
 	}
-      strcpy(tmp, rel->path);
+      l = strlen(rpath);
+      memcpy(tmp, rpath, l);
+      tmp += l;
     }
   else if (base == 0)
     {
-      strcpy(tmp, rel->path);
+      l = strlen(rpath);
+      memcpy(tmp, rpath, l);
+      tmp += l;
     }
-  else if (rel->path[0] == 0)
+  else if (rpath[0] == 0)
     {
       if (base->hasNoPath == NO)
 	{
 	  *tmp++ = '/';
 	}
-      strcpy(tmp, base->path);
+      l = strlen(base->path);
+      memcpy(tmp, base->path, l);
+      tmp += l;
     }
   else
     {
@@ -221,12 +307,15 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
       if (end != 0)
 	{
 	  *tmp++ = '/';
-	  strncpy(tmp, start, end - start);
+	  memcpy(tmp, start, end - start);
 	  tmp += (end - start);
 	}
       *tmp++ = '/';
-      strcpy(tmp, rel->path);
+      l = strlen(rpath);
+      memcpy(tmp, rpath, l);
+      tmp += l;
     }
+  *tmp = '\0';
 
   if (standardize == YES)
     {
@@ -248,7 +337,8 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 		}
 	      else
 		{
-		  strcpy(tmp, &tmp[2]);
+		  l = strlen(&tmp[2]) + 1;
+		  memmove(tmp, &tmp[2], l);
 		}
 	    }
 	  else
@@ -264,7 +354,8 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 	{
 	  if (tmp[0] == '/' && tmp[1] == '/')
 	    {
-	      strcpy(tmp, &tmp[1]);
+	      l = strlen(&tmp[1]) + 1;
+	      memmove(tmp, &tmp[1], l);
 	    }
 	  else
 	    {
@@ -296,7 +387,8 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
 	    }
 	  else
 	    {
-	      strcpy(tmp, next);
+	      l = strlen(next) + 1;
+	      memmove(tmp, next, l);
 	    }
 	}
       /*
@@ -305,7 +397,7 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
       tmp = ptr;
       if (*tmp == '\0')
 	{
-	  strcpy(tmp, "/");
+	  memcpy(tmp, "/", 2);
 	}
     }
   ptr = &ptr[strlen(ptr)];
@@ -313,22 +405,25 @@ static char *buildURL(parsedURL *base, parsedURL *rel, BOOL standardize)
   if (rel->parameters != 0)
     {
       *ptr++ = ';';
-      strcpy(ptr, rel->parameters);
-      ptr = &ptr[strlen(ptr)];
+      l = strlen(rel->parameters);
+      memcpy(ptr, rel->parameters, l);
+      ptr += l;
     }
   if (rel->query != 0)
     {
       *ptr++ = '?';
-      strcpy(ptr, rel->query);
-      ptr = &ptr[strlen(ptr)];
+      l = strlen(rel->query);
+      memcpy(ptr, rel->query, l);
+      ptr += l;
     }
   if (rel->fragment != 0)
     {
       *ptr++ = '#';
-      strcpy(ptr, rel->fragment);
-      ptr = &ptr[strlen(ptr)];
+      l = strlen(rel->fragment);
+      memcpy(ptr, rel->fragment, l);
+      ptr += l;
     }
-
+  *ptr = '\0';
   return buf;
 }
 
@@ -542,8 +637,7 @@ static unsigned	urlAlign;
 {
   NSString	*aUrlString = [NSString alloc];
 
-  aPath
-    = [aPath stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
+  aPath = [aPath _stringByAddingPercentEscapes];
   if ([aHost length] > 0)
     {
       NSRange	r = [aHost rangeOfString: @"@"];
@@ -670,8 +764,11 @@ static unsigned	urlAlign;
 {
   /* RFC 2396 'reserved' characters ...
    * as modified by RFC2732
+   * static const char *reserved = ";/?:@&=+$,[]";
    */
-  static const char *reserved = ";/?:@&=+$,[]";
+  /* Same as reserved set but allow the hash character in a path too.
+   */
+  static const char *filepath = ";/?:@&=+$,[]#";
 
   if ([aUrlString isKindOfClass: [NSString class]] == NO)
     {
@@ -765,9 +862,6 @@ static unsigned	urlAlign;
         {
           if (strcmp(buf->scheme, "file") == 0)
 	    {
-	      usesFragments = NO;
-	      usesParameters = NO;
-	      usesQueries = NO;
 	      buf->isFile = YES;
 	    }
 	  else if (strcmp(buf->scheme, "mailto") == 0)
@@ -990,15 +1084,15 @@ static unsigned	urlAlign;
 	      if (buf->fragment == 0 && base != 0)
 		{
 		  buf->fragment = base->fragment;
-		  if (legal(buf->fragment, reserved) == NO)
-		    {
-		      [NSException raise: NSInvalidArgumentException
-                        format: @"[%@ %@](%@, %@) "
-		        @"illegal character in fragment part",
-                        NSStringFromClass([self class]),
-                        NSStringFromSelector(_cmd),
-                        aUrlString, aBaseUrl];
-		    }
+		}
+	      if (legal(buf->fragment, filepath) == NO)
+		{
+		  [NSException raise: NSInvalidArgumentException
+		    format: @"[%@ %@](%@, %@) "
+		    @"illegal character in fragment part",
+		    NSStringFromClass([self class]),
+		    NSStringFromSelector(_cmd),
+		    aUrlString, aBaseUrl];
 		}
 	    }
 
@@ -1019,15 +1113,15 @@ static unsigned	urlAlign;
 	      if (buf->query == 0 && base != 0)
 		{
 		  buf->query = base->query;
-		  if (legal(buf->query, reserved) == NO)
-		    {
-		      [NSException raise: NSInvalidArgumentException
-                        format: @"[%@ %@](%@, %@) "
-		        @"illegal character in query part",
-                        NSStringFromClass([self class]),
-                        NSStringFromSelector(_cmd),
-                        aUrlString, aBaseUrl];
-		    }
+		}
+	      if (legal(buf->query, filepath) == NO)
+		{
+		  [NSException raise: NSInvalidArgumentException
+		    format: @"[%@ %@](%@, %@) "
+		    @"illegal character in query part",
+		    NSStringFromClass([self class]),
+		    NSStringFromSelector(_cmd),
+		    aUrlString, aBaseUrl];
 		}
 	    }
 
@@ -1048,15 +1142,15 @@ static unsigned	urlAlign;
 	      if (buf->parameters == 0 && base != 0)
 		{
 		  buf->parameters = base->parameters;
-		  if (legal(buf->parameters, reserved) == NO)
-		    {
-		      [NSException raise: NSInvalidArgumentException
-                        format: @"[%@ %@](%@, %@) "
-		        @"illegal character in parameters part",
-                        NSStringFromClass([self class]),
-                        NSStringFromSelector(_cmd),
-                        aUrlString, aBaseUrl];
-		    }
+		}
+	      if (legal(buf->parameters, filepath) == NO)
+		{
+		  [NSException raise: NSInvalidArgumentException
+		    format: @"[%@ %@](%@, %@) "
+		    @"illegal character in parameters part",
+		    NSStringFromClass([self class]),
+		    NSStringFromSelector(_cmd),
+		    aUrlString, aBaseUrl];
 		}
 	    }
 
@@ -1064,7 +1158,14 @@ static unsigned	urlAlign;
 	    {
 	      buf->user = 0;
 	      buf->password = 0;
-	      buf->host = (char*) "localhost";
+	      if (base != 0 && base->host != 0)
+		{
+		  buf->host = base->host;
+		}
+	      else if (buf->host != 0 && *buf->host == 0)
+		{
+		  buf->host = 0;
+		}
 	      buf->port = 0;
 	      buf->isGeneric = YES;
 	    }
@@ -1086,7 +1187,7 @@ static unsigned	urlAlign;
 	{
 	  buf->hasNoPath = YES;
 	}
-      if (legal(buf->path, reserved) == NO)
+      if (legal(buf->path, filepath) == NO)
 	{
 	  [NSException raise: NSInvalidArgumentException
             format: @"[%@ %@](%@, %@) "
@@ -1127,8 +1228,8 @@ static unsigned	urlAlign;
 {
   if (NSShouldRetainWithZone(self, zone) == NO)
     {
-      return [[isa allocWithZone: zone] initWithString: _urlString
-					 relativeToURL: _baseURL];
+      return [[[self class] allocWithZone: zone] initWithString: _urlString
+						  relativeToURL: _baseURL];
     }
   else
     {
@@ -1224,7 +1325,7 @@ static unsigned	urlAlign;
  */
 - (NSURL*) absoluteURL
 {
-  if (_baseURL != nil)
+  if (_baseURL == nil)
     {
       return self;
     }
@@ -1262,8 +1363,9 @@ static unsigned	urlAlign;
 
 - (char*) _path: (char*)buf
 {
-  char		*ptr = buf;
-  char		*tmp = buf;
+  char	*ptr = buf;
+  char	*tmp = buf;
+  int	l;
 
   if (myData->pathIsAbsolute == YES)
     {
@@ -1273,14 +1375,16 @@ static unsigned	urlAlign;
 	}
       if (myData->path != 0)
 	{
-          strcpy(tmp, myData->path);
+	  l = strlen(myData->path);
+          memcpy(tmp, myData->path, l + 1);
 	}
     }
   else if (_baseURL == nil)
     {
       if (myData->path != 0)
 	{
-          strcpy(tmp, myData->path);
+	  l = strlen(myData->path);
+          memcpy(tmp, myData->path, l + 1);
 	}
     }
   else if (*myData->path == 0)
@@ -1291,7 +1395,8 @@ static unsigned	urlAlign;
 	}
       if (baseData->path != 0)
 	{
-          strcpy(tmp, baseData->path);
+	  l = strlen(baseData->path);
+          memcpy(tmp, baseData->path, l + 1);
 	}
     }
   else
@@ -1308,7 +1413,8 @@ static unsigned	urlAlign;
       *tmp++ = '/';
       if (myData->path != 0)
 	{
-          strcpy(tmp, myData->path);
+	  l = strlen(myData->path);
+          memcpy(tmp, myData->path, l + 1);
 	}
     }
 
@@ -1632,13 +1738,20 @@ static unsigned	urlAlign;
  */
 - (NSString*) relativePath
 {
-  NSString	*path = nil;
-
-  if (myData->path != 0)
+  if (nil == _baseURL)
     {
-      path = [NSString stringWithUTF8String: myData->path];
+      return [self path];
+    }
+  else
+    {
+      NSString	*path = nil;
+
+      if (myData->path != 0)
+	{
+	  path = [NSString stringWithUTF8String: myData->path];
+	}
+      return path;
     }
-  return path;
 }
 
 /**
@@ -1688,7 +1801,21 @@ static unsigned	urlAlign;
 
   if (range.length > 0)
     {
-      return [_urlString substringFromIndex: range.location + 1];
+      NSString *specifier;
+
+      /* MacOSX compatibility - in the case where there is no
+       * host in the URL, just return the path (without the "//").
+       * For all other cases we return the whole specifier.
+       */
+      if (nil == [self host])
+	{
+          specifier = [_urlString substringFromIndex: NSMaxRange(range)];
+	}
+      else
+	{
+          specifier = [_urlString substringFromIndex: range.location+1];
+	}
+      return specifier;
     }
   else
     {
