@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSUnarchiver class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2011-03-17 12:53:48 +0000 (Thu, 17 Mar 2011) $ $Revision: 32630 $
    */
 
 #import "common.h"
@@ -35,8 +35,8 @@
 /*
  *	Setup for inline operation of arrays.
  */
-#define	GSI_ARRAY_RETAIN(A, X)	
-#define	GSI_ARRAY_RELEASE(A, X)	
+#define	GSI_ARRAY_NO_RETAIN	1
+#define	GSI_ARRAY_NO_RELEASE	1
 #define	GSI_ARRAY_TYPES	GSUNION_OBJ|GSUNION_SEL|GSUNION_PTR
 
 #include "GNUstepBase/GSIArray.h"
@@ -86,13 +86,13 @@ typeToName1(char type)
 
 	  if (bufptr == buf1)
 	    {
-		bufptr = buf2;
+	      bufptr = buf2;
 	    }
 	  else
 	    {
 	      bufptr = buf1;
 	    }
-	  sprintf(bufptr, "unknown type info - 0x%x", type);
+	  snprintf(bufptr, 32, "unknown type info - 0x%x", type);
 	  return bufptr;
 	}
     }
@@ -131,13 +131,13 @@ typeToName2(char type)
 
 	  if (bufptr == buf1)
 	    {
-		bufptr = buf2;
+	      bufptr = buf2;
 	    }
 	  else
 	    {
 	      bufptr = buf1;
 	    }
-	  sprintf(bufptr, "unknown type info - 0x%x", type);
+	  snprintf(bufptr, 32, "unknown type info - 0x%x", type);
 	  return bufptr;
 	}
     }
@@ -521,7 +521,7 @@ static Class NSDataMallocClass;
 }
 
 - (void) decodeArrayOfObjCType: (const char*)type
-			 count: (unsigned)expected
+			 count: (NSUInteger)expected
 			    at: (void*)buf
 {
   unsigned int	i;
@@ -753,7 +753,7 @@ static Class NSDataMallocClass;
 		  className = @"_NSUnarchiverUnknownClass";
 		}
 	      classInfo = [objDict objectForKey: className];
-	      if (classInfo == nil)
+	      if (nil == classInfo)
 		{
 		  classInfo = [NSUnarchiverObjectInfo newWithName: className];
 		  c = NSClassFromString(className);
@@ -770,10 +770,6 @@ static Class NSDataMallocClass;
 		  [objDict setObject: classInfo forKey: className];
 		  RELEASE(classInfo);
 		}
-	      else
-		{
-		  c = classInfo->class;
-		}
 	      RELEASE(className);
 	      classInfo->version = (NSInteger)cver;
 	      GSIArrayAddItem(clsMap, (GSIArrayItem)((id)classInfo));
@@ -1196,7 +1192,6 @@ static Class NSDataMallocClass;
               {
                 NSLog(@"Loss of information converting decoded value to uint16_t");
               }
-            *(uint8_t*)address = (uint8_t)big;
             *(uint16_t*)address = (uint16_t)big;
             return;
           case 4:
