@@ -21,25 +21,25 @@
    Boston, MA 02111 USA.
 
    */
-#include "config.h"
+#include "common.h"
 #include "GNUstepBase/preface.h"
 
-#include <Foundation/NSData.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSEnumerator.h>
-#include <Foundation/NSRunLoop.h>
-#include <Foundation/NSException.h>
-#include <Foundation/NSError.h>
-#include <Foundation/NSValue.h>
-#include <Foundation/NSHost.h>
-#include <Foundation/NSProcessInfo.h>
-#include <Foundation/NSDebug.h>
-#include <Foundation/NSByteOrder.h>
+#import "Foundation/NSData.h"
+#import "Foundation/NSArray.h"
+#import "Foundation/NSDictionary.h"
+#import "Foundation/NSEnumerator.h"
+#import "Foundation/NSRunLoop.h"
+#import "Foundation/NSException.h"
+#import "Foundation/NSError.h"
+#import "Foundation/NSValue.h"
+#import "Foundation/NSHost.h"
+#import "Foundation/NSProcessInfo.h"
+#import "Foundation/NSByteOrder.h"
+#import "GNUstepBase/NSObject+GNUstepBase.h"
 
-#include "../GSPrivate.h"
-#include "../GSStream.h"
-#include "../GSSocketStream.h"
+#import "../GSPrivate.h"
+#import "../GSStream.h"
+#import "../GSSocketStream.h"
 
 #define	BUFFERSIZE	(BUFSIZ*64)
 
@@ -1016,7 +1016,7 @@
     fileSystemRepresentation];
 
   saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
-  saAttr.bInheritHandle = TRUE;
+  saAttr.bInheritHandle = FALSE;
   saAttr.lpSecurityDescriptor = NULL;
 
   handle = CreateFileW(name,
@@ -1067,7 +1067,7 @@ done:
   int rc;
 
   saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
-  saAttr.bInheritHandle = TRUE;
+  saAttr.bInheritHandle = FALSE;
   saAttr.lpSecurityDescriptor = NULL;
 
   /*
@@ -1217,13 +1217,13 @@ done:
 
 - (id) initWithData: (NSData *)data
 {
-  RELEASE(self);
+  DESTROY(self);
   return [[GSDataInputStream alloc] initWithData: data];
 }
 
 - (id) initWithFileAtPath: (NSString *)path
 {
-  RELEASE(self);
+  DESTROY(self);
   return [[GSFileInputStream alloc] initWithFileAtPath: path];
 }
 
@@ -1262,20 +1262,20 @@ done:
 
 - (id) initToBuffer: (uint8_t *)buffer capacity: (NSUInteger)capacity
 {
-  RELEASE(self);
+  DESTROY(self);
   return [[GSBufferOutputStream alloc] initToBuffer: buffer capacity: capacity];
 }
 
 - (id) initToFileAtPath: (NSString *)path append: (BOOL)shouldAppend
 {
-  RELEASE(self);
+  DESTROY(self);
   return [[GSFileOutputStream alloc] initToFileAtPath: path
 					       append: shouldAppend];  
 }
 
 - (id) initToMemory
 {
-  RELEASE(self);
+  DESTROY(self);
   return [[GSDataOutputStream alloc] init];
 }
 
@@ -1346,7 +1346,7 @@ done:
   NSAssert(handle == INVALID_HANDLE_VALUE, NSInternalInconsistencyException);
 
   saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
-  saAttr.bInheritHandle = TRUE;
+  saAttr.bInheritHandle = FALSE;
   saAttr.lpSecurityDescriptor = NULL;
 
   handle = CreateNamedPipeW([path fileSystemRepresentation],
